@@ -26,65 +26,83 @@
 ########################################################################### */
 
 Importer.loadQtBinding( "qt.core" );
-Importer.loadQtBinding( "qt.gui" ); //for QPixmap
-Importer.loadQtBinding( "qt.network" );
+Importer.loadQtBinding( "qt.gui" );     // for QPixmap
 Importer.loadQtBinding( "qt.xml" );
 
 service_name = "Free Music Charts";
-html = "The rules for the Darkerradio Free Music Charts are quite simple: the best 15 songs from the last month and five new ones are the candidates for the next voting. You have up to five votes.<br/><br/>You can cast your votes at <a href=\"http://www.darkerradio.com/\">www.darkerradio.com</a> (in the right column, scroll down a bit), support for voting from within Amarok might be added later.";
+html = "<div style=\"background-color:#999999\"><center><b>Free Music Charts</b></center></div><div style=\"color:#153860\"><br/>The rules for the Darkerradio.com Free Music Charts are quite simple: the best 15 songs from the last month and five new ones are the candidates for the next voting. Only open music is allowed to take part, since February 2009 a song can stay in the charts for a time of six months max. You have up to five votes.<br/><br/>You can cast your votes by going to the menu bar: <i>Tools -> Free Music Charts Voting</i></div>";
 
-// temporary location for testing purposes, will very likely
-// be moved to darkerradio.com
-xmlUrl = new QUrl( "http://krohlas.de/fmc.xml" );
-http   = new QHttp;
-data   = new QIODevice;
-doc    = new QDomDocument( "doc" );
-elt    = new QDomElement;
-elt2   = new QDomElement;
-shows  = new QDomNodeList;
-script = new FreeMusicCharts();
+votingUrl = new QUrl( "http://www.darkerradio.com/free-music-charts/free-music-charts-voting/" );
+xmlUrl    = new QUrl( "http://krohlas.de/fmc.xml" );
+data      = new QIODevice;
+doc       = new QDomDocument( "doc" );
+elt       = new QDomElement;
+elt2      = new QDomElement;
+shows     = new QDomNodeList;
+script    = new FreeMusicCharts();
+
+currentFilter = "";
+
+Amarok.Window.addToolsSeparator();
+Amarok.Window.addToolsMenu( "votingGui", "Free Music Charts Voting", "amarok" );
 
 script.populate.connect( onPopulate );
 script.customize.connect( onCustomize );
+Amarok.Window.ToolsMenu.votingGui['triggered()'].connect( onVote );
 
 /* Initialization of service */
 function FreeMusicCharts() {
   Amarok.debug( "creating fmc service..." );
-  ScriptableServiceScript.call( this, "Free Music Charts", 2, "Free Music Charts from Darkerradio.com", html, false );
+  ScriptableServiceScript.call( this, "Free Music Charts", 2, "Free Music Charts from Darkerradio.com", html, true );
   Amarok.debug( "done creating fmc service!" );
 }
 
 
+/*###########################################################################
+#   Helper functions for parsing                                          #
+########################################################################### */
+
 /* Get info for shows */
 function fmcShowsXmlParser( reply ) {
   Amarok.debug( "start fmc shows xml parsing..." );
   try {
-    doc.setContent( reply );
-    shows = doc.elementsByTagName( "show" );
-    Amarok.debug ( "got " + shows.length() + " shows!" );
+    if( shows.length() == 0 ) {
+      doc.setContent( reply );
+      shows = doc.elementsByTagName( "show" );
+      Amarok.debug( "got " + shows.length() + " shows!" );
+    }
 
     var showTitles = new Array( shows.length() );
 
-    elt = shows.at( 0 ); // latest show
-    html = html + "<br/><br/>Next radio show airs on Tuesday, ";
+    elt  = shows.at( 0 ); // latest show
+    html = html + "<div style=\"color:#153860\"><br/>The next radio show airs on Tuesday, ";
     html = html + elt.firstChildElement( "nextdate" ).text() + ". ";
-    html = html + "The voting ends the day before.";
+    html = html + "The voting ends the day before.</div>";
 
-    item = Amarok.StreamItem;
-    item.level = 1;
+    item             = Amarok.StreamItem;
+    item.level       = 1;
     item.playableUrl = "";
-    item.infoHtml = html;
-    item.coverUrl = Amarok.Info.scriptPath() + "/FMCShow.png";
+    item.infoHtml    = html;
+    item.coverUrl    = Amarok.Info.scriptPath() + "/FMCShow.png";
+
+    if( shows.length() == 0 ) {
+      Amarok.Window.Statusbar.longMessage( "<b>Free Music Charts</b><br/><br/>Download of charts seems to have <font color=red><b>failed</b></font>. Please check your internet connection." );
+      item.itemName = "Download failed :(";
+      item.callbackData = "failed";
+      script.insertItem( item )
+    }
 
     var i = 0;
     for ( ; i < shows.length(); i++ ) {
-      elt = shows.at( i );
-      elt2 = elt.firstChildElement( "title" );
-      item.itemName = elt2.text();
-      // this is needed to identify the item when we need to expand
-      // it in onPopulate( level, -->callbackData<--, filter )
-      item.callbackData = i;
-      script.insertItem( item );
+      if( containsFilterMatch( i ) ) {
+        elt = shows.at( i );
+        elt2 = elt.firstChildElement( "title" );
+        item.itemName = elt2.text();
+        // this is needed to identify the item when we need to expand
+        // it in onPopulate( level, -->callbackData<--, filter )
+        item.callbackData = i;
+        script.insertItem( item );
+      }
     }
   }
   catch( err ) {
@@ -106,8 +124,8 @@ function fmcTracksXmlParser( url ) {
   for ( ; i < shows.length(); i++ ) {
     Amarok.debug( "found a show..." );
     tempShow = shows.at( i );
-    j = tempShow.firstChildElement( "songcount" ).text();
-    title = tempShow.firstChildElement( "title" ).text();
+    j        = tempShow.firstChildElement( "songcount" ).text();
+    title    = tempShow.firstChildElement( "title" ).text();
     tempShow = tempShow.firstChildElement( "songs" );
     songs    = tempShow.firstChildElement( "song" );
 
@@ -130,6 +148,10 @@ function fmcTracksXmlParser( url ) {
 }
 
 
+/*###########################################################################
+#   Event handlers                                                        #
+########################################################################### */
+
 /* Set service icon and emblem */
 function onCustomize() {
   Amarok.debug ( "customizing fmc service" );
@@ -146,45 +168,57 @@ function onCustomize() {
 /* Fill tree view in Amarok */
 function onPopulate( level, callbackData, filter ) {
   var i = 0;
+
+  filter        = filter.replace( "%20", " " );
+  filter        = filter.trim();
+  currentFilter = filter.toLowerCase();
+
   Amarok.debug( "populating fmc level: " + level );
 
-  if ( level == 1 ) { // the shows
-    Amarok.debug( "fetching fmc xml..." );
-    Amarok.Window.Statusbar.longMessage( "Free Music Charts: Fetching charts. This might take some seconds, depending on the speed of your internet connection..." );
-    try {
-      qurl = new QUrl( xmlUrl );
-      a = new Downloader( qurl, fmcShowsXmlParser );
-    }
-    catch( err ) {
-      Amarok.debug( err );
+  if( level == 1 ) { // the shows
+    if( shows.length() == 0 ) { // we have to fetch the xml
+      try {
+        Amarok.debug( "fetching fmc xml..." );
+        Amarok.Window.Statusbar.longMessage( "<b>Free Music Charts</b><br/><br/>Fetching charts.<br/>This might take some seconds, depending on the speed of your internet connection..." );
+        a = new Downloader( xmlUrl, fmcShowsXmlParser );
+      }
+      catch( err ) {
+        Amarok.debug( err );
+      }
+
+      Amarok.debug( "done fetching fmc xml..." );
     }
 
-  Amarok.debug( "done fetching fmc xml..." );
+    else
+      fmcShowsXmlParser(); // we already have the xml, now we need to filter
   // No script.donePopulating(); here, as the downloader returns
   // immediately, even before the parser is being run.
   // Instead call it at then end of fmcShowsXmlParser(...).
   }
 
-  else if ( level == 0 ) { // the tracks from each show
+  else if( level == 0 ) { // the tracks from each show
     Amarok.debug( "populating fmc track level..." );
+    var addAll = false;
     elt = shows.at( callbackData ); // jump to the correct show
 
-    item = Amarok.StreamItem;
-    item.level = 0;
+    item              = Amarok.StreamItem;
+    item.level        = 0;
     item.callbackData = "";
-    item.album = elt.firstChildElement( "title" ).text();
+    item.album        = elt.firstChildElement( "title" ).text();
+
+    addAll = isFilterMatch( item.album );
 
     /* The podcasts */
     item.infoHtml    = html;
     item.artist      = "Free Music Charts";
     item.itemName    = "Podcast (MP3)";
     item.playableUrl = elt.firstChildElement( "podcastmp3" ).text();
-    if (item.playableUrl != "not yet available" )
+    if( item.playableUrl != "not yet available" && ( isFilterMatch( item.itemName ) || addAll ) )
       script.insertItem( item );
 
     item.itemName    = "Podcast (OGG)";
     item.playableUrl = elt.firstChildElement( "podcastogg" ).text();
-    if (item.playableUrl != "not yet available" )
+    if( item.playableUrl != "not yet available" && ( isFilterMatch( item.itemName ) || addAll ) )
       script.insertItem( item );
 
     /* The songs */
@@ -195,7 +229,7 @@ function onPopulate( level, callbackData, filter ) {
 
     var songArtistTitle = new Array( 2 );
 
-    for ( ; i != 0; i-- ) {
+    for( ; i != 0; i-- ) {
       // beautify rank for newcomers
       elt2 = elt.firstChildElement( "rank" );
       if( elt2.text() == "99" ) item.itemName = "New";
@@ -212,13 +246,14 @@ function onPopulate( level, callbackData, filter ) {
 
       elt2 = elt.firstChildElement( "url" );
       item.playableUrl = elt2.text();
-      item.infoHtml = "<center><b>Chart positions of<br/>";
-      item.infoHtml = item.infoHtml + elt.firstChildElement( "name" ).text();
-      item.infoHtml = item.infoHtml + "</b></center><br/>";
-      item.infoHtml = item.infoHtml + fmcTracksXmlParser( elt2.text() );
+      item.infoHtml = "<div style=\"background-color:#999999\"><center><b>Chart positions of<br/>";
+      item.infoHtml = item.infoHtml + elt.firstChildElement( "name" ).text() + "</div>";
+      item.infoHtml = item.infoHtml + "</b></center><div style=\"color:#153860\">";
+      item.infoHtml = item.infoHtml + fmcTracksXmlParser( elt2.text() ) + "</div>";
 
       elt = elt.nextSiblingElement( "song" );
-      script.insertItem( item );
+      if( addAll || isFilterMatch( item.itemName ) )
+        script.insertItem( item );
     }
 
     Amarok.debug( "done populating fmc track level..." );
@@ -226,3 +261,76 @@ function onPopulate( level, callbackData, filter ) {
   }
 }
 
+
+function onVote() {
+  Amarok.debug( "FMC: onVote" );
+  QDesktopServices.openUrl( votingUrl );
+}
+
+
+/*###########################################################################
+#   Helper functions                                                      #
+########################################################################### */
+
+/* checks weather a show contains a match for the current filter */
+function containsFilterMatch( id ) {
+  Amarok.debug( "FMC: containsFilterMatch" );
+
+  if( currentFilter == "" ) // no filter -> matches everywhere
+    return true;
+
+  else {
+    // podcast names are not in the xml
+    var itemNameLowerCase = "podcast (mp3)";
+    if( itemNameLowerCase.indexOf( currentFilter ) != -1 )
+      return true;
+
+    itemNameLowerCase = "podcast (ogg)";
+    if( itemNameLowerCase.indexOf( currentFilter ) != -1 )
+      return true;
+
+    var tempElt = new QDomElement;
+    tempElt = shows.at( id );
+
+    // match the show title
+    itemNameLowerCase = tempElt.firstChildElement( "title" ).text();
+    itemNameLowerCase = itemNameLowerCase.toLowerCase();
+    if( itemNameLowerCase.indexOf( currentFilter ) != -1 )
+      return true;
+
+    var i   = tempElt.firstChildElement( "songcount" ).text();
+    tempElt = tempElt.firstChildElement( "songs" ); // ascend to songs
+    tempElt = tempElt.firstChildElement( "song" );  // ascent to first song
+
+    for( ; i != 0; i-- ) {
+      itemNameLowerCase = tempElt.firstChildElement( "name" ).text();
+      itemNameLowerCase = itemNameLowerCase.toLowerCase();
+
+      if( itemNameLowerCase.indexOf( currentFilter ) != -1 )
+        return true;
+
+      tempElt = tempElt.nextSiblingElement( "song" );
+    }
+  }
+
+  return false;
+}
+
+
+/* checks weather a song matches a filter */
+function isFilterMatch( itemName ) {
+  Amarok.debug( "FMC: isFilterMatch" );
+  var itemNameLowerCase = itemName.toLowerCase();
+
+  if( ( currentFilter != "" ) && ( itemNameLowerCase.indexOf( currentFilter ) == -1 ) )
+    return false;
+
+  return true;
+}
+
+
+String.prototype.trim = function() {
+  a = this.replace(/^\s+/, '');
+  return a.replace(/\s+$/, '');
+}
+
