@@ -1,5 +1,6 @@
 /*
     Copyright (c) 2004-2005 Max Howell <max.howell@methylblue.com>
+    Copyright (c) 2009 Mark Kretschmann <kretschmann@kde.org>
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -18,12 +19,14 @@
 #ifndef AMAROK_H
 #define AMAROK_H
 
+#include "amarok_export.h"
+
 #include <KActionCollection>
 #include <KConfig>
 #include <KIO/NetAccess>
 #include <KUrl> // recursiveUrlExpand
 
-#include "amarok_export.h"
+#include <QPointer>
 
 class QColor;
 class QDateTime;
@@ -31,6 +34,7 @@ class QEvent;
 class QMutex;
 class QPixmap;
 class QWidget;
+
 namespace KIO { class Job; }
 
 namespace Amarok
@@ -42,6 +46,8 @@ namespace Amarok
     const int GUI_THREAD_ID = 0;
 
     extern QMutex globalDirsMutex; // defined in App.cpp
+    extern QPointer<KActionCollection> actionCollectionObject; // defined in App.cpp
+
 
     namespace ColorScheme
     {
@@ -71,19 +77,9 @@ namespace Amarok
 
     /**
      * @return the KActionCollection used by Amarok
-     * The KActionCollection is owned by the PlaylistWindow, so you must ensure
-     * you don't try to use this before then, but we've taken steps to prevent
-     * this eventuality - you should be safe.
      */
     KActionCollection *actionCollection(); //defined in App.cpp
 
-    /**
-     * Invoke the external web browser set in Amarok's configuration.
-     * @param url The URL to be opened in the browser.
-     * @return True if the browser could be started.
-     */
-    AMAROK_EXPORT bool invokeBrowser( const QString& url ); //defined in App.cpp
-
     /**
      * Compute score for a track that has finished playing.
      */
@@ -263,6 +259,13 @@ namespace Amarok
      * @param reverse if true, The Eagles -> Eagles, The. If false, Eagles, The -> The Eagles
      */
     AMAROK_EXPORT void manipulateThe( QString &str, bool reverse );
+
+    /**
+     * Returns the highlight color which should be used instead of the color from KDE.
+     * @return Highlight color, which is the KDE highlight color, with reduced saturation (less contrast).
+     */
+    AMAROK_EXPORT QColor highlightColor();  //defined in App.cpp
+
 }
 
 
@@ -301,6 +304,6 @@ namespace Amarok
 
 
 /// Update this when necessary
-#define APP_VERSION "2.0.2"
+#define APP_VERSION "2.0.90"
 
 #endif
