@@ -27,9 +27,11 @@
 
 const QString AmarokMimeData::TRACK_MIME = "application/x-amarok-tracks";
 const QString AmarokMimeData::PLAYLIST_MIME = "application/x-amarok-playlists";
-const QString AmarokMimeData::PLAYLISTBROWSERGROUP_MIME = "application/x-amarok-playlistbrowsergroup";
+// const QString AmarokMimeData::PLAYLISTBROWSERGROUP_MIME = "application/x-amarok-playlistbrowsergroup";
 const QString AmarokMimeData::PODCASTCHANNEL_MIME = "application/x-amarok-podcastchannel";
 const QString AmarokMimeData::PODCASTEPISODE_MIME = "application/x-amarok-podcastepisode";
+const QString AmarokMimeData::AMAROKURL_MIME = "application/x-amarok-amarokurl";
+const QString AmarokMimeData::BOOKMARKGROUP_MIME = "application/x-amarok-bookmarkgroup";
 
 
 class AmarokMimeData::Private
@@ -46,12 +48,15 @@ public:
 
     Meta::TrackList tracks;
     Meta::PlaylistList playlists;
-    SqlPlaylistGroupList playlistGroups;
+//     SqlPlaylistGroupList playlistGroups;
     Meta::PodcastChannelList m_podcastChannels;
     Meta::PodcastEpisodeList m_podcastEpisodes;
     QList<QueryMaker*> queryMakers;
     QMap<QueryMaker*, Meta::TrackList> trackMap;
     QMap<QueryMaker*, Meta::PlaylistList> playlistMap;
+    BookmarkList bookmarks;
+    BookmarkGroupList bookmarkGroups;
+
     bool deleteQueryMakers;
     int completedQueries;
 
@@ -72,20 +77,26 @@ AmarokMimeData::~AmarokMimeData()
 QStringList
 AmarokMimeData::formats() const
 {
+    DEBUG_BLOCK
+
     QStringList formats( QMimeData::formats() );
-    if( !d->tracks.isEmpty() || !d->queryMakers.isEmpty() || !d->playlistGroups.isEmpty() )
+    if( !d->tracks.isEmpty() || !d->queryMakers.isEmpty() /*|| !d->playlistGroups.isEmpty()*/ || !d->bookmarks.isEmpty() || !d->bookmarkGroups.isEmpty() )
     {
         formats.append( TRACK_MIME );
         formats.append( PLAYLIST_MIME );
-        formats.append( PLAYLISTBROWSERGROUP_MIME );
+//         formats.append( PLAYLISTBROWSERGROUP_MIME );
         formats.append( PODCASTCHANNEL_MIME );
         formats.append( PODCASTEPISODE_MIME );
+        formats.append( BOOKMARKGROUP_MIME );
+        formats.append( AMAROKURL_MIME );
 
         if( !formats.contains( "text/uri-list" ) )
             formats.append( "text/uri-list" );
         if( !formats.contains( "text/plain" ) )
             formats.append( "text/plain" );
     }
+
+    debug() << "returning formats: " << formats;
     return formats;
 }
 
@@ -96,12 +107,16 @@ AmarokMimeData::hasFormat( const QString &mimeType ) const
         return !d->tracks.isEmpty() || !d->queryMakers.isEmpty();
     else if( mimeType == PLAYLIST_MIME )
         return !d->playlists.isEmpty() || !d->queryMakers.isEmpty();
-    else if( mimeType == PLAYLISTBROWSERGROUP_MIME )
-        return !d->playlistGroups.isEmpty();
+//     else if( mimeType == PLAYLISTBROWSERGROUP_MIME )
+//         return !d->playlistGroups.isEmpty();
     else if( mimeType == PODCASTCHANNEL_MIME )
         return !d->m_podcastChannels.isEmpty();
     else if( mimeType == PODCASTEPISODE_MIME )
         return !d->m_podcastEpisodes.isEmpty();
+    else if( mimeType == BOOKMARKGROUP_MIME )
+        return !d->bookmarkGroups.isEmpty();
+    else if( mimeType == AMAROKURL_MIME )
+        return !d->bookmarks.isEmpty();
     else if( mimeType == "text/uri-list" || mimeType == "text/plain" )
         return !d->tracks.isEmpty() || !d->playlists.isEmpty()
             || !d->m_podcastChannels.isEmpty() || !d->m_podcastEpisodes.isEmpty()
@@ -185,7 +200,7 @@ AmarokMimeData::addPlaylists( const Meta::PlaylistList &playlists )
 {
     d->playlists << playlists;
 }
-
+/*
 SqlPlaylistGroupList AmarokMimeData::sqlPlaylistsGroups() const
 {
     return d->playlistGroups;
@@ -199,7 +214,7 @@ void AmarokMimeData::setPlaylistGroups( const SqlPlaylistGroupList & groups )
 void AmarokMimeData::addPlaylistGroups(const SqlPlaylistGroupList & groups)
 {
     d->playlistGroups << groups;
-}
+}*/
 
 Meta::PodcastChannelList
 AmarokMimeData::podcastChannels() const
@@ -256,6 +271,36 @@ AmarokMimeData::setQueryMakers( const QList<QueryMaker*> &queryMakers )
     d->queryMakers << queryMakers;
 }
 
+BookmarkList AmarokMimeData::bookmarks() const
+{
+    return d->bookmarks;
+}
+
+void AmarokMimeData::setBookmarks( const BookmarkList &bookmarks )
+{
+    d->bookmarks = bookmarks;
+}
+
+void AmarokMimeData::addBookmarks( const BookmarkList &bookmarks )
+{
+    d->bookmarks << bookmarks;
+}
+
+BookmarkGroupList AmarokMimeData::bookmarkGroups() const
+{
+    return d->bookmarkGroups;
+}
+
+void AmarokMimeData::setBookmarkGroups( const BookmarkGroupList &groups )
+{
+    d->bookmarkGroups = groups;
+}
+
+void AmarokMimeData::addBookmarkGroups( const BookmarkGroupList &groups )
+{
+    d->bookmarkGroups << groups;
+}
+
 QVariant
 AmarokMimeData::retrieveData( const QString &mimeType, QVariant::Type type ) const
 {
@@ -355,4 +400,5 @@ AmarokMimeData::queryDone()
     d->completedQueries++;
 }
 
+
 #include "AmarokMimeData.moc"
