@@ -14,20 +14,22 @@ email                : markey@web.de
  *   (at your option) any later version.                                   *
  *                                                                         *
  ***************************************************************************/
+
 #include "App.h"
 
 #include "Amarok.h"
 #include "amarokconfig.h"
+#include "amarokurls/AmarokUrl.h"
 #include "CollectionManager.h"
 #include "ConfigDialog.h"
 #include "covermanager/CoverFetcher.h"
+#include "dbus/CollectionDBusHandler.h"
 #include "Debug.h"
 #include "EngineController.h"
 #include "firstruntutorial/FirstRunTutorial.h"
 #include "MainWindow.h"
 #include "Meta.h"
 #include "meta/MetaConstants.h"
-#include "metadata/tplugins.h"
 #include "MountPointManager.h"
 #include "Osd.h"
 #include "PlayerDBusHandler.h"
@@ -53,20 +55,28 @@ email                : markey@web.de
 #include <KJob>
 #include <KJobUiDelegate>
 #include <KLocale>
-#include <KRun>                          //Amarok::invokeBrowser()
-#include <KShell>
 #include <KShortcutsDialog>              //slotConfigShortcuts()
 #include <KSplashScreen>
 #include <KStandardDirs>
 
+#include <QByteArray>
 #include <QFile>
 #include <KPixmapCache>
 #include <QStringList>
 #include <QTimer>                       //showHyperThreadingWarning()
 #include <QtDBus/QtDBus>
 
+#ifdef TAGLIB_EXTRAS_FOUND
+#include <audiblefiletyperesolver.h>
+#include <asffiletyperesolver.h>
+#include <wavfiletyperesolver.h>
+#include <realmediafiletyperesolver.h>
+#include <mp4filetyperesolver.h>
+#endif
+
 QMutex Debug::mutex;
 QMutex Amarok::globalDirsMutex;
+QPointer<KActionCollection> Amarok::actionCollectionObject;
 
 int App::mainThreadId = 0;
 
@@ -79,8 +89,8 @@ extern void setupEventHandler_mac(long);
 AMAROK_EXPORT KAboutData aboutData( "amarok", 0,
     ki18n( "Amarok" ), APP_VERSION,
     ki18n( "The audio player for KDE" ), KAboutData::License_GPL,
-    ki18n( "(C) 2002-2003, Mark Kretschmann\n(C) 2003-2008, The Amarok Development Squad" ),
-    ki18n( "IRC:\nirc.freenode.net - #amarok, #amarok.de, #amarok.es, #amarok.fr\n\nFeedback:\namarok@kde.org\n\n(Build Date: " __DATE__ ")" ),
+    ki18n( "(C) 2002-2003, Mark Kretschmann\n(C) 2003-2009, The Amarok Development Squad" ),
+    ki18n( "IRC:\nirc.freenode.net - #amarok, #amarok.de, #amarok.es, #amarok.fr\n\nFeedback:\namarok@kde.org\n\n(Build Date: %1)" ).subs( __DATE__ ),
              ( "http://amarok.kde.org" ) );
 
 App::App()
@@ -91,7 +101,7 @@ App::App()
     PERF_LOG( "Begin Application ctor" )
 
     // required for last.fm plugin to grab app version
-    setApplicationVersion( "2.0-SVN" );
+    setApplicationVersion( "2.1-SVN" );
 
     if( AmarokConfig::showSplashscreen() && !isSessionRestored() )
     {
@@ -107,9 +117,15 @@ App::App()
         m_splash->show();
     }
 
+#ifdef TAGLIB_EXTRAS_FOUND
     PERF_LOG( "Registering taglib plugins" )
-    registerTaglibPlugins();
+    TagLib::FileRef::addFileTypeResolver(new MP4FileTypeResolver);
+    TagLib::FileRef::addFileTypeResolver(new ASFFileTypeResolver);
+    TagLib::FileRef::addFileTypeResolver(new RealMediaFileTypeResolver);
+    TagLib::FileRef::addFileTypeResolver(new AudibleFileTypeResolver);
+    TagLib::FileRef::addFileTypeResolver(new WavFileTypeResolver);
     PERF_LOG( "Done Registering taglib plugins" )
+#endif
 
     qRegisterMetaType<Meta::DataPtr>();
     qRegisterMetaType<Meta::DataList>();
@@ -203,10 +219,14 @@ App::~App()
             {
                 AmarokConfig::setResumeTrack( track->playableUrl().prettyUrl() );
                 AmarokConfig::setResumeTime( The::engineController()->trackPosition() * 1000 );
+                AmarokConfig::setLastPlaying( Playlist::Model::instance()->activeRow() );
             }
         }
-        else 
+        else
+        {
             AmarokConfig::setResumeTrack( QString() ); //otherwise it'll play previous resume next time!
+            AmarokConfig::setLastPlaying( -1 );
+        }
     }
 
     The::engineController()->endSession(); //records final statistics
@@ -233,20 +253,20 @@ App::~App()
     // Config is written. Go figure!
     AmarokConfig::setLastPlaying( Playlist::Model::instance()->rowForTrack( Playlist::Model::instance()->activeTrack() ) );
 
-    AmarokConfig::setVersion( APP_VERSION );
     AmarokConfig::self()->writeConfig();
 
     mainWindow()->deleteBrowsers();
     delete mainWindow();
 
+    CollectionManager::destroy();
+    MountPointManager::destroy();
     Playlist::Actions::destroy();
     Playlist::Model::destroy();
     PlaylistManager::destroy();
-    CollectionManager::destroy();
-    MountPointManager::destroy();
     EngineController::destroy();
     CoverFetcher::destroy();
 
+
 #ifdef Q_WS_WIN
     // work around for KUniqueApplication being not completely implemented on windows
     QDBusConnectionInterface* dbusService;
@@ -280,8 +300,16 @@ App::handleCliArgs() //static
 //             if( url.protocol() == "itpc" || url.protocol() == "pcast" )
 //                 PlaylistBrowserNS::instance()->addPodcast( url );
 //             else
+
+            if ( url.protocol() == "amarok" ) {
+
+                AmarokUrl aUrl( url.url() );
+                aUrl.run();
+
+            } else {
                 list << url;
                 DEBUG_LINE_INFO
+            }
         }
 
         int options = Playlist::AppendAndPlay;
@@ -353,15 +381,26 @@ App::handleCliArgs() //static
     */
 
     const bool debugWasJustEnabled = !Amarok::config().readEntry( "Debug Enabled", false ) && args->isSet( "debug" );
+    const bool debugIsDisabled = !args->isSet( "debug" );
+
     Amarok::config().writeEntry( "Debug Enabled", args->isSet( "debug" ) );
 
     // Debug output will only work from this point on. If Amarok was run without debug output before,
     // then a part of the output (until this point) will be missing. Inform the user about this:
     if( debugWasJustEnabled )
     {
-       debug() << "************************************************************************************************************";
-       debug() << "** DEBUGGING OUTPUT IS NOW ENABLED. PLEASE NOTE THAT YOU WILL ONLY SEE THE FULL OUTPUT ON THE NEXT START. **";
-       debug() << "************************************************************************************************************";
+        debug() << "************************************************************************************************************";
+        debug() << "** DEBUGGING OUTPUT IS NOW ENABLED. PLEASE NOTE THAT YOU WILL ONLY SEE THE FULL OUTPUT ON THE NEXT START. **";
+        debug() << "************************************************************************************************************";
+    }
+    else if( debugIsDisabled )
+    {
+        Amarok::config().writeEntry( "Debug Enabled", true );
+        debug() << "**********************************************************************************************";
+        debug() << "** AMAROK WAS STARTED IN NORMAL MODE. IF YOU WANT TO SEE DEBUGGING INFORMATION, PLEASE USE: **";
+        debug() << "** amarok --debug                                                                           **";
+        debug() << "**********************************************************************************************";
+        Amarok::config().writeEntry( "Debug Enabled", false );
     }
 
     static bool firstTime = true;
@@ -485,13 +524,13 @@ void App::applySettings( bool firstTime )
     //on startup we need to show the window, but only if it wasn't hidden on exit
     //and always if the trayicon isn't showing
     QWidget* main_window = mainWindow();
-    
+
     if( ( main_window && firstTime && !Amarok::config().readEntry( "HiddenOnExit", false ) ) || ( main_window && !AmarokConfig::showTrayIcon() ) )
     {
         PERF_LOG( "showing main window again" )
         main_window->show();
         PERF_LOG( "after showing mainWindow" )
-    } 
+    }
 
     { //<Engine>
         if( The::engineController()->volume() != AmarokConfig::masterVolume() )
@@ -503,6 +542,9 @@ void App::applySettings( bool firstTime )
             Amarok::OSD::instance()->setEnabled( osdEnabled );
         }
 
+        if( The::engineController()->isMuted() != AmarokConfig::muteState() )
+            The::engineController()->setMuted( AmarokConfig::muteState() );
+
 #if 0
     // Audio CD is not currently supported
     Amarok::actionCollection()->action( "play_audiocd" )->setEnabled( false );
@@ -512,7 +554,7 @@ void App::applySettings( bool firstTime )
 
     {   // delete unneeded cover images from cache
         PERF_LOG( "Begin cover handling" )
-        const QString size = QString::number( AmarokConfig::coverPreviewSize() ) + '@';
+        const QString size = QString::number( 100 ) + '@';
         const QDir cacheDir = Amarok::saveLocation( "albumcovers/cache/" );
         const QStringList obsoleteCovers = cacheDir.entryList( QStringList("*") );
         foreach( const QString &it, obsoleteCovers )
@@ -549,14 +591,11 @@ App::continueInit()
 
     m_tray = new Amarok::TrayIcon( mainWindow() );
 
-    PERF_LOG( "Start init of MainWindow" )
-    mainWindow()->init(); //creates the playlist, browsers, etc.
-    PERF_LOG( "Init of MainWindow done" )
-
     PERF_LOG( "Creating DBus handlers" )
     new Amarok::RootDBusHandler();
     new Amarok::PlayerDBusHandler();
     new Amarok::TracklistDBusHandler();
+    new CollectionDBusHandler( this );
     QDBusConnection::sessionBus().registerService("org.mpris.amarok");
     PERF_LOG( "Done creating DBus handlers" )
     //DON'T DELETE THIS NEXT LINE or the app crashes when you click the X (unless we reimplement closeEvent)
@@ -589,10 +628,14 @@ App::continueInit()
     m_splash = 0;
     PERF_LOG( "App init done" )
     KConfigGroup config = KGlobal::config()->group( "General" );
-    const bool firstruntut = config.readEntry( "FirstRunTutorial", false );
+
+    // NOTE: First Run Tutorial disabled for 2.1-beta1 release (too buggy / unfinished)
+#if 0
+    const bool firstruntut = config.readEntry( "FirstRunTutorial", true );
     debug() << "Checking whether to run first run tutorial..." << firstruntut;
     if( firstruntut )
     {
+        config.writeEntry( "FirstRunTutorial", false );
         debug() << "Starting first run tutorial";
         FirstRunTutorial *frt = new FirstRunTutorial( mainWindow() );
         QTimer::singleShot( 1000, frt, SLOT( initOverlay() ) );
@@ -603,6 +646,7 @@ App::continueInit()
         slotConfigAmarok( "CollectionConfig" );
         config.writeEntry( "First Run", false );
     }
+#endif
 }
 
 void App::slotConfigEqualizer() //SLOT
@@ -633,20 +677,6 @@ void App::slotConfigShortcuts()
     AmarokConfig::self()->writeConfig();
 }
 
-void App::slotConfigToolBars()
-{
-    KEditToolBar dialog( Amarok::actionCollection(), mainWindow() );
-    dialog.setResourceFile( mainWindow()->xmlFile() );
-
-    dialog.showButton( KEditToolBar::Apply, false );
-
-//     if( dialog.exec() )
-//     {
-//         mainWindow()->reloadXML();
-//         mainWindow()->createGUI();
-//     }
-}
-
 KIO::Job *App::trashFiles( const KUrl::List &files )
 {
     KIO::Job *job = KIO::trash( files );
@@ -812,9 +842,15 @@ namespace Amarok
         return pApp->mainWindow();
     }
 
-    KActionCollection *actionCollection()
+    KActionCollection* actionCollection()  // TODO: constify?
     {
-        return pApp->mainWindow()->actionCollection();
+        if( !actionCollectionObject )
+        {
+            actionCollectionObject = new KActionCollection( pApp );
+            actionCollectionObject->setObjectName( "Amarok-KActionCollection" );
+        }
+
+        return actionCollectionObject;
     }
 
     KConfigGroup config( const QString &group )
@@ -823,24 +859,14 @@ namespace Amarok
         return KGlobal::config()->group( group );
     }
 
-    bool invokeBrowser( const QString& url )
+    QColor highlightColor()
     {
-        //URL can be in whatever forms KUrl understands - ie most.
-        QString browser = AmarokConfig::externalBrowser();
+        QColor highlight( App::instance()->palette().highlight().color() );
+        qreal saturation = highlight.saturationF();
+        saturation *= 0.6;
+        highlight.setHsvF( highlight.hueF(), saturation, highlight.valueF(), highlight.alphaF() );
 
-        // HACK to get around KConfigDialog. See configdialog/dialogs/GeneralConfig.cpp
-#ifdef Q_WS_MAC
-        if( browser == i18n( "Default Browser" ) )
-            browser = "open";
-#else
-        if( browser == i18n( "Default KDE Browser" ) )
-            browser = "xdg-open";
-#endif
-
-        const QString cmd = KShell::quoteArg( browser ) 
-                            + ' ' + KShell::quoteArg( KUrl( url ).url() );
-
-        return ( KRun::runCommand( cmd, 0L ) > 0 );
+        return highlight;
     }
 
     namespace ColorScheme
@@ -997,15 +1023,14 @@ namespace Amarok
      */
     QString decapitateString( const QString &input, const QString &ref )
     {
-        //Sometimes it's good to be c-like...
         int len;    //the length of common prefix calculated so far
-        for( len = 0; len < input.length() && len < ref.length(); len++ )
+        for ( len = 0; len < input.length() && len < ref.length(); len++ )
         {
             if ( input.at( len ).toUpper() != ref.at( len ).toUpper() )
                 break;
         }
 
-        return input.right(input.length() - len).trimmed();
+        return input.right( input.length() - len ).trimmed();
     }
 
     KIO::Job *trashFiles( const KUrl::List &files ) { return App::instance()->trashFiles( files ); }
