@@ -21,11 +21,12 @@
 #include <config-amarok.h>  
 
 #include "amarok_export.h"
+
 #include <KUniqueApplication>   //baseclass
 #include <KUrl>
 
-#include <QByteArray>
 #include <QHash>
+#include <QPointer>
 #include <QString>
 
 namespace Amarok {
@@ -51,10 +52,9 @@ class AMAROK_EXPORT App : public KUniqueApplication
         static App *instance() { return static_cast<App*>( kapp ); }
 
         void setUniqueInstance( bool isUnique ) { m_isUniqueInstance = isUnique; }
-        bool isUniqueInstance() const { return m_isUniqueInstance; }
+        bool isNonUniqueInstance() const { return m_isUniqueInstance; }
 
-	Amarok::TrayIcon* trayIcon() { return m_tray; }
-	
+        Amarok::TrayIcon* trayIcon() { return m_tray; }
         static void handleCliArgs();
         static void initCliArgs( int argc, char *argv[] );
         static void initCliArgs();
@@ -78,7 +78,6 @@ class AMAROK_EXPORT App : public KUniqueApplication
         void applySettings( bool firstTime = false );
         void slotConfigAmarok( const QString& page = QString() );
         void slotConfigShortcuts();
-        void slotConfigToolBars();
         void slotConfigEqualizer();
         KIO::Job *trashFiles( const KUrl::List &files );
         void quit();
