@@ -24,7 +24,8 @@
 #include "MainWindow.h"
 #include "meta/Meta.h"
 #include "meta/MetaConstants.h"
-#include "meta/MultiPlayableCapability.h"
+#include "meta/capabilities/MultiPlayableCapability.h"
+#include "meta/capabilities/MultiSourceCapability.h"
 #include "playlist/PlaylistActions.h"
 #include "playlistmanager/PlaylistManager.h"
 #include "PluginManager.h"
@@ -41,6 +42,8 @@
 
 #include <QTimer>
 
+#include <cmath>
+
 namespace The {
     EngineController* engineController() { return EngineController::instance(); }
 }
@@ -61,48 +64,15 @@ EngineController::destroy()
 }
 
 EngineController::EngineController()
-    : m_media( 0 )
-    , m_audio( 0 )
-    , m_playWhenFetched( true )
+    : m_playWhenFetched( true )
     , m_fadeoutTimer( new QTimer( this ) )
 {
     DEBUG_BLOCK
 
-    PERF_LOG( "EngineController: loading phonon objects" )
-    m_media = new Phonon::MediaObject( this );
-    m_audio = new Phonon::AudioOutput( Phonon::MusicCategory, this );
-
-    m_path = Phonon::createPath(m_media, m_audio);
-
-    m_media->setTickInterval( 100 );
-    debug() << "Tick Interval (actual): " << m_media->tickInterval();
-    PERF_LOG( "EngineController: loaded phonon objects" )
+    initializePhonon();
 
     m_fadeoutTimer->setSingleShot( true );
 
-    //TODO: The xine engine does not support crossfading.
-    // I cannot get the gstreamer engine to work, will test this once I do.
-
-//     if( AmarokConfig::trackDelayLength() > -1 )
-//         m_media->setTransitionTime( AmarokConfig::trackDelayLength() ); // Also Handles gapless.
-//     else if( AmarokConfig::crossfadeLength() > 0 )  // TODO: Handle the possible options on when to crossfade.. the values are not documented anywhere however
-//         m_media->setTransitionTime( -AmarokConfig::crossfadeLength() );
-
-    connect( m_media, SIGNAL( finished() ), SLOT( slotTrackEnded() ) );
-
-    // Get the next track when there is 2 seconds left on the current one.
-    m_media->setPrefinishMark( 2000 );
-    connect( m_media, SIGNAL( prefinishMarkReached(qint32) ), SLOT( slotPrefinishMarkReached( qint32 ) ) );
-    connect( m_media, SIGNAL( aboutToFinish()), SLOT( slotAboutToFinish() ) );
-
-    connect( m_media, SIGNAL( metaDataChanged() ), SLOT( slotMetaDataChanged() ) );
-    connect( m_media, SIGNAL( stateChanged( Phonon::State, Phonon::State ) ),
-                      SLOT( slotStateChanged( Phonon::State, Phonon::State ) ) );
-    connect( m_media, SIGNAL( tick( qint64 ) ), SLOT( slotTick( qint64 ) ) );
-    connect( m_media, SIGNAL( totalTimeChanged( qint64 ) ), SLOT( slotTrackLengthChanged( qint64 ) ) );
-    connect( m_media, SIGNAL( currentSourceChanged( const Phonon::MediaSource & ) ),
-                       SLOT( slotNewTrackPlaying( const Phonon::MediaSource & ) ) );
-
     connect( m_fadeoutTimer, SIGNAL( timeout() ), SLOT( slotStopFadeout() ) );
 
     s_instance = this;
@@ -118,6 +88,59 @@ EngineController::~EngineController()
     delete m_audio;
 }
 
+void
+EngineController::initializePhonon()
+{
+    DEBUG_BLOCK
+
+    delete m_media;
+    delete m_audio;
+    delete m_preamp;
+
+    PERF_LOG( "EngineController: loading phonon objects" )
+    m_media = new Phonon::MediaObject( this );
+    m_audio = new Phonon::AudioOutput( Phonon::MusicCategory, this );
+
+    m_path = Phonon::createPath( m_media, m_audio );
+    
+    // HACK we turn off replaygain manually on OSX, until the phonon coreaudio backend is fixed.
+    // as the default is specified in the .cfg file, we can't just tell it to be a different default on OSX
+#ifdef Q_WS_MAC
+    AmarokConfig::setReplayGainMode( AmarokConfig::EnumReplayGainMode::Off );
+#endif
+
+    // only create pre-amp if we have replaygain on, preamp can cause phonon issues
+    if( AmarokConfig::replayGainMode() != AmarokConfig::EnumReplayGainMode::Off )
+    {   
+        m_preamp = new Phonon::VolumeFaderEffect( this );
+        m_path.insertEffect( m_preamp );
+    }
+
+    m_media->setTickInterval( 100 );
+    debug() << "Tick Interval (actual): " << m_media->tickInterval();
+    PERF_LOG( "EngineController: loaded phonon objects" )
+
+    // Get the next track when there is 2 seconds left on the current one.
+    m_media->setPrefinishMark( 2000 );
+
+    connect( m_media, SIGNAL( finished() ), SLOT( slotQueueEnded() ) );
+    connect( m_media, SIGNAL( aboutToFinish()), SLOT( slotAboutToFinish() ) );
+    connect( m_media, SIGNAL( metaDataChanged() ), SLOT( slotMetaDataChanged() ) );
+    connect( m_media, SIGNAL( stateChanged( Phonon::State, Phonon::State ) ), SLOT( slotStateChanged( Phonon::State, Phonon::State ) ) );
+    connect( m_media, SIGNAL( tick( qint64 ) ), SLOT( slotTick( qint64 ) ) );
+    connect( m_media, SIGNAL( totalTimeChanged( qint64 ) ), SLOT( slotTrackLengthChanged( qint64 ) ) );
+    connect( m_media, SIGNAL( currentSourceChanged( const Phonon::MediaSource & ) ), SLOT( slotNewTrackPlaying( const Phonon::MediaSource & ) ) );
+
+    
+    //TODO: The xine engine does not support crossfading. Cannot get the gstreamer engine to work, will test this once I do.
+#if 0
+    if( AmarokConfig::trackDelayLength() > -1 )
+        m_media->setTransitionTime( AmarokConfig::trackDelayLength() ); // Also Handles gapless.
+    else if( AmarokConfig::crossfadeLength() > 0 )  // TODO: Handle the possible options on when to crossfade.. the values are not documented anywhere however
+        m_media->setTransitionTime( -AmarokConfig::crossfadeLength() );
+#endif
+}
+
 
 //////////////////////////////////////////////////////////////////////////////////////////
 // PUBLIC
@@ -217,7 +240,7 @@ EngineController::endSession()
     //only update song stats, when we're not going to resume it
     if ( !AmarokConfig::resumePlayback() && m_currentTrack )
     {
-        playbackEnded( trackPosition(), m_currentTrack->length(), "quit" );
+        playbackEnded( trackPosition(), m_currentTrack->length(), EngineObserver::EndedQuit );
         emit trackChanged( Meta::TrackPtr( 0 ) );
     }
 }
@@ -256,8 +279,10 @@ EngineController::play( const Meta::TrackPtr& track, uint offset )
         return;
 
     m_currentTrack = track;
-    delete m_multi;
-    m_multi = m_currentTrack->as<Meta::MultiPlayableCapability>();
+    delete m_multiPlayback;
+    delete m_multiSource;
+    m_multiPlayback = m_currentTrack->as<Meta::MultiPlayableCapability>();
+    m_multiSource  = m_currentTrack->as<Meta::MultiSourceCapability>();
 
     m_nextTrack.clear();
     m_nextUrl.clear();
@@ -265,11 +290,18 @@ EngineController::play( const Meta::TrackPtr& track, uint offset )
 
     m_currentTrack->prepareToPlay();
 
-    if( m_multi )
+    if( m_multiPlayback )
     {
         m_media->stop();
-        connect( m_multi, SIGNAL( playableUrlFetched( const KUrl & ) ), this, SLOT( slotPlayableUrlFetched( const KUrl & ) ) );
-        m_multi->fetchFirst();
+        connect( m_multiPlayback, SIGNAL( playableUrlFetched( const KUrl & ) ), this, SLOT( slotPlayableUrlFetched( const KUrl & ) ) );
+        m_multiPlayback->fetchFirst();
+    }
+    else if ( m_multiSource )
+    {
+        m_media->stop();
+        debug() << "Got a MultiSource Track with " <<  m_multiSource->sources().count() << " sources";
+        connect( m_multiSource, SIGNAL( urlChanged( const KUrl & ) ), this, SLOT( slotPlayableUrlFetched( const KUrl & ) ) );
+        playUrl( m_currentTrack->playableUrl(), 0 );
     }
     else
     {
@@ -311,7 +343,8 @@ EngineController::stop( bool forceInstant ) //SLOT
     DEBUG_BLOCK
 
     // need to get a new instance of multi if played again
-    delete m_multi;
+    delete m_multiPlayback;
+    delete m_multiSource;
 
     m_mutex.lock();
     m_nextTrack.clear();
@@ -326,7 +359,7 @@ EngineController::stop( bool forceInstant ) //SLOT
         const int pos = trackPosition();
         const int length = m_currentTrack->length();
         m_currentTrack->finishedPlaying( double(pos)/double(length) );
-        playbackEnded( pos, length, "stop" );
+        playbackEnded( pos, length, EngineObserver::EndedStopped );
         emit trackChanged( Meta::TrackPtr( 0 ) );
     }
 
@@ -427,13 +460,9 @@ int
 EngineController::setVolume( int percent ) //SLOT
 {
     if( percent < 0 ) percent = 0;
-    if( percent > 100 ) percent = 100;
+    else if( percent > 100 ) percent = 100;
 
-    // If we're explicitly setting the volume, then I think it's safe
-    // to assume that we don't want the audio stream muted
-    m_audio->setMuted( false );
-
-    qreal newVolume = percent / 100.0; //Phonon's volume is 0.0 - 1.0
+    qreal newVolume = percent / 100.0; // Phonon's volume is 0.0 - 1.0
     m_audio->setVolume( newVolume );
     AmarokConfig::setMasterVolume( percent );
     volumeChangedNotify( percent );
@@ -455,16 +484,20 @@ EngineController::isMuted() const
 }
 
 void
-EngineController::mute() //SLOT
+EngineController::setMuted( bool mute ) //SLOT
 {
-    // if it's already muted then we restore to previous value
-    int newPercent = m_audio->isMuted() ? volume() : 0;
+    m_audio->setMuted( mute ); // toggle mute
 
-    m_audio->setMuted( !isMuted() ); // toggle mute
+    AmarokConfig::setMuteState( mute );
+    muteStateChangedNotify( mute );
 
-    AmarokConfig::setMasterVolume( newPercent );
-    volumeChangedNotify( newPercent );
-    emit volumeChanged( newPercent );
+    emit muteStateChanged( mute );
+}
+
+void
+EngineController::toggleMute() //SLOT
+{
+    setMuted( !isMuted() );
 }
 
 Meta::TrackPtr
@@ -489,11 +522,11 @@ EngineController::setNextTrack( Meta::TrackPtr track )
     QMutexLocker locker( &m_mutex );
 
     if( !track )
-    {
-        track->prepareToPlay();
-        if( track->playableUrl().isEmpty() )
-            return;
-    }
+        return;
+
+    track->prepareToPlay();
+    if( track->playableUrl().isEmpty() )
+        return;
 
     if( m_media->state() == Phonon::PlayingState ||
         m_media->state() == Phonon::BufferingState )
@@ -549,43 +582,56 @@ EngineController::slotTick( qint64 position )
 }
 
 
-void
-EngineController::slotPrefinishMarkReached( qint32 msecToEnd )
-{
-    // For some resason, phonon occasionally emits this right when the track starts playing.
-    if( msecToEnd < 0 || m_multi ) //skip for multi tracks as metadata might otehrwise go out of sync
-        return;
-    else
-        slotAboutToFinish();
-}
-
 void
 EngineController::slotAboutToFinish()
 {
     DEBUG_BLOCK
     debug() << "Track finished completely, updating statistics";
-    m_currentTrack->finishedPlaying( 1.0 ); // If we reach aboutToFinish, the track is done as far as we are concerned.
-    if( m_multi )
+    if( m_currentTrack ) // not sure why this should not be the case, but sometimes happens. don't crash.
+        m_currentTrack->finishedPlaying( 1.0 ); // If we reach aboutToFinish, the track is done as far as we are concerned.
+    if( m_multiPlayback )
     {
+        DEBUG_LINE_INFO
         m_mutex.lock();
         m_playWhenFetched = false;
         m_mutex.unlock();
-        m_multi->fetchNext();
+        m_multiPlayback->fetchNext();
+        debug() << "The queue has: " << m_media->queue().size() << " tracks in it";
     }
+    else if( m_multiSource )
+    {
+        debug() << "source finished, lets get the next one";
+        KUrl nextSource = m_multiSource->next();
 
+        if ( !nextSource.isEmpty() )
+        { //more sources
+            m_mutex.lock();
+            m_playWhenFetched = false;
+            m_mutex.unlock();
+            debug() << "playing next source: " << nextSource;
+            slotPlayableUrlFetched( nextSource );
+        }
+        else if( m_media->queue().isEmpty() )
+        { //go to next track
+            The::playlistActions()->requestNextTrack();
+            debug() << "no more sources, skip to next track";
+        }
+         
+    }
     else if( m_media->queue().isEmpty() )
         The::playlistActions()->requestNextTrack();
 }
 
 void
-EngineController::slotTrackEnded()
+EngineController::slotQueueEnded()
 {
     DEBUG_BLOCK
 
-    if( m_currentTrack && !m_multi )
+    if( m_currentTrack && !m_multiPlayback && !m_multiSource )
     {
         m_currentTrack->finishedPlaying( 1.0 );
         emit trackFinished();
+        playbackEnded( trackPosition(), m_currentTrack->length(), EngineObserver::EndedStopped );
         m_currentTrack = 0;
     }
 
@@ -594,10 +640,12 @@ EngineController::slotTrackEnded()
     // Non-local urls are not enqueued so we must play them explicitly.
     if( m_nextTrack )
     {
+        DEBUG_LINE_INFO
         play( m_nextTrack );
     }
     else if( !m_nextUrl.isEmpty() )
     {
+        DEBUG_LINE_INFO
         playUrl( m_nextUrl, 0 );
     }
     else
@@ -607,6 +655,8 @@ EngineController::slotTrackEnded()
     m_mutex.unlock();
 }
 
+static const qreal log10over20 = 0.1151292546497022842; // ln(10) / 20
+
 void
 EngineController::slotNewTrackPlaying( const Phonon::MediaSource &source )
 {
@@ -623,6 +673,34 @@ EngineController::slotNewTrackPlaying( const Phonon::MediaSource &source )
     if( !m_nextUrl.isEmpty() )
         m_nextUrl.clear();
 
+    if ( m_currentTrack && AmarokConfig::replayGainMode() != AmarokConfig::EnumReplayGainMode::Off )
+    {
+        if( !m_preamp ) // replaygain was just turned on, and amarok was started with it off
+        {
+            m_preamp = new Phonon::VolumeFaderEffect( this );
+            m_path.insertEffect( m_preamp );
+        }
+        
+        Meta::Track::ReplayGainMode mode = ( AmarokConfig::replayGainMode() == AmarokConfig::EnumReplayGainMode::Track)
+                                         ? Meta::Track::TrackReplayGain
+                                         : Meta::Track::AlbumReplayGain;
+        // gain is usually negative (but may be positive)
+        qreal gain = m_currentTrack->replayGain( mode );
+        // peak is usually positive and smaller than gain (but may be negative)
+        qreal peak = m_currentTrack->replayPeakGain( mode );
+        if ( gain + peak > 0.0 )
+        {
+            debug() << "Gain of" << gain << "would clip at absolute peak of" << gain + peak;
+            gain -= gain + peak;
+        }
+        debug() << "Using gain of" << gain << "with relative peak of" << peak;
+        // we calculate the volume change ourselves, because m_preamp->setVolumeDecibel is
+        // a little confused about minus signs
+        m_preamp->setVolume( exp( gain * log10over20 ) );
+    }
+    else if( m_preamp )
+        m_preamp->setVolumeDecibel( 0.0 );
+
     // state never changes if tracks are queued, but we need this to update the caption
     stateChangedNotify( m_media->state(), m_media->state() );
 
@@ -634,12 +712,58 @@ void
 EngineController::slotStateChanged( Phonon::State newState, Phonon::State oldState ) //SLOT
 {
     DEBUG_BLOCK
-    // Sanity checks
-    if( newState == oldState || newState == Phonon::BufferingState )
+
+    // Sanity checks:
+    if( newState == oldState )
         return;
 
+    // The following check is an attempt to fix http://bugs.kde.org/show_bug.cgi?id=180339
+    // ("amarok stops playing tracks") and other issues with Phonon.
+    // The theory:
+    // It has been observed that Phonon will sometimes emit a stateChanged() with
+    // _both_ oldState and newState == 0, which makes little sense. After that it goes
+    // berserk, until you restart Amarok. 
+    // Now we try to detect this weird state, and then try to destroy and recreate all
+    // Phonon objects, in the hope of fixing the situation. Fingers crossed.
+    if( newState == Phonon::LoadingState && oldState == Phonon::LoadingState )
+    {
+        warning() << "Trying to re-initialize Phonon. Fingers crossed.";
+        initializePhonon();
+        newState = Phonon::ErrorState;  // Indicate error
+    }
+
     if( newState == Phonon::ErrorState )  // If media is borked, skip to next track
+    {
         warning() << "Phonon failed to play this URL. Error: " << m_media->errorString();
+        if( m_multiPlayback )
+        {
+            DEBUG_LINE_INFO
+            m_mutex.lock();
+            m_playWhenFetched = false;
+            m_mutex.unlock();
+            m_multiPlayback->fetchNext();
+            debug() << "The queue has: " << m_media->queue().size() << " tracks in it";
+        }
+        else if( m_multiSource )
+        {
+            debug() << "source error, lets get the next one";
+            KUrl nextSource = m_multiSource->next();
+
+            if ( !nextSource.isEmpty() )
+            { //more sources
+                m_mutex.lock();
+                m_playWhenFetched = false;
+                m_mutex.unlock();
+                debug() << "playing next source: " << nextSource;
+                slotPlayableUrlFetched( nextSource );
+            }
+            else if( m_media->queue().isEmpty() )
+                The::playlistActions()->requestNextTrack();
+        }
+
+        else if( m_media->queue().isEmpty() )
+            The::playlistActions()->requestNextTrack();
+    }
 
     stateChangedNotify( newState, oldState );
 
@@ -653,27 +777,31 @@ void
 EngineController::slotPlayableUrlFetched( const KUrl &url )
 {
     DEBUG_BLOCK
-
+    debug() << "Fetched url: " << url;
     if( url.isEmpty() )
     {
+        DEBUG_LINE_INFO
         The::playlistActions()->requestNextTrack();
         return;
     }
 
     if( !m_playWhenFetched )
     {
+        DEBUG_LINE_INFO
         m_mutex.lock();
         m_media->clearQueue();
         if( url.isLocalFile() )
             m_media->enqueue( url );
         m_nextTrack.clear();
         m_nextUrl = url;
+        debug() << "The next url we're playing is: " << m_nextUrl;
         // reset this flag each time
         m_playWhenFetched = true;
         m_mutex.unlock();
     }
     else
     {
+        DEBUG_LINE_INFO
         m_mutex.lock();
         playUrl( url, 0 );
         m_mutex.unlock();
