@@ -29,20 +29,17 @@
 #include "PaletteHandler.h"
 #include "ScriptManager.h"
 #include "SearchWidget.h"
-#include "Sidebar.h"
 #include "Sidebar.moc"
-#include "statusbar/StatusBar.h"
 #include "amarokconfig.h"
+#include "amarokurls/AmarokUrlHandler.h"
 #include "browsers/collectionbrowser/CollectionWidget.h"
 #include "browsers/filebrowser/FileBrowser.h"
 #include "browsers/playlistbrowser/PlaylistBrowser.h"
 #include "browsers/servicebrowser/ServiceBrowser.h"
-#include "services/ServicePluginManager.h"
-#include "services/scriptable/ScriptableService.h"
 #include "collection/CollectionManager.h"
 #include "context/ContextScene.h"
 #include "context/ContextView.h"
-#include "context/plasma/plasma.h"
+#include "context/ToolbarView.h"
 #include "covermanager/CoverManager.h" // for actions
 #include "playlist/PlaylistActions.h"
 #include "playlist/PlaylistController.h"
@@ -51,35 +48,41 @@
 #include "playlist/view/graphic/PlaylistGraphicsView.h"
 #include "playlistmanager/PlaylistFileProvider.h"
 #include "playlistmanager/PlaylistManager.h"
-#include "queuemanager/QueueManager.h"
+#include "services/ServicePluginManager.h"
+#include "services/scriptable/ScriptableService.h"
+#include "statusbar/StatusBar.h"
 #include "widgets/Splitter.h"
-#include "widgets/TrackTooltip.h"
 //#include "mediabrowser.h"
 
 #include <QCheckBox>
 #include <QDesktopWidget>
 #include <QList>
-#include <QVBoxLayout>
 #include <QSizeGrip>
+#include <QVBoxLayout>
 
-#include <kabstractfilewidget.h> //savePlaylist()
 #include <KAction>          //m_actionCollection
 #include <KActionCollection>
 #include <KApplication>     //kapp
 #include <KFileDialog>      //savePlaylist(), openPlaylist()
 #include <KInputDialog>     //slotAddStream()
 #include <KLocale>
-#include <KMessageBox>      //savePlaylist()
 #include <KMenu>
 #include <KMenuBar>
+#include <KMessageBox>      //savePlaylist()
 #include <KPixmapCache>
 #include <KStandardAction>
 #include <KWindowSystem>
+#include <kabstractfilewidget.h> //savePlaylist()
+
+#include <plasma/plasma.h>
 
 #ifdef Q_WS_X11
 #include <fixx11h.h>
 #endif
 
+#ifdef Q_WS_MAC
+#include "mac/GrowlInterface.h"
+#endif
 
 // Let people know OS X and Windows versions are still work-in-progress
 #if defined(Q_WS_MAC) || defined(Q_WS_WIN)
@@ -100,10 +103,10 @@ class ContextWidget : public KVBox
         }
 };
 
-MainWindow *MainWindow::s_instance = 0;
+QPointer<MainWindow> MainWindow::s_instance = 0;
 
 MainWindow::MainWindow()
-    : KXmlGuiWindow( 0 )
+    : KMainWindow( 0 )
     , EngineObserver( The::engineController() )
     , m_lastBrowser( 0 )
 {
@@ -114,6 +117,7 @@ MainWindow::MainWindow()
 
 #ifdef Q_WS_MAC
     QSizeGrip* grip = new QSizeGrip( this );
+    GrowlInterface* growl = new GrowlInterface( qApp->applicationName() );
 #endif
     //create this object now as we might run into issues if anyone tries to use it during initialization
     //make room for a full width statusbar at the bottom of everything
@@ -149,6 +153,8 @@ MainWindow::MainWindow()
 
     The::paletteHandler()->setPalette( palette() );
     setPlainCaption( i18n( AMAROK_CAPTION ) );
+
+    init();  // We could as well move the code from init() here, but meh.. getting a tad long
 }
 
 MainWindow::~MainWindow()
@@ -169,6 +175,11 @@ MainWindow::~MainWindow()
     delete m_playlistFiles;
     delete m_contextView;
     delete m_corona;
+    delete m_splitter;
+    delete The::statusBar();
+    delete m_controlBar;
+    delete The::svgHandler();
+    delete The::paletteHandler();
 }
 
 
@@ -180,13 +191,10 @@ MainWindow::~MainWindow()
 void
 MainWindow::init()
 {
-    layout()->setContentsMargins( 0, 0, 0, 0 );
-    layout()->setSpacing( 0 );
     DEBUG_BLOCK
 
-    //this function is necessary because Amarok::actionCollection() returns our actionCollection
-    //via the App::m_pMainWindow pointer since App::m_pMainWindow is not defined until
-    //the above ctor returns it causes a crash unless we do the initialisation in 2 stages.
+    layout()->setContentsMargins( 0, 0, 0, 0 );
+    layout()->setSpacing( 0 );
 
     QBoxLayout * toolbarSpacer = new QHBoxLayout( this );
     toolbarSpacer->setContentsMargins( 0, 0, 0, 10 );
@@ -215,6 +223,7 @@ MainWindow::init()
     m_contextWidget = new ContextWidget( this );
     PERF_LOG( "ContextWidget created" )
     m_contextWidget->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Maximum );
+    m_contextWidget->setSpacing( 0 );
     m_contextWidget->setFrameShape( QFrame::NoFrame );
     m_contextWidget->setFrameShadow( QFrame::Sunken );
     PERF_LOG( "Creating ContexScene" )
@@ -298,14 +307,12 @@ MainWindow::init()
 
         internetContentServiceBrowser->setScriptableServiceManager( The::scriptableServiceManager() );
         PERF_LOG( "ScriptableServiceManager done" )
-        
+
         #undef addBrowserMacro
         PERF_LOG( "finished MainWindow::init" )
     }
     //</Browsers>
 
-    TrackToolTip::instance(); //Instantiate
-
     if( AmarokConfig::panelsSavedState()[0] != -1 )
     {
         QByteArray sPanels;
@@ -315,7 +322,8 @@ MainWindow::init()
 
         m_splitter->restoreState( sPanels );
     }
-    
+
+    The::amarokUrlHandler(); //Instantiate
 }
 
 void
@@ -327,6 +335,9 @@ MainWindow::createContextView( Plasma::Containment *containment )
     PERF_LOG( "Creating ContexView" )
     m_contextView = new Context::ContextView( containment, m_corona, m_contextWidget );
     m_contextView->setFrameShape( QFrame::NoFrame );
+    m_contextToolbarView = new Context::ToolbarView( containment, m_corona, m_contextWidget );
+    m_contextToolbarView->setFrameShape( QFrame::NoFrame );
+    m_contextView->showHome();
     PERF_LOG( "ContexView created" )
 }
 
@@ -386,7 +397,7 @@ void
 MainWindow::keyPressEvent( QKeyEvent *e )
 {
     if( !( e->modifiers() & Qt::ControlModifier ) )
-        return KXmlGuiWindow::keyPressEvent( e );
+        return KMainWindow::keyPressEvent( e );
 
     int n = -1;
     switch( e->key() )
@@ -397,7 +408,7 @@ MainWindow::keyPressEvent( QKeyEvent *e )
         case Qt::Key_3: n = 3; break;
         case Qt::Key_4: n = 4; break;
         default:
-            return KXmlGuiWindow::keyPressEvent( e );
+            return KMainWindow::keyPressEvent( e );
     }
     if( n == 0 && m_browsers->currentIndex() >= 0 )
         m_browsers->showWidget( m_browsers->currentIndex() );
@@ -408,9 +419,13 @@ MainWindow::keyPressEvent( QKeyEvent *e )
 void
 MainWindow::closeEvent( QCloseEvent *e )
 {
+    DEBUG_BLOCK
+
 #ifdef Q_WS_MAC
+
     Q_UNUSED( e );
     hide();
+
 #else
 
     //KDE policy states we should hide to tray and not quit() when the
@@ -422,14 +437,15 @@ MainWindow::closeEvent( QCloseEvent *e )
                 i18n( "<qt>Closing the main window will keep Amarok running in the System Tray. "
                       "Use <B>Quit</B> from the menu, or the Amarok tray icon to exit the application.</qt>" ),
                 i18n( "Docking in System Tray" ), "hideOnCloseInfo" );
+
         hide();
         e->ignore();
+        return;
     }
-    else
-    {
-        e->accept();
-        kapp->quit();
-    }
+
+    e->accept();
+    kapp->quit();
+
 #endif
 }
 
@@ -442,6 +458,8 @@ MainWindow::sizeHint() const
 void
 MainWindow::exportPlaylist() const //SLOT
 {
+    DEBUG_BLOCK
+
     KFileDialog fileDialog( KUrl("kfiledialog:///amarok-playlist-export"), QString(), 0 );
     QCheckBox *saveRelativeCheck = new QCheckBox( i18n("Use relative path for &saving") );
 
@@ -465,11 +483,10 @@ void
 MainWindow::savePlaylist() const
 {
     DEBUG_BLOCK
-    //TODO make a nice dialog for choosing name and potentially parent group
-    //if( !playlistName.isEmpty() )
-
-    QString playlistName( i18n("Playlist") );
-    The::playlistModel()->savePlaylist( playlistName );
+    //TODO: inline rename
+    const QString name = KInputDialog::getText( i18n("Save playlist"),
+            i18n("Enter name for the new playlist:"), i18n("Playlist") );
+    The::playlistModel()->savePlaylist( name.trimmed() );
 }
 
 
@@ -497,7 +514,7 @@ MainWindow::slotAddLocation( bool directPlay ) //SLOT
     dlg.setMode( KFile::Files /*| KFile::Directory */); // Directory mode is fucked up - selects the parent dir
     dlg.exec();
     files = dlg.selectedUrls();
-    
+
     if( files.isEmpty() )
         return;
 
@@ -541,30 +558,12 @@ MainWindow::showScriptSelector() //SLOT
     ScriptManager::instance()->raise();
 }
 
-void
-MainWindow::showQueueManager() //SLOT
-{
-    if( QueueManagerNS::QueueManager::instance() )
-    {
-        QueueManagerNS::QueueManager::instance()->raise();
-        return;
-    }
-
-    QueueManagerNS::QueueManager dialog;
-    if( dialog.exec() == QDialog::Accepted )
-    {
-        // TODO: alter queue
-    }
-}
-
 /**
  * "Toggle Main Window" global shortcut connects to this slot
  */
 void
 MainWindow::showHide() //SLOT
 {
-    DEBUG_BLOCK
-
     setVisible( !isVisible() );
 }
 
@@ -607,7 +606,7 @@ MainWindow::isReallyShown() const
 void
 MainWindow::createActions()
 {
-    KActionCollection* const ac = actionCollection();
+    KActionCollection* const ac = Amarok::actionCollection();
     const EngineController* const ec = The::engineController();
     const Playlist::Actions* const pa = The::playlistActions();
     const Playlist::Controller* const pc = The::playlistController();
@@ -645,7 +644,7 @@ MainWindow::createActions()
     connect( action, SIGNAL( triggered(bool) ), SLOT( slotShowCoverManager() ) );
     ac->addAction( "cover_manager", action );
 
-    
+
 //     KAction *update_podcasts = new KAction( this );
 //     update_podcasts->setText( i18n( "Update Podcasts" ) );
 //     //update_podcasts->setIcon( KIcon("view-refresh-amarok") );
@@ -668,10 +667,6 @@ MainWindow::createActions()
     connect(action, SIGNAL(triggered(bool)), SLOT(showScriptSelector()));
     ac->addAction( "script_manager", action );
 
-    action = new KAction( KIcon( "go-bottom-amarok"), i18n( "Queue Manager" ), this );
-    connect(action, SIGNAL(triggered(bool)), SLOT(showQueueManager()));
-    ac->addAction( "queue_manager", action );
-
     action = new KAction( KIcon( "media-seek-forward-amarok" ), i18n("&Seek Forward"), this );
     ac->addAction( "seek_forward", action );
     action->setShortcut( Qt::Key_Right );
@@ -684,10 +679,6 @@ MainWindow::createActions()
     action->setGlobalShortcut( KShortcut( Qt::META + Qt::SHIFT + Qt::Key_Minus ) );
     connect(action, SIGNAL(triggered(bool)), ec, SLOT(seekBackward()));
 
-    action = new KAction( KIcon("view-statistics-amarok"), i18n( "Statistics" ), this );
-    connect(action, SIGNAL(triggered(bool)), SLOT(showStatistics()));
-    ac->addAction( "statistics", action );
-
     PERF_LOG( "MainWindow::createActions 6" )
     action = new KAction( KIcon("collection-refresh-amarok"), i18n( "Update Collection" ), this );
     connect(action, SIGNAL(triggered(bool)), CollectionManager::instance(), SLOT(checkCollectionChanges()));
@@ -732,7 +723,7 @@ MainWindow::createActions()
     action = new KAction( i18n( "Mute Volume" ), this );
     ac->addAction( "mute", action );
     action->setGlobalShortcut( KShortcut( Qt::META + Qt::Key_M ) );
-    connect( action, SIGNAL( triggered() ), ec, SLOT( mute() ) );
+    connect( action, SIGNAL( triggered() ), ec, SLOT( toggleMute() ) );
 
     action = new KAction( i18n( "Love Current Track" ), this );
     ac->addAction( "loveTrack", action );
@@ -783,6 +774,7 @@ MainWindow::createActions()
     new Amarok::RepeatAction( ac, this );
     new Amarok::RandomAction( ac, this );
     new Amarok::FavorAction( ac, this );
+    new Amarok::ReplayGainModeAction( ac, this );
 
     ac->addAssociatedWidget( this );
     foreach (QAction* action, ac->actions())
@@ -825,61 +817,61 @@ MainWindow::createMenus()
     actionsMenu = new KMenu( m_menubar );
     actionsMenu->setTitle( i18n("&Amarok") );
 #endif
-    actionsMenu->addAction( actionCollection()->action("playlist_playmedia") );
-    actionsMenu->addAction( actionCollection()->action("play_audiocd") );
+    actionsMenu->addAction( Amarok::actionCollection()->action("playlist_playmedia") );
+    actionsMenu->addAction( Amarok::actionCollection()->action("play_audiocd") );
     actionsMenu->addSeparator();
-    actionsMenu->addAction( actionCollection()->action("prev") );
-    actionsMenu->addAction( actionCollection()->action("play_pause") );
-    actionsMenu->addAction( actionCollection()->action("stop") );
-    actionsMenu->addAction( actionCollection()->action("next") );
+    actionsMenu->addAction( Amarok::actionCollection()->action("prev") );
+    actionsMenu->addAction( Amarok::actionCollection()->action("play_pause") );
+    actionsMenu->addAction( Amarok::actionCollection()->action("stop") );
+    actionsMenu->addAction( Amarok::actionCollection()->action("next") );
 
 
 #ifndef Q_WS_MAC    // Avoid duplicate "Quit" in OS X dock menu
     actionsMenu->addSeparator();
-    actionsMenu->addAction( actionCollection()->action(KStandardAction::name(KStandardAction::Quit)) );
+    actionsMenu->addAction( Amarok::actionCollection()->action(KStandardAction::name(KStandardAction::Quit)) );
 #endif
     //END Actions menu
 
     //BEGIN Playlist menu
     KMenu *playlistMenu = new KMenu( m_menubar );
     playlistMenu->setTitle( i18n("&Playlist") );
-    playlistMenu->addAction( actionCollection()->action("playlist_add") );
-    playlistMenu->addAction( actionCollection()->action("stream_add") );
-    playlistMenu->addAction( actionCollection()->action("playlist_save") );
-    playlistMenu->addAction( actionCollection()->action("playlist_burn") );
-    playlistMenu->addAction( actionCollection()->action("podcasts_update") );
+    playlistMenu->addAction( Amarok::actionCollection()->action("playlist_add") );
+    playlistMenu->addAction( Amarok::actionCollection()->action("stream_add") );
+    playlistMenu->addAction( Amarok::actionCollection()->action("playlist_save") );
+    playlistMenu->addAction( Amarok::actionCollection()->action("playlist_burn") );
+    playlistMenu->addAction( Amarok::actionCollection()->action("podcasts_update") );
     playlistMenu->addSeparator();
-    playlistMenu->addAction( actionCollection()->action("playlist_undo") );
-    playlistMenu->addAction( actionCollection()->action("playlist_redo") );
+    playlistMenu->addAction( Amarok::actionCollection()->action("playlist_undo") );
+    playlistMenu->addAction( Amarok::actionCollection()->action("playlist_redo") );
     playlistMenu->addSeparator();
-    playlistMenu->addAction( actionCollection()->action("playlist_clear") );
-    playlistMenu->addAction( actionCollection()->action("playlist_shuffle") );
+    playlistMenu->addAction( Amarok::actionCollection()->action("playlist_clear") );
+    playlistMenu->addAction( Amarok::actionCollection()->action("playlist_shuffle") );
 
-    QAction *repeat = actionCollection()->action("repeat");
+    QAction *repeat = Amarok::actionCollection()->action("repeat");
     playlistMenu->addAction( repeat );
 
-    KSelectAction *random = static_cast<KSelectAction*>( actionCollection()->action("random_mode") );
+    KSelectAction *random = static_cast<KSelectAction*>( Amarok::actionCollection()->action("random_mode") );
     playlistMenu->addAction( random );
     random->menu()->addSeparator();
-    random->menu()->addAction( actionCollection()->action("favor_tracks") );
+    random->menu()->addAction( Amarok::actionCollection()->action("favor_tracks") );
 
     playlistMenu->addSeparator();
     //FIXME: REENABLE When ported
-//     playlistMenu->addAction( actionCollection()->action("queue_selected") );
-    playlistMenu->addAction( actionCollection()->action("playlist_remove_duplicates") );
-    playlistMenu->addAction( actionCollection()->action("playlist_select_all") );
+//     playlistMenu->addAction( Amarok::actionCollection()->action("queue_selected") );
+    playlistMenu->addAction( Amarok::actionCollection()->action("playlist_remove_duplicates") );
+    playlistMenu->addAction( Amarok::actionCollection()->action("playlist_select_all") );
 
     //END Playlist menu
 
     //BEGIN Tools menu
     m_toolsMenu = new KMenu( m_menubar );
     m_toolsMenu->setTitle( i18n("&Tools") );
-    m_toolsMenu->addAction( actionCollection()->action("cover_manager") );
+    m_toolsMenu->addAction( Amarok::actionCollection()->action("cover_manager") );
 //FIXME: Reenable when ported//working
-//     m_toolsMenu->addAction( actionCollection()->action("queue_manager") );
-    m_toolsMenu->addAction( actionCollection()->action("script_manager") );
+//     m_toolsMenu->addAction( Amarok::actionCollection()->action("queue_manager") );
+    m_toolsMenu->addAction( Amarok::actionCollection()->action("script_manager") );
     m_toolsMenu->addSeparator();
-    m_toolsMenu->addAction( actionCollection()->action("update_collection") );
+    m_toolsMenu->addAction( Amarok::actionCollection()->action("update_collection") );
     //END Tools menu
 
     //BEGIN Settings menu
@@ -887,9 +879,18 @@ MainWindow::createMenus()
     m_settingsMenu->setTitle( i18n("&Settings") );
     //TODO use KStandardAction or KXmlGuiWindow
 
-    m_settingsMenu->addAction( actionCollection()->action(KStandardAction::name(KStandardAction::ConfigureToolbars)) );
-    m_settingsMenu->addAction( actionCollection()->action(KStandardAction::name(KStandardAction::KeyBindings)) );
-    m_settingsMenu->addAction( actionCollection()->action(KStandardAction::name(KStandardAction::Preferences)) );
+    // the phonon-coreaudio  backend has major issues with either the VolumeFaderEffect itself
+    // or with it in the pipeline. track playback stops every ~3-4 tracks, and on tracks >5min it
+    // stops at about 5:40. while we get this resolved upstream, don't make playing amarok such on osx.
+    // so we disable replaygain on osx
+#ifndef Q_WS_MAC
+    m_settingsMenu->addAction( Amarok::actionCollection()->action("replay_gain_mode") );
+    m_settingsMenu->addSeparator();
+#endif
+
+    m_settingsMenu->addAction( Amarok::actionCollection()->action(KStandardAction::name(KStandardAction::ConfigureToolbars)) );
+    m_settingsMenu->addAction( Amarok::actionCollection()->action(KStandardAction::name(KStandardAction::KeyBindings)) );
+    m_settingsMenu->addAction( Amarok::actionCollection()->action(KStandardAction::name(KStandardAction::Preferences)) );
     //END Settings menu
 
     m_menubar->addMenu( actionsMenu );
@@ -984,14 +985,29 @@ void MainWindow::engineStateChanged( Phonon::State state, Phonon::State oldState
 
 void MainWindow::metadataChanged( Meta::TrackPtr track )
 {
-    DEBUG_BLOCK
-
     setPlainCaption( i18n( "%1 - %2  ::  %3", track->artist() ? track->artist()->prettyName() : i18n( "Unknown" ), track->prettyName(), AMAROK_CAPTION ) );
 }
- 
+
+CollectionWidget * MainWindow::collectionBrowser()
+{
+    return qobject_cast<CollectionWidget *>( m_browsers->at( 0 ) );
+}
+
+QString MainWindow::activeBrowserName()
+{
+    return m_browserNames[ m_browsers->currentIndex() ];
+}
+
+PlaylistBrowserNS::PlaylistBrowser * MainWindow::playlistBrowser()
+{
+    return qobject_cast<PlaylistBrowserNS::PlaylistBrowser *>( m_browsers->at( 2 ) );
+}
+
 namespace The {
     MainWindow* mainWindow() { return MainWindow::s_instance; }
 }
 
 #include "MainWindow.moc"
 
+
+
