@@ -19,16 +19,21 @@
 #include "amarok_export.h"
 #include "meta/Meta.h"
 #include "EngineObserver.h"
+#include "Sidebar.h"
 
+#include <KMainWindow>
 #include <KVBox>
-#include <KXmlGuiWindow>
 
+#include <QPointer>
+
+class CollectionWidget;
 class ContextWidget;
 class MainToolbar;
 class MainWindow;
 class PlaylistFileProvider;
 class SearchWidget;
-class SideBar;
+
+namespace PlaylistBrowserNS { class PlaylistBrowser; }
 
 namespace Plasma { class Containment; }
 namespace Playlist { class Widget; }
@@ -36,6 +41,7 @@ namespace Playlist { class Widget; }
 namespace Context {
     class ContextScene;
     class ContextView;
+    class ToolbarView;
 }
 
 class KMenu;
@@ -53,7 +59,7 @@ namespace The {
   *
   * This is the main window widget.
   */
-class AMAROK_EXPORT MainWindow : public KXmlGuiWindow, public EngineObserver, public Meta::Observer
+class AMAROK_EXPORT MainWindow : public KMainWindow, public EngineObserver, public Meta::Observer
 {
     friend MainWindow* The::mainWindow();
 
@@ -63,8 +69,6 @@ class AMAROK_EXPORT MainWindow : public KXmlGuiWindow, public EngineObserver, pu
         MainWindow();
        ~MainWindow();
 
-        void init();
-
         //allows us to switch browsers from within other browsers etc
         void showBrowser( const QString& name );
         void addBrowser( const QString &name, QWidget *widget, const QString &text, const QString &icon );
@@ -75,10 +79,16 @@ class AMAROK_EXPORT MainWindow : public KXmlGuiWindow, public EngineObserver, pu
         void activate();
 
         SideBar *sideBar() const { return m_browsers; }
-        KMenu   *ToolsMenu() const { return m_toolsMenu; }
-        KMenu   *SettingsMenu() const { return m_settingsMenu; }
+        QPointer<KMenu> ToolsMenu() const { return m_toolsMenu; }
+        QPointer<KMenu> SettingsMenu() const { return m_settingsMenu; }
+		QPointer<Playlist::Widget> playlistWidget() { return m_playlistWidget; }
         void deleteBrowsers();
 
+        QString activeBrowserName();
+
+        CollectionWidget * collectionBrowser();
+        PlaylistBrowserNS::PlaylistBrowser * playlistBrowser();
+
         //will return the size of the rect defined top, right and left by the main toolbar and bottom by the context view.
         QSize backgroundSize();
 
@@ -110,7 +120,6 @@ class AMAROK_EXPORT MainWindow : public KXmlGuiWindow, public EngineObserver, pu
         void slotPlayMedia();
         void slotAddLocation( bool directPlay = false );
         void slotAddStream();
-        void showQueueManager();
         void showScriptSelector();
 
     protected:
@@ -129,28 +138,30 @@ class AMAROK_EXPORT MainWindow : public KXmlGuiWindow, public EngineObserver, pu
         void setRating5() { setRating( 5 ); }
 
     private:
+        void init();
         void setRating( int n );
         void showBrowser( const int index );
 
-        QMenuBar      *m_menubar;
-        KMenu         *m_toolsMenu;
-        KMenu         *m_settingsMenu;
-        SideBar       *m_browsers;
-        QStringList    m_browserNames;
-        KMenu         *m_searchMenu;
+        QPointer<QMenuBar>  m_menubar;
+        QPointer<KMenu>     m_toolsMenu;
+        QPointer<KMenu>     m_settingsMenu;
+        QPointer<SideBar>   m_browsers;
+        QStringList         m_browserNames;
+        QPointer<KMenu>     m_searchMenu;
+        QPointer<KVBox>     m_statusbarArea;
 
-        KVBox *m_statusbarArea;
+        QPointer<SearchWidget>     m_searchWidget;
+        QPointer<MainToolbar>      m_controlBar;
+        QPointer<Playlist::Widget> m_playlistWidget;
+        QPointer<QTimer>           m_timer;  //search filter timer
+        QPointer<QSplitter>        m_splitter;
 
-        SearchWidget     *m_searchWidget;
-        MainToolbar      *m_controlBar;
-        Playlist::Widget *m_playlistWidget;
-        QTimer           *m_timer;  //search filter timer
-        QSplitter        *m_splitter;
-        QByteArray        m_splitterState;
+        QByteArray                 m_splitterState;
 
-        ContextWidget *m_contextWidget;
-        Context::ContextScene *m_corona;
-        Context::ContextView *m_contextView;
+        QPointer<ContextWidget>         m_contextWidget;
+        QPointer<Context::ContextScene> m_corona;
+        QPointer<Context::ContextView>  m_contextView;
+        QPointer<Context::ToolbarView>  m_contextToolbarView;
 
         PlaylistFileProvider *m_playlistFiles;
         Meta::TrackPtr m_currentTrack;
@@ -160,7 +171,7 @@ class AMAROK_EXPORT MainWindow : public KXmlGuiWindow, public EngineObserver, pu
         int     m_lastBrowser;
         int     m_searchField;
 
-        static MainWindow *s_instance;
+        static QPointer<MainWindow> s_instance;
 
     private slots:
         void createContextView( Plasma::Containment *c );
