@@ -36,7 +36,7 @@
 #include "PluginManager.h"
 #include "AmarokProcess.h"
 #include "ScriptManager.h"
-#include "StatusBar.h"
+#include "statusbar/StatusBar.h"
 
 #include <KIO/Job>
 #include <KMessageBox>
@@ -402,7 +402,7 @@ MediaDevice::kioCopyTrack( const KUrl &src, const KUrl &dst )
                           src.prettyUrl(),
                           dst.prettyUrl()
                         ),
-                    KDE::StatusBar::Error );
+                    StatusBar::Error );
         }
         else
         {
@@ -413,7 +413,7 @@ MediaDevice::kioCopyTrack( const KUrl &src, const KUrl &dst )
                 // probably s.th. went wrong
                 The::statusBar()->longMessage(
                         i18n( "Media Device: Reading tags from %1 failed", dst.prettyUrl() ),
-                        KDE::StatusBar::Error );
+                        StatusBar::Error );
             }
         }
     }
@@ -493,7 +493,7 @@ MediaDevice::connectDevice( bool silent )
             {
                 The::statusBar()->longMessage(
                         i18n( "Failed to purge podcasts already played" ),
-                        KDE::StatusBar::Sorry );
+                        StatusBar::Sorry );
             }
             else if( numDeleted > 0 )
             {
@@ -560,7 +560,7 @@ MediaDevice::disconnectDevice( bool postDisconnectHook )
     {
         The::statusBar()->longMessage(
                 i18n( "Post-disconnect command failed, before removing device, please make sure that it is safe to do so." ),
-                KDE::StatusBar::Information );
+                StatusBar::Information );
         result = false;
     }
     else
@@ -786,7 +786,7 @@ MediaDevice::transferFiles()
             {
                 The::statusBar()->shortMessage( i18n( "Track already on media device: %1",
                                                                            track->prettyUrl() ),
-                        KDE::StatusBar::Sorry );
+                        StatusBar::Sorry );
                 existing += track->url();
                 setProgress( progress() + 1 );
                 continue;
@@ -831,7 +831,7 @@ MediaDevice::transferFiles()
                 if( !isPlayable( track ) )
                 {
                     The::statusBar()->shortMessage( i18n( "Track not playable on media device: %1", track->prettyUrl() ),
-                            KDE::StatusBar::Sorry );
+                            StatusBar::Sorry );
                     unplayable += (*it)->url();
                     transferredItem->setFailed();
                     setProgress( progress() + 1 );
@@ -846,7 +846,7 @@ MediaDevice::transferFiles()
                 {
                     The::statusBar()->longMessage(
                             i18n( "Failed to copy track to media device: %1", track->url() ),
-                            KDE::StatusBar::Sorry );
+                            StatusBar::Sorry );
                     transferredItem->setFailed();
                 }
             }
@@ -943,7 +943,7 @@ MediaDevice::transferFiles()
         QString longMsg = i18n( "The following tracks were not transferred: ");
         for( KUrl::List::ConstIterator it = existing.constBegin(), end = existing.constEnd();
                 it != end;
-                it++ )
+                ++it )
         {
             longMsg += "<br>" + (*it).prettyUrl();
         }
@@ -953,11 +953,11 @@ MediaDevice::transferFiles()
         {
             longMsg += "<br>" + (*it).prettyUrl();
         }
-        The::statusBar()->shortLongMessage( msg, longMsg, KDE::StatusBar::Sorry );
+        The::statusBar()->longMessage( longMsg, StatusBar::Sorry );
     }
     else if( !msg.isEmpty() )
     {
-        The::statusBar()->shortMessage( msg, KDE::StatusBar::Sorry );
+        The::statusBar()->shortMessage( msg, StatusBar::Sorry );
     }
 
     m_parent->updateButtons();
@@ -1150,7 +1150,7 @@ MediaDevice::isPlayable( const Meta::TrackPtr track )
     if( supportedFiletypes().isEmpty() )
         return true;
 
-    QString type = track->url().section( ".", -1 ).toLower();
+    QString type = track->url().section( '.', -1 ).toLower();
     return supportedFiletypes().contains( type );
 }
 
@@ -1163,7 +1163,7 @@ MediaDevice::isPreferredFormat( const Meta::TrackPtr track )
     if( supportedFiletypes().isEmpty() )
         return true;
 
-    QString type = track->url().section( ".", -1 ).toLower();
+    QString type = track->url().section( '.', -1 ).toLower();
     return ( type == supportedFiletypes().first() );
 }
 
