@@ -26,7 +26,7 @@ class QAction;
 class QMenu;
 
 class SearchWidget;
-class CollectionTreeView;
+class CollectionBrowserTreeView;
 
 class CollectionWidget : public KVBox
 {
@@ -34,30 +34,42 @@ class CollectionWidget : public KVBox
     public:
         CollectionWidget( const char* name , QWidget *parent );
         static CollectionWidget *instance() { return s_instance; }
-        CollectionTreeView *view() const { return m_treeView; }
+        CollectionBrowserTreeView *view() const { return m_treeView; }
+
+        /**
+         * Apply a filter to the tree view.
+         * @param filter The filter to apply.
+         */
+        void setFilter( const QString &filter );
+        QString filter() const;
+
+        QList<int> levels();
+
     public slots:
-        //void slotSetFilter();
 
-    private slots:
         void customFilter( QAction * );
+        void sortByAlbum();
+        void sortByArtist();
         void sortByArtistAlbum();
         void sortByGenreArtist();
         void sortByGenreArtistAlbum();
         void slotShowYears( bool checked );
+        void slotShowCovers( bool checked );
 
     private:
         SearchWidget        *m_searchWidget;
-        CollectionTreeView  *m_treeView;
-        
+        CollectionBrowserTreeView  *m_treeView;
+
         QAction             *m_firstLevelSelectedAction;
         QAction             *m_secondLevelSelectedAction;
         QAction             *m_thirdLevelSelectedAction;
-        
+
         QMenu               *m_firstLevel;
         QMenu               *m_secondLevel;
         QMenu               *m_thirdLevel;
-        
+
         QList<int>          m_levels;
+
         static CollectionWidget *s_instance;
 };
 
