@@ -61,10 +61,11 @@ FileBrowser::Widget::Widget( const char * name , QWidget *parent )
     m_filePlacesModel = new KFilePlacesModel( this );
     m_urlNav = new KUrlNavigator( m_filePlacesModel, KUrl( QDir::home().path() ), this );
 
-    setContentsMargins( 0, 0, 0, 0 );
+    m_dirOperator = new MyDirOperator( QDir::home().path(), this );
 
-    setFrameShape( QFrame::StyledPanel );
-    setFrameShadow( QFrame::Sunken );
+    connect( m_dirOperator, SIGNAL( viewChanged( QAbstractItemView * ) ), this, SLOT( selectorViewChanged( QAbstractItemView * ) ) );
+    setStretchFactor( m_dirOperator, 2 );
+    m_dirOperator->setSizePolicy( QSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding ) );
 
     KHBox* filterBox = new KHBox( this );
     m_filterButton = new QToolButton( filterBox );
@@ -79,25 +80,13 @@ FileBrowser::Widget::Widget( const char * name , QWidget *parent )
     connect( m_filter, SIGNAL( editTextChanged( const QString& ) ), SLOT( slotFilterChange( const QString& ) ) );
     connect( m_filter, SIGNAL( returnPressed( const QString& ) ), m_filter, SLOT( addToHistory( const QString& ) ) );
 
-    m_dirOperator = new MyDirOperator( QDir::home().path(), this );
-
-    QPalette p = m_dirOperator->palette();
-    QColor c = p.color( QPalette::Base );
-    c.setAlpha( 0 );
-    p.setColor( QPalette::Base, c );
-    m_dirOperator->setPalette( p );
-
-    connect( m_dirOperator, SIGNAL( viewChanged( QAbstractItemView * ) ), this, SLOT( selectorViewChanged( QAbstractItemView * ) ) );
-    setStretchFactor( m_dirOperator, 2 );
-    m_dirOperator->setSizePolicy( QSizePolicy( QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding ) );
-
     KActionCollection *coll = m_dirOperator->actionCollection();
     // some shortcuts of diroperator that clash with Kate
     coll->action( "delete" )->setShortcut( Qt::ALT + Qt::Key_Delete );
 
     // bookmarks action!
     KActionMenu *acmBookmarks = new KActionMenu( KIcon( "bookmarks" ), i18n( "Bookmarks" ), this );
-    // m_actionCollection->addAction( "bookmarks", acmBookmarks ); // TODO DISABLED FOR 2.0 final because of crashes
+    m_actionCollection->addAction( "bookmarks", acmBookmarks );
     acmBookmarks->setDelayed( false );
     m_bookmarkHandler = new KBookmarkHandler( this, acmBookmarks->menu() );
 
@@ -109,7 +98,7 @@ FileBrowser::Widget::Widget( const char * name , QWidget *parent )
     connect( m_dirOperator, SIGNAL( urlEntered( const KUrl& ) ), this, SLOT( dirUrlEntered( const KUrl& ) ) );
 
     // Connect the bookmark handler
-    connect( m_bookmarkHandler, SIGNAL( openUrl( const QString& ) ), this, SLOT( setDir( const QString& ) ) );
+    connect( m_bookmarkHandler, SIGNAL( openUrl( const KUrl& ) ), this, SLOT( setDir( const KUrl& ) ) );
 
     waitingUrl.clear();
 
@@ -149,16 +138,17 @@ void FileBrowser::Widget::readConfig()
     KConfigGroup config = Amarok::config( "File Browser" );
 
     m_filter->setMaxCount( config.readEntry( "Filter History Length", 9 ) );
+
     setDir( config.readEntry( "Current Directory" ) );
 
     // KDirOperator view configuration:
-    m_dirOperator->setView( config.readEntry( "View Style" ) == "Detail" ? KFile::Detail : KFile::Simple );
-    m_dirOperator->view()->setSelectionMode( QAbstractItemView::ExtendedSelection );
-    m_dirOperator->view()->setContentsMargins( 0, 0, 0, 0 );
-    m_dirOperator->view()->setFrameShape( QFrame::NoFrame );
+    m_dirOperator->setView( config.readEntry( "View Style", "Simple" ) == "Detail" ? KFile::Detail : KFile::Simple );
     m_dirOperator->view()->setSelectionMode( QAbstractItemView::ExtendedSelection );
+    if( config.hasKey( "Sorting" ) )
+        m_dirOperator->setSorting( static_cast<QDir::SortFlags>( config.readEntry( "Sorting" ).toInt() ) );
 
     m_filter->setHistoryItems( config.readEntry( "Filter History", QStringList() ), true );
+
     lastFilter = config.readEntry( "Last Filter" );
 }
 
@@ -170,6 +160,7 @@ void FileBrowser::Widget::writeConfig()
     KConfigGroup config = Amarok::config( "File Browser" );
 
     config.writeEntry( "Current Directory", m_dirOperator->url() );
+    config.writeEntry( "Sorting", QString::number( static_cast<QDir::SortFlags>( m_dirOperator->sorting() ) ) );
     config.writeEntry( "Filter History Length", m_filter->maxCount() );
     config.writeEntry( "Filter History", m_filter->historyItems() );
     config.writeEntry( "Last Filter", lastFilter );
@@ -221,7 +212,8 @@ void FileBrowser::Widget::slotFilterChange( const QString & nf )
     }
     else
     {
-        f = '*' + f + '*'; // add regexp matches surrounding the filter
+        if ( !f.startsWith( '*' ) && !f.endsWith( '*' ) )
+            f = '*' + f + '*'; // add regexp matches surrounding the filter
         m_dirOperator->setNameFilter( f );
         lastFilter = f;
         m_filterButton->setToolTip( i18n( "Clear filter" ) );
@@ -326,8 +318,5 @@ bool FileBrowser::Widget::eventFilter( QObject* o, QEvent *e )
 
 //END Protected
 
-
-// kate: space-indent on; indent-width 4; replace-tabs on;
-
 #include "FileBrowser.moc"
 
