@@ -22,9 +22,8 @@
 #include "CollectionManager.h"
 #include "PaletteHandler.h"
 #include "playlist/PlaylistModel.h"
+#include "PlaylistsInGroupsProxy.h"
 #include "context/popupdropper/libpud/PopupDropperAction.h"
-#include "SqlPlaylist.h"
-#include "SqlPlaylistGroup.h"
 #include "SvgHandler.h"
 #include "statusbar/StatusBar.h"
 #include "UserPlaylistModel.h"
@@ -43,30 +42,26 @@
 #include <typeinfo>
 
 PlaylistBrowserNS::PlaylistCategory::PlaylistCategory( QWidget * parent )
-    : Amarok::Widget( parent )
+    : QWidget( parent )
 {
     setContentsMargins(0,0,0,0);
     m_toolBar = new QToolBar( this );
     m_toolBar->setToolButtonStyle( Qt::ToolButtonTextBesideIcon );
 
-    m_playlistView = new UserPlaylistTreeView( this );
+
+    m_groupedProxy = new PlaylistsInGroupsProxy( The::userPlaylistModel() );
+
+    m_playlistView = new UserPlaylistTreeView( m_groupedProxy, this );
     m_playlistView->setFrameShape( QFrame::NoFrame );
     m_playlistView->setContentsMargins(0,0,0,0);
-    m_playlistView->setModel( PlaylistBrowserNS::UserModel::instance() );
     m_playlistView->header()->hide();
 
-    //m_playlistView->setSelectionMode(QAbstractItemView::SingleSelection);
     m_playlistView->setDragEnabled(true);
     m_playlistView->setAcceptDrops(true);
     m_playlistView->setDropIndicatorShown(true);
 
     m_playlistView->setEditTriggers( QAbstractItemView::NoEditTriggers );
 
-    //connect( m_playlistView, SIGNAL( activated( const QModelIndex & ) ), this, SLOT( itemActivated(  const QModelIndex & ) ) );
-    //connect( m_playlistView, SIGNAL( customContextMenuRequested( const QPoint & ) ), this, SLOT( showContextMenu( const QPoint & ) ) );
-
-    connect( PlaylistBrowserNS::UserModel::instance(), SIGNAL( editIndex( const QModelIndex & ) ), m_playlistView, SLOT( edit( const QModelIndex & ) ) );
-
     connect( The::paletteHandler(), SIGNAL( newPalette( const QPalette & ) ), SLOT( newPalette( const QPalette & ) ) );
 
     QVBoxLayout *vLayout = new QVBoxLayout( this );
@@ -78,13 +73,13 @@ PlaylistBrowserNS::PlaylistCategory::PlaylistCategory( QWidget * parent )
 
     m_addGroupAction = new KAction( KIcon("media-track-add-amarok" ), i18n( "Add Folder" ), this  );
     m_toolBar->addAction( m_addGroupAction );
-    connect( m_addGroupAction, SIGNAL( triggered( bool ) ), PlaylistBrowserNS::UserModel::instance(), SLOT( createNewGroup() ) );
+//TODO:     connect( m_addGroupAction, SIGNAL( triggered( bool ) ), m_groupedProxy, SLOT( createNewGroup() ) );
 
     m_playlistView->setNewGroupAction( m_addGroupAction );
 
-    KAction* addStreamAction = new KAction( KIcon("list-add"), i18n("Add Stream"), this );
-    m_toolBar->addAction( addStreamAction );
-    connect( addStreamAction, SIGNAL( triggered( bool ) ), this, SLOT( showAddStreamDialog() ) );
+//     KAction* addStreamAction = new KAction( KIcon("list-add"), i18n("Add Stream"), this );
+//     m_toolBar->addAction( addStreamAction );
+//     connect( addStreamAction, SIGNAL( triggered( bool ) ), this, SLOT( showAddStreamDialog() ) );
 }
 
 
@@ -92,14 +87,14 @@ PlaylistBrowserNS::PlaylistCategory::~PlaylistCategory()
 {
 }
 
-
+/*
 void
 PlaylistBrowserNS::PlaylistCategory::showAddStreamDialog()
 {
-    KDialog *dialog = new PlaylistBrowserNS::StreamEditor( this );
+    KDialog *dialog = new PlaylistBrowserNS::treamEditor( this );
     connect( dialog, SIGNAL( okClicked() ), this, SLOT( streamDialogConfirmed() ) );
-}
-
+}*/
+/*
 void
 PlaylistBrowserNS::PlaylistCategory::streamDialogConfirmed()
 {
@@ -109,13 +104,14 @@ PlaylistBrowserNS::PlaylistCategory::streamDialogConfirmed()
     Meta::TrackPtr track = CollectionManager::instance()->trackForUrl(  dialog->streamUrl() );
     if( !track.isNull() )
     {
-        PlaylistBrowserNS::UserModel::instance()->createNewStream(  dialog->streamName(), track );
+        m_model->createNewStream(  dialog->streamName(), track );
     }
     else
     {
         The::statusBar()->longMessage( i18n("The stream URL provided was not valid.") );
     }
 }
+*/
 
 PlaylistBrowserNS::StreamEditor::StreamEditor( QWidget* parent )
     : KDialog( parent )
