@@ -22,16 +22,49 @@
 #include "Debug.h"
 #include "PodcastMeta.h"
 #include "PodcastProvider.h"
+#include "context/popupdropper/libpud/PopupDropperAction.h"
+#include "context/popupdropper/libpud/PopupDropperItem.h"
+#include "context/popupdropper/libpud/PopupDropper.h"
+#include "PodcastCategory.h"
 #include "playlistmanager/PlaylistManager.h"
+#include "SvgHandler.h"
 
 #include <QInputDialog>
 #include <KIcon>
 #include <QListIterator>
 #include <typeinfo>
 
+namespace The
+{
+    PlaylistBrowserNS::PodcastModel* podcastModel()
+    {
+        return PlaylistBrowserNS::PodcastModel::instance();
+    }
+}
+
+PlaylistBrowserNS::PodcastModel* PlaylistBrowserNS::PodcastModel::s_instance = 0;
+
+PlaylistBrowserNS::PodcastModel*
+PlaylistBrowserNS::PodcastModel::instance()
+{
+    return s_instance ? s_instance : new PodcastModel();
+}
+
+void
+PlaylistBrowserNS::PodcastModel::destroy()
+{
+    if (s_instance) {
+        delete s_instance;
+        s_instance = 0;
+    }
+}
+
 PlaylistBrowserNS::PodcastModel::PodcastModel()
  : QAbstractItemModel()
+ , m_appendAction( 0 )
+ , m_loadAction( 0 )
 {
+    s_instance = this;
     QList<Meta::PlaylistPtr> playlists =
             The::playlistManager()->playlistsOfCategory( PlaylistManager::PodcastChannel );
     QListIterator<Meta::PlaylistPtr> i(playlists);
@@ -41,7 +74,6 @@ PlaylistBrowserNS::PodcastModel::PodcastModel()
     connect( The::playlistManager(), SIGNAL(updated()), SLOT(slotUpdate()));
 }
 
-
 PlaylistBrowserNS::PodcastModel::~PodcastModel()
 {
 }
@@ -51,8 +83,9 @@ PlaylistBrowserNS::PodcastModel::data(const QModelIndex & index, int role) const
 {
 
     if ( !index.isValid() ||
-         ( role != Qt::DisplayRole && role != Qt::DecorationRole &&
-           role != ShortDescriptionRole && role != OnDiskRole ) )
+         ( role != Qt::DisplayRole && role != Qt::DecorationRole
+           && role != Qt::ToolTipRole
+           && role != ShortDescriptionRole && role != OnDiskRole ) )
     {
         return QVariant();
     }
@@ -90,12 +123,14 @@ PlaylistBrowserNS::PodcastModel::data(const QModelIndex & index, int role) const
         return QVariant();
     }
 
-    if ( role == Qt::DisplayRole )
-        return title;
-    else if ( role == Qt::DecorationRole )
+    if( role == Qt::DisplayRole )
+        return title.simplified(); //whitespace removed from start and end + extra whitespace in between
+    else if( role == Qt::DecorationRole )
         return QVariant( icon );
-    else if ( role == OnDiskRole )
+    else if( role == OnDiskRole )
         return QVariant( isOnDisk );
+    else if( role == Qt::ToolTipRole )
+        return title;
 
     // At this point role can only be ShortDescriptionRole
     return description;
@@ -165,6 +200,9 @@ PlaylistBrowserNS::PodcastModel::parent(const QModelIndex & index) const
 
     Meta::PodcastMetaCommon *podcastMetaCommon = static_cast<Meta::PodcastMetaCommon *>(index.internalPointer());
 
+    if ( !podcastMetaCommon )
+        return QModelIndex();
+
     if ( podcastMetaCommon->podcastType() ==  Meta::ChannelType )
     {
         return QModelIndex();
@@ -360,6 +398,8 @@ PlaylistBrowserNS::PodcastModel::addPodcast()
 {
     debug() << "adding Podcast";
 
+    //TODO: request the user to which PodcastProvider he wants to add it in case
+    // of multiple (enabled) Podcast Providers.
     PodcastProvider * podcastProvider = The::playlistManager()->defaultPodcasts();
     if( podcastProvider )
     {
@@ -386,7 +426,7 @@ PlaylistBrowserNS::PodcastModel::addPodcast()
 }
 
 void
-PlaylistBrowserNS::PodcastModel::loadItems(QModelIndexList list, Playlist::AddOptions insertMode)
+PlaylistBrowserNS::PodcastModel::loadItems( QModelIndexList list, Playlist::AddOptions insertMode )
 {
     Meta::TrackList episodes;
     Meta::PlaylistList channels;
@@ -446,8 +486,7 @@ PlaylistBrowserNS::PodcastModel::removeSubscription( Meta::PodcastChannelPtr cha
 {
     debug() << "remove Podcast subscription for " << channel->title();
     //HACK: since we only have one PodcastProvider implementation
-    PlaylistProvider *provider = The::playlistManager()->playlistProvider(
-            PlaylistManager::PodcastChannel, i18n( "Local Podcasts" ) );
+    PlaylistProvider *provider = The::playlistManager()->defaultPodcasts();
     if( provider )
     {
         PodcastProvider * podcastProvider = static_cast<PodcastProvider *>(provider);
@@ -473,8 +512,7 @@ PlaylistBrowserNS::PodcastModel::refreshPodcast( Meta::PodcastChannelPtr channel
 {
     debug() << "refresh Podcast " << channel->title();
     //HACK: since we only have one PodcastProvider implementation
-    PlaylistProvider *provider = The::playlistManager()->playlistProvider(
-            PlaylistManager::PodcastChannel, i18n( "Local Podcasts" ) );
+    PlaylistProvider *provider = The::playlistManager()->defaultPodcasts();
     if( provider )
     {
         PodcastProvider * podcastProvider = static_cast<PodcastProvider *>(provider);
@@ -496,7 +534,12 @@ PlaylistBrowserNS::PodcastModel::downloadItems( QModelIndexList list )
         Meta::PodcastMetaCommon *pmc = static_cast<Meta::PodcastMetaCommon *>(index.internalPointer());
         if( pmc->podcastType() ==  Meta::EpisodeType )
         {
-            downloadEpisode( Meta::PodcastEpisodePtr( reinterpret_cast<Meta::PodcastEpisode *>(pmc) ) );
+            Meta::PodcastEpisodePtr episode
+                = Meta::PodcastEpisodePtr( dynamic_cast<Meta::PodcastEpisode *>(pmc) );
+            if( episode.isNull() )
+                debug() << "could not downcast PodcastMetaCommon pointer!";
+            else
+                downloadEpisode( episode );
         }
         else if( pmc->podcastType() ==  Meta::ChannelType )
         {
@@ -530,8 +573,7 @@ PlaylistBrowserNS::PodcastModel::downloadEpisode( Meta::PodcastEpisodePtr episod
     DEBUG_BLOCK
     debug() << "downloading " << episode->title();
     //HACK: since we only have one PodcastProvider implementation
-    PlaylistProvider *provider = The::playlistManager()->playlistProvider(
-            PlaylistManager::PodcastChannel, i18n( "Local Podcasts" ) );
+    PlaylistProvider *provider = The::playlistManager()->defaultPodcasts();
     if( provider )
     {
         PodcastProvider * podcastProvider = static_cast<PodcastProvider *>(provider);
@@ -565,8 +607,6 @@ PlaylistBrowserNS::PodcastModel::deleteDownloadedEpisode( Meta::PodcastEpisodePt
 void
 PlaylistBrowserNS::PodcastModel::configureChannels( QModelIndexList list )
 {
-    DEBUG_BLOCK
-    debug() << "number of items: " << list.count();
     foreach( const QModelIndex &index, list )
     {
         Meta::PodcastMetaCommon *pmc = static_cast<Meta::PodcastMetaCommon *>(index.internalPointer());
@@ -584,8 +624,7 @@ PlaylistBrowserNS::PodcastModel::configureChannel( Meta::PodcastChannelPtr chann
     DEBUG_BLOCK
     debug() << "configuring " << channel->title();
     //HACK: since we only have one PodcastProvider implementation
-    PlaylistProvider *provider = The::playlistManager()->playlistProvider(
-            PlaylistManager::PodcastChannel, i18n( "Local Podcasts" ) );
+    PlaylistProvider *provider = The::playlistManager()->defaultPodcasts();
     if( provider )
     {
         PodcastProvider * podcastProvider = static_cast<PodcastProvider *>(provider);
@@ -610,4 +649,145 @@ PlaylistBrowserNS::PodcastModel::emitLayoutChanged()
     emit( layoutChanged() );
 }
 
+QList<PopupDropperAction *>
+PlaylistBrowserNS::PodcastModel::actionsFor( const QModelIndexList &indices )
+{
+    QList<PopupDropperAction *> actions;
+
+    m_selectedEpisodes.clear();
+    m_selectedChannels.clear();
+    m_selectedEpisodes << selectedEpisodes( indices );
+    m_selectedChannels << selectedChannels( indices );
+
+    if( indices.isEmpty() )
+        return actions;
+
+    actions << createCommonActions( indices );
+
+    //HACK: since we only have one PodcastProvider implementation
+    PodcastProvider *provider = The::playlistManager()->defaultPodcasts();
+    if( provider )
+    {
+        if( !m_selectedEpisodes.isEmpty() )
+            actions << provider->episodeActions( m_selectedEpisodes );
+        if( !m_selectedChannels.isEmpty() )
+            actions << provider->channelActions( m_selectedChannels );
+    }
+
+    return actions;
+}
+
+QList< PopupDropperAction * >
+PlaylistBrowserNS::PodcastModel::createCommonActions( QModelIndexList indices )
+{
+    Q_UNUSED( indices )
+    QList< PopupDropperAction * > actions;
+
+    if( m_appendAction == 0 )
+    {
+        m_appendAction = new PopupDropperAction(
+            The::svgHandler()->getRenderer( "amarok/images/pud_items.svg" ),
+            "append",
+            KIcon( "media-track-add-amarok" ),
+            i18n( "&Append to Playlist" ),
+            this
+        );
+        connect( m_appendAction, SIGNAL( triggered() ), this, SLOT( slotAppend() ) );
+    }
+
+    if( m_loadAction == 0 )
+    {
+        m_loadAction = new PopupDropperAction(
+            The::svgHandler()->getRenderer( "amarok/images/pud_items.svg" ),
+            "load",
+            KIcon( "folder-open" ),
+            i18nc( "Replace the currently loaded tracks with these",
+            "&Load" ),
+            this
+        );
+        connect( m_loadAction, SIGNAL( triggered() ), this, SLOT( slotLoad() ) );
+    }
+
+    actions << m_appendAction;
+    actions << m_loadAction;
+
+    return actions;
+}
+
+Meta::PodcastChannelList
+PlaylistBrowserNS::PodcastModel::selectedChannels( const QModelIndexList &indices )
+{
+    Meta::PodcastChannelList channels;
+    Meta::PodcastMetaCommon *pmc = 0;
+    foreach( const QModelIndex &index, indices )
+    {
+        if( !index.isValid() )
+            break;
+
+        pmc = static_cast<Meta::PodcastMetaCommon *>(index.internalPointer());
+        if( !pmc )
+            break;
+
+        switch( pmc->podcastType() )
+        {
+            case Meta::EpisodeType:
+                break;
+            case Meta::ChannelType:
+                channels << Meta::PodcastChannelPtr( static_cast<Meta::PodcastChannel *>(pmc) );
+                break;
+        }
+    }
+    return channels;
+}
+
+Meta::PodcastEpisodeList
+PlaylistBrowserNS::PodcastModel::selectedEpisodes( const QModelIndexList &indices )
+{
+    Meta::PodcastEpisodeList episodes;
+    Meta::PodcastMetaCommon *pmc = 0;
+    foreach( const QModelIndex &index, indices )
+    {
+        if( !index.isValid() )
+            break;
+
+        pmc = static_cast<Meta::PodcastMetaCommon *>(index.internalPointer());
+        switch( pmc->podcastType() )
+        {
+            case Meta::EpisodeType:
+                episodes << Meta::PodcastEpisodePtr( static_cast<Meta::PodcastEpisode *>(pmc) );
+                break;
+            case Meta::ChannelType:
+                episodes << static_cast<Meta::PodcastChannel *>(pmc)->episodes();
+                break;
+        }
+    }
+    return episodes;
+}
+
+void
+PlaylistBrowserNS::PodcastModel::slotAppend()
+{
+    Meta::PodcastEpisodeList episodes = selectedEpisodes();
+    if( !episodes.empty() )
+        The::playlistController()->insertOptioned(
+                podcastEpisodesToTracks( episodes ), Playlist::Append );
+}
+
+void
+PlaylistBrowserNS::PodcastModel::slotLoad()
+{
+    Meta::PodcastEpisodeList episodes = selectedEpisodes();
+    if( !episodes.empty() )
+        The::playlistController()->insertOptioned(
+                podcastEpisodesToTracks( episodes ), Playlist::Replace );
+}
+
+Meta::TrackList
+PlaylistBrowserNS::PodcastModel::podcastEpisodesToTracks( Meta::PodcastEpisodeList episodes )
+{
+    Meta::TrackList tracks;
+    foreach( Meta::PodcastEpisodePtr episode, episodes )
+        tracks << Meta::TrackPtr::staticCast( episode );
+    return tracks;
+}
 #include "PodcastModel.moc"
