@@ -16,11 +16,11 @@
  *   Free Software Foundation, Inc.,                                       *
  *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.         *
  ***************************************************************************/
- 
+
 #ifndef USERPLAYLISTTREEVIEW_H
 #define USERPLAYLISTTREEVIEW_H
 
-#include "SqlPlaylistViewItem.h"
+
 #include "widgets/PrettyTreeView.h"
 
 class PopupDropper;
@@ -30,12 +30,14 @@ class KAction;
 
 namespace PlaylistBrowserNS {
 
+class MetaPlaylistModel;
+
 class UserPlaylistTreeView : public Amarok::PrettyTreeView
 {
     Q_OBJECT
 
 public:
-    UserPlaylistTreeView( QWidget *parent = 0 );
+    explicit UserPlaylistTreeView( MetaPlaylistModel *model, QWidget *parent = 0 );
     ~UserPlaylistTreeView();
 
     void setNewGroupAction( KAction * action );
@@ -49,24 +51,10 @@ protected:
 
     void contextMenuEvent( QContextMenuEvent* event );
 
-private slots:
-    void slotLoad();
-    void slotAppend();
-    void slotDelete();
-    void slotRename();
-
 private:
-    QSet<SqlPlaylistViewItemPtr> selectedItems() const;
-    QList<PopupDropperAction *> createCommonActions( QModelIndexList indices );
-
+    MetaPlaylistModel *m_model;
     PopupDropper* m_pd;
 
-    PopupDropperAction *m_appendAction;
-    PopupDropperAction *m_loadAction;
-
-    PopupDropperAction *m_deleteAction;
-    PopupDropperAction *m_renameAction;
-
     KAction *m_addGroupAction;
 
     QPoint m_dragStartPosition;
