@@ -25,39 +25,36 @@
 #include "EngineController.h"
 #include "MetaQueryMaker.h"
 #include "meta/file/File.h"
+#include "meta/cue/Cue.h"
 #include "meta/stream/Stream.h"
 #include "PluginManager.h"
 #include "SqlStorage.h"
 
-#include <QtAlgorithms>
-#include <QtGlobal>
 #include <QList>
 #include <QMetaEnum>
 #include <QMetaObject>
 #include <QPair>
 #include <QTimer>
 
-#include <KService>
-#include <KConfigGroup>
-#include <KFileItem>
+#include <KBuildSycocaProgressDialog>
 #include <KGlobal>
 #include <KMessageBox>
 #include <KRun>
-#include <KSharedConfig>
+#include <KService>
 
 #include <cstdlib>
 
-typedef QPair<Collection*, CollectionManager::CollectionStatus> CollectionPair;
+typedef QPair<Amarok::Collection*, CollectionManager::CollectionStatus> CollectionPair;
 
 struct CollectionManager::Private
 {
     QList<CollectionPair> collections;
-    QList<CollectionFactory*> factories;
+    QList<Amarok::CollectionFactory*> factories;
     SqlStorage *sqlDatabase;
-    QList<Collection*> unmanagedCollections;
-    QList<Collection*> managedCollections;
-    QList<TrackProvider*> trackProviders;
-    Collection *primaryCollection;
+    QList<Amarok::Collection*> unmanagedCollections;
+    QList<Amarok::Collection*> managedCollections;
+    QList<Amarok::TrackProvider*> trackProviders;
+    Amarok::Collection *primaryCollection;
 };
 
 CollectionManager* CollectionManager::s_instance = 0;
@@ -91,7 +88,7 @@ CollectionManager::CollectionManager()
 CollectionManager::~CollectionManager()
 {
     DEBUG_BLOCK
-    
+
     d->collections.clear();
     d->unmanagedCollections.clear();
     d->trackProviders.clear();
@@ -108,27 +105,32 @@ CollectionManager::init()
 
     d->sqlDatabase = 0;
     d->primaryCollection = 0;
+
     KService::List plugins = PluginManager::query( "[X-KDE-Amarok-plugintype] == 'collection'" );
     debug() << "Received [" << QString::number( plugins.count() ) << "] collection plugin offers";
-//     assert( plugins.count() );
-    //This code was in enginecontroller originally, it now ends up here.
-    if( plugins.count() == 0 )
-    {
-        KRun::runCommand( "kbuildsycoca4 --noincremental", 0 );
 
-        KMessageBox::error( 0, i18n(
-                "<p>Amarok could not find any collection plugins. "
-                "Amarok is now updating the KDE configuration database. Please wait a couple of minutes, then restart Amarok.</p>"
-                "<p>If this does not help, "
-                "it is likely that Amarok is installed under the wrong prefix, please fix your installation using:<pre>"
-                "$ cd /path/to/amarok/source-code/<br>"
-                "$ su -c \"make uninstall\"<br>"
-                "$ ./configure --prefix=`kde-config --prefix` && su -c \"make install\"<br>"
-                "$ kbuildsycoca4 --noincremental<br>"
-                "$ amarok</pre>"
-                "More information can be found in the README file. For further assistance join us at #amarok on irc.freenode.net.</p>" ) );
-        // don't use QApplication::exit, as the eventloop may not have started yet
-        std::exit( EXIT_SUCCESS );
+    if( plugins.isEmpty() )
+    {
+        debug() << "No Amarok plugins found, running kbuildsycoca4.";
+        KBuildSycocaProgressDialog::rebuildKSycoca( 0 );
+
+        plugins = PluginManager::query( "[X-KDE-Amarok-plugintype] == 'collection'" );
+        debug() << "Second attempt: Received [" << QString::number( plugins.count() ) << "] collection plugin offers";
+
+        if( plugins.isEmpty() )
+        {
+            KMessageBox::error( 0, i18n(
+                    "<p>Amarok could not find any collection plugins. "
+                    "it is possible that Amarok is installed under the wrong prefix, please fix your installation using:<pre>"
+                    "$ cd /path/to/amarok/source-code/<br>"
+                    "$ su -c \"make uninstall\"<br>"
+                    "$ cmake -DCMAKE_INSTALL_PREFIX=`kde4-config --prefix` && su -c \"make install\"<br>"
+                    "$ kbuildsycoca4 --noincremental<br>"
+                    "$ amarok</pre>"
+                    "More information can be found in the README file. For further assistance join us at #amarok on irc.freenode.net.</p>" ) );
+            // don't use QApplication::exit, as the eventloop may not have started yet
+            std::exit( EXIT_SUCCESS );
+        }
     }
 
     foreach( KService::Ptr service, plugins )
@@ -136,10 +138,10 @@ CollectionManager::init()
         Amarok::Plugin *plugin = PluginManager::createFromService( service );
         if ( plugin )
         {
-            CollectionFactory* factory = dynamic_cast<CollectionFactory*>( plugin );
+            Amarok::CollectionFactory* factory = dynamic_cast<Amarok::CollectionFactory*>( plugin );
             if ( factory )
             {
-                connect( factory, SIGNAL( newCollection( Collection* ) ), this, SLOT( slotNewCollection( Collection* ) ) );
+                connect( factory, SIGNAL( newCollection( Amarok::Collection* ) ), this, SLOT( slotNewCollection( Amarok::Collection* ) ) );
                 d->factories.append( factory );
                 factory->init();
             }
@@ -184,7 +186,7 @@ CollectionManager::checkCollectionChanges()
 QueryMaker*
 CollectionManager::queryMaker() const
 {
-    QList<Collection*> colls;
+    QList<Amarok::Collection*> colls;
     foreach( const CollectionPair &pair, d->collections )
     {
         if( pair.second & CollectionQueryable )
@@ -196,7 +198,7 @@ CollectionManager::queryMaker() const
 }
 
 void
-CollectionManager::slotNewCollection( Collection* newCollection )
+CollectionManager::slotNewCollection( Amarok::Collection* newCollection )
 {
     if( !newCollection )
     {
@@ -243,7 +245,7 @@ CollectionManager::slotNewCollection( Collection* newCollection )
 void
 CollectionManager::slotRemoveCollection()
 {
-    Collection* collection = qobject_cast<Collection*>( sender() );
+    Amarok::Collection* collection = qobject_cast<Amarok::Collection*>( sender() );
     if( collection )
     {
         CollectionStatus status = collectionStatus( collection->collectionId() );
@@ -279,7 +281,7 @@ CollectionManager::slotRemoveCollection()
 void
 CollectionManager::slotCollectionChanged()
 {
-    Collection *collection = dynamic_cast<Collection*>( sender() );
+    Amarok::Collection *collection = dynamic_cast<Amarok::Collection*>( sender() );
     if( collection )
     {
         CollectionStatus status = collectionStatus( collection->collectionId() );
@@ -290,10 +292,10 @@ CollectionManager::slotCollectionChanged()
     }
 }
 
-QList<Collection*>
+QList<Amarok::Collection*>
 CollectionManager::viewableCollections() const
 {
-    QList<Collection*> result;
+    QList<Amarok::Collection*> result;
     foreach( const CollectionPair &pair, d->collections )
     {
         if( pair.second & CollectionViewable )
@@ -304,10 +306,10 @@ CollectionManager::viewableCollections() const
     return result;
 }
 
-QList<Collection*>
+QList<Amarok::Collection*>
 CollectionManager::queryableCollections() const
 {
-    QList<Collection*> result;
+    QList<Amarok::Collection*> result;
     foreach( const CollectionPair &pair, d->collections )
         if( pair.second & CollectionQueryable )
             result << pair.first;
@@ -315,7 +317,7 @@ CollectionManager::queryableCollections() const
 
 }
 
-Collection*
+Amarok::Collection*
 CollectionManager::primaryCollection() const
 {
     return d->primaryCollection;
@@ -357,7 +359,7 @@ CollectionManager::trackForUrl( const KUrl &url )
         return Meta::TrackPtr( 0 );
     }
 
-    foreach( TrackProvider *provider, d->trackProviders )
+    foreach( Amarok::TrackProvider *provider, d->trackProviders )
     {
         if( provider->possiblyContainsTrack( url ) )
         {
@@ -371,7 +373,15 @@ CollectionManager::trackForUrl( const KUrl &url )
         return Meta::TrackPtr( new MetaStream::Track( url ) );
 
     if( url.protocol() == "file" && EngineController::canDecode( url ) )
+    {
+        KUrl cuesheet = MetaCue::Track::locateCueSheet( url );
+        if( !cuesheet.isEmpty() ) {
+            debug() << "Creating MetaCue::Track";
+            return Meta::TrackPtr( new MetaCue::Track( url, cuesheet ) );
+        }
+        debug() << "Creating MetaFile::Track";
         return Meta::TrackPtr( new MetaFile::Track( url ) );
+    }
 
     return Meta::TrackPtr( 0 );
 }
@@ -457,7 +467,7 @@ CollectionManager::slotContinueRelatedArtists() //SLOT
 }
 
 void
-CollectionManager::addUnmanagedCollection( Collection *newCollection, CollectionStatus defaultStatus )
+CollectionManager::addUnmanagedCollection( Amarok::Collection *newCollection, CollectionStatus defaultStatus )
 {
     if( newCollection && d->unmanagedCollections.indexOf( newCollection ) == -1 )
     {
@@ -480,7 +490,7 @@ CollectionManager::addUnmanagedCollection( Collection *newCollection, Collection
 }
 
 void
-CollectionManager::removeUnmanagedCollection( Collection *collection )
+CollectionManager::removeUnmanagedCollection( Amarok::Collection *collection )
 {
     //do not remove it from collection if it is not in unmanagedCollections
     if( collection && d->unmanagedCollections.removeAll( collection ) )
@@ -532,10 +542,10 @@ CollectionManager::collectionStatus( const QString &collectionId ) const
     return CollectionDisabled;
 }
 
-QHash<Collection*, CollectionManager::CollectionStatus>
+QHash<Amarok::Collection*, CollectionManager::CollectionStatus>
 CollectionManager::collections() const
 {
-    QHash<Collection*, CollectionManager::CollectionStatus> result;
+    QHash<Amarok::Collection*, CollectionManager::CollectionStatus> result;
     foreach( const CollectionPair &pair, d->collections )
     {
         result.insert( pair.first, pair.second );
@@ -544,16 +554,17 @@ CollectionManager::collections() const
 }
 
 void
-CollectionManager::addTrackProvider( TrackProvider *provider )
+CollectionManager::addTrackProvider( Amarok::TrackProvider *provider )
 {
     d->trackProviders.append( provider );
     emit trackProviderAdded( provider );
 }
 
 void
-CollectionManager::removeTrackProvider( TrackProvider *provider )
+CollectionManager::removeTrackProvider( Amarok::TrackProvider *provider )
 {
     d->trackProviders.removeAll( provider );
 }
 
 #include "CollectionManager.moc"
+
