@@ -73,6 +73,10 @@ DaapCollectionFactory::connectToManualServers()
     {
         debug() << "Adding server " << server;
         QStringList current = server.split( ':', QString::KeepEmptyParts );
+        //handle invalid urls gracefully
+        if( current.count() < 2 )
+            continue;
+            
         QString host = current.first();
         quint16 port = current.last().toUShort();
 
@@ -88,11 +92,18 @@ DaapCollectionFactory::serverOffline( DNSSD::RemoteService::Ptr service )
     QString key =  serverKey( service.data()->hostName(), service.data()->port() );
     if( m_collectionMap.contains( key ) )
     {
-        DaapCollection* coll = m_collectionMap[ key ];
+        DaapCollection *coll = m_collectionMap[ key ];
         if( coll )
+        {
             coll->serverOffline();  //collection will be deleted by collectionmanager
+        }
         else
+        {
             warning() << "collection already null";
+        }
+        
+        m_collectionMap.remove( key );
+
     }
     else
         warning() << "removing non-existent service";
@@ -110,7 +121,7 @@ DaapCollectionFactory::foundDaap( DNSSD::RemoteService::Ptr service )
 void
 DaapCollectionFactory::resolvedDaap( bool success )
 {
-    DEBUG_BLOCK
+  //  DEBUG_BLOCK
     const DNSSD::RemoteService* service =  dynamic_cast<const DNSSD::RemoteService*>(sender());
     if( !success || !service ) return;
     debug() << service->serviceName() << ' ' << service->hostName() << ' ' << service->domain() << ' ' << service->type();
@@ -178,6 +189,8 @@ DaapCollectionFactory::resolvedManualServerIp( QHostInfo hostInfo )
 void
 DaapCollectionFactory::resolvedServiceIp( QHostInfo hostInfo )
 {
+    DEBUG_BLOCK
+   // debug() << "got address:" << hostInfo.addresses() << "and lookup hash contains id" << hostInfo.lookupId() << "?" << m_lookupHash.contains(hostInfo.lookupId());
     if ( !m_lookupHash.contains(hostInfo.lookupId()) )
         return;
 
@@ -188,10 +201,12 @@ DaapCollectionFactory::resolvedServiceIp( QHostInfo hostInfo )
     QString ip = hostInfo.addresses().at(0).toString();
     quint16 port = m_lookupHash.value( hostInfo.lookupId() );
 
+   // debug() << "already added server?" << m_collectionMap.contains(serverKey( host, port ));
     if( m_collectionMap.contains(serverKey( host, port )) ) //same server from multiple interfaces
         return;
 
-    DaapCollection *coll = new DaapCollection( host, ip, port );
+   // debug() << "creating daap collection with" << host << ip << port;
+    QPointer<DaapCollection> coll( new DaapCollection( host, ip, port ) );
     connect( coll, SIGNAL( collectionReady() ), SLOT( slotCollectionReady() ) );
     connect( coll, SIGNAL( remove() ), SLOT( slotCollectionDownloadFailed() ) );
     m_collectionMap.insert( serverKey( host, port ), coll );
@@ -239,7 +254,11 @@ DaapCollection::collectionId() const
 QString
 DaapCollection::prettyName() const
 {
-    return "daap://" + m_host;
+    QString host = m_host;
+    // No need to be overly verbose
+    if( host.endsWith( ".local" ) )
+        host = host.remove( QRegExp(".local$") );
+    return i18n("Music share at %1", host);
 }
 
 void
