@@ -18,8 +18,10 @@
 
 #define DEBUG_PREFIX "IpodCollection"
 
-#include "meta/CollectionCapability.h"
 #include "IpodCollection.h"
+#include "IpodDeviceInfo.h"
+
+#include "meta/capabilities/CollectionCapability.h"
 #include "IpodCollectionLocation.h"
 #include "IpodMeta.h"
 #include "CollectionCapabilityIpod.h"
@@ -28,19 +30,20 @@
 #include "amarokconfig.h"
 #include "Debug.h"
 
+#include "MediaDeviceCache.h"
 #include "MediaDeviceMonitor.h"
 #include "MemoryQueryMaker.h"
 
-#include <KIcon>
 #include <KMessageBox>
 #include <KUrl>
 
-#include <QStringList>
+#include <solid/device.h>
+
 
 AMAROK_EXPORT_PLUGIN( IpodCollectionFactory )
 
 IpodCollectionFactory::IpodCollectionFactory()
-    : CollectionFactory()
+    : Amarok::CollectionFactory()
 {
     //nothing to do
 }
@@ -57,16 +60,23 @@ IpodCollectionFactory::init()
 
     // connect to the monitor
 
-    // TODO: the connection to this slot needs to be redone elsewhere
+ //   connect( this, SIGNAL( ipodDetected( const MediaDeviceInfo & ) ),
+//             MediaDeviceMonitor::instance(), SIGNAL( deviceDetected( const MediaDeviceInfo & ) ) );
+
     connect( MediaDeviceMonitor::instance(), SIGNAL( ipodReadyToConnect( const QString &, const QString & ) ),
              SLOT( ipodDetected( const QString &, const QString & ) ) );
+
+    // HACK: emitting old signal to avoid refactoring applet yet
+    connect( this, SIGNAL( tellIpodDetected( const QString &, const QString & ) ),
+             MediaDeviceMonitor::instance(), SIGNAL( ipodDetected( const QString &, const QString & ) ) );
+
     connect( MediaDeviceMonitor::instance(), SIGNAL( ipodReadyToDisconnect( const QString & ) ),
              SLOT( deviceRemoved( const QString & ) ) );
 
     connect( MediaDeviceMonitor::instance(), SIGNAL( deviceRemoved( const QString & ) ), SLOT( deviceRemoved( const QString & ) ) );
 
     // HACK: Usability: Force auto-connection of device upon detection
-    MediaDeviceMonitor::instance()->checkDevicesForIpod();
+    checkDevicesForIpod();
 }
 
 void
@@ -92,10 +102,10 @@ void
 IpodCollectionFactory::deviceRemoved( const QString &udi )
 {
     DEBUG_BLOCK
-    if (  m_collectionMap.contains( udi ) )
+    if( m_collectionMap.contains( udi ) )
     {
         IpodCollection* coll = m_collectionMap[ udi ];
-        if (  coll )
+        if( coll )
         {
             m_collectionMap.remove( udi ); // remove from map
             coll->deviceRemoved();  //collection will be deleted by collectionmanager
@@ -119,14 +129,65 @@ void
 IpodCollectionFactory::slotCollectionReady()
 {
     DEBUG_BLOCK
-    IpodCollection *collection = dynamic_cast<IpodCollection*>(  sender() );
+    IpodCollection *collection = dynamic_cast<IpodCollection*>( sender() );
     if( collection )
     {
         debug() << "emitting ipod collection newcollection";
-        emit newCollection(  collection );
+        emit newCollection( collection );
     }
 }
 
+void
+IpodCollectionFactory::checkDevicesForIpod()
+{
+    QStringList udiList = MediaDeviceMonitor::instance()->getDevices();
+
+    /* poll udi list for supported devices */
+    foreach( const QString &udi, udiList )
+    {
+        /* if ipod device found, emit signal */
+        if( isIpod( udi ) )
+        {
+            // HACK: Usability: Force auto-connection of device upon detection
+            QString mountpoint = MediaDeviceCache::instance()->volumeMountPoint(udi);
+            ipodDetected( mountpoint, udi );
+
+            //MediaDeviceInfo *deviceinfo =
+            new IpodDeviceInfo( mountpoint, udi );
+            //emit ipodDetected( deviceinfo );
+            // HACK: emit old signal to avoid refactor of applet yet
+            emit tellIpodDetected( mountpoint, udi );
+        }
+    }
+}
+
+bool
+IpodCollectionFactory::isIpod( const QString &udi ) const
+{
+    DEBUG_BLOCK
+
+    Solid::Device device;
+
+    device = Solid::Device(udi);
+    /* going until we reach a vendor, e.g. Apple */
+    while ( device.isValid() && device.vendor().isEmpty() )
+    {
+        device = Solid::Device( device.parentUdi() );
+    }
+
+    debug() << "Device udi: " << udi;
+    debug() << "Device name: " << MediaDeviceCache::instance()->deviceName(udi);
+    debug() << "Mount point: " << MediaDeviceCache::instance()->volumeMountPoint(udi);
+    if ( device.isValid() )
+    {
+        debug() << "vendor: " << device.vendor() << ", product: " << device.product();
+    }
+
+    /* if iPod found, return true */
+    return device.product() == "iPod";
+}
+
+
 //IpodCollection
 
 IpodCollection::IpodCollection( const QString &mountPoint, const QString &udi )
@@ -142,11 +203,28 @@ IpodCollection::IpodCollection( const QString &mountPoint, const QString &udi )
     connectDevice();
 }
 
+bool
+IpodCollection::possiblyContainsTrack( const KUrl &url ) const
+{
+    // We could simply check for iPod_Control except that we could actually have multiple ipods connected
+    return url.url().startsWith( m_mountPoint ) || url.url().startsWith( "file://" + m_mountPoint );
+}
+
+Meta::TrackPtr
+IpodCollection::trackForUrl( const KUrl &url )
+{
+    QString uid = url.url();
+    if( uid.startsWith("file://") )
+        uid = uid.remove( 0, 7 );
+    Meta::TrackPtr ipodTrack = m_trackMap.value( uid );
+    return ipodTrack ? ipodTrack : Collection::trackForUrl(url);
+}
+
 bool
 IpodCollection::hasCapabilityInterface( Meta::Capability::Type type ) const
 {
     DEBUG_BLOCK
-            switch( type )
+    switch( type )
     {
         case Meta::Capability::Collection:
             return true;
@@ -160,7 +238,7 @@ Meta::Capability*
 IpodCollection::asCapabilityInterface( Meta::Capability::Type type )
 {
     DEBUG_BLOCK
-            switch( type )
+    switch( type )
     {
         case Meta::Capability::Collection:
             return new Meta::CollectionCapabilityIpod( this );
@@ -170,26 +248,12 @@ IpodCollection::asCapabilityInterface( Meta::Capability::Type type )
 }
 
 void
-IpodCollection::copyTrackToDevice( const Meta::TrackPtr &track )
-{
-    m_handler->copyTrackToDevice( track );
-    return;
-}
-
-bool
-IpodCollection::deleteTrackFromDevice( const Meta::IpodTrackPtr &track )
+IpodCollection::copyTrackListToDevice( const Meta::TrackList tracklist )
 {
     DEBUG_BLOCK
-
-    // remove the track from the device
-    if( !m_handler->deleteTrackFromDevice( track ) )
-        return false;
-
-    // remove the track from the collection maps too
-    removeTrack( track );
-
-    debug() << "deleteTrackFromDevice returning true";
-    return true;
+    connect( m_handler, SIGNAL( copyTracksDone( bool  ) ),
+             SLOT( slotCopyTracksCompleted( bool ) ), Qt::QueuedConnection );
+    m_handler->copyTrackListToDevice( tracklist );
 }
 
 void
@@ -258,7 +322,7 @@ IpodCollection::removeTrack( const Meta::IpodTrackPtr &track )
 }
 
 void
-IpodCollection::updateTags( Meta::IpodTrack *track)
+IpodCollection::updateTags( Meta::IpodTrack *track )
 {
     DEBUG_BLOCK
     Meta::IpodTrackPtr trackPtr( track );
@@ -272,7 +336,7 @@ IpodCollection::updateTags( Meta::IpodTrack *track)
 void
 IpodCollection::writeDatabase()
 {
-    m_handler->writeITunesDB( false );
+    m_handler->writeDatabase();
 }
 
 IpodCollection::~IpodCollection()
@@ -328,31 +392,20 @@ IpodCollection::setTrackToDelete( const Meta::IpodTrackPtr &track )
     m_trackToDelete = track;
 }
 
-void
-IpodCollection::deleteTrackToDelete()
-{
-    deleteTrackFromDevice( m_trackToDelete );
-}
-
-void
-IpodCollection::deleteTrackSlot( Meta::IpodTrackPtr track)
-{
-    deleteTrackFromDevice( track );
-}
-
 void
 IpodCollection::deleteTracksSlot( Meta::TrackList tracklist )
 {
     DEBUG_BLOCK
+    connect( m_handler, SIGNAL( deleteTracksDone() ),
+                        SLOT( slotDeleteTracksCompleted() ), Qt::QueuedConnection );
 
-    // remove the tracks from the device
-    if( !m_handler->deleteTracksFromDevice( tracklist ) )
-            debug() << "Failed to delete tracks";
-
-    // remove the tracks from the collection maps too
+    // remove the tracks from the collection maps
     foreach( Meta::TrackPtr track, tracklist )
         removeTrack( Meta::IpodTrackPtr::staticCast( track ) );
 
+    // remove the tracks from the device
+    m_handler->deleteTrackListFromDevice( tracklist );
+
 /*
     const QString text( i18nc( "@info", "Do you really want to delete these %1 tracks?", tracklist.count() ) );
     const bool del = KMessageBox::warningContinueCancel(this,
@@ -360,8 +413,6 @@ IpodCollection::deleteTracksSlot( Meta::TrackList tracklist )
             QString() ) == KMessageBox::Continue;
 */
 
-    
-
     // inform treeview collection has updated
     emit updated();
 }
@@ -374,11 +425,34 @@ IpodCollection::slotDisconnect()
 }
 
 void
-IpodCollection::copyTracksCompleted()
+IpodCollection::slotCopyTracksCompleted( bool success )
+{
+    DEBUG_BLOCK
+
+    // HACK: write database regardless
+    // See note about "success" in IpodHandler::copyTrackListToDevice
+    debug() << "Trying to write iTunes database";
+    m_handler->writeDatabase();
+
+    // inform collection location that copying is done
+
+    emit copyTracksCompleted( success );
+
+    // inform treeview collection has updated
+
+    emit updated();
+}
+
+void
+IpodCollection::slotDeleteTracksCompleted()
 {
     DEBUG_BLOCK
     debug() << "Trying to write iTunes database";
-    m_handler->writeITunesDB( false ); // false, since not threaded, implement later
+
+    m_handler->writeDatabase();
+
+    // inform treeview collection has updated
+    emit updated();
 }
 
 void
