@@ -38,11 +38,14 @@ public:
     void removeTemporaryTables();
     void copyToPermanentTables();
     void cleanPermanentTables();
+    void upgradeVersion1to2();
 
     void deleteAllRedundant( const QString &type ); //type is artist,album,genre,composer or year
 
     void removeFilesInDir( int deviceid, const QString &rdir );
     void removeFilesInDirFromTemporaryTables( int deviceid, const QString &rdir );
+    
+    void cleanupDatabase();
 
 private:
     /** creates all the necessary tables, indexes etc. for the database */
