@@ -3,6 +3,7 @@
  *  Copyright (c) 2007 Maximilian Kossick <maximilian.kossick@googlemail.com>
  *  Copyright (c) 2007 Casey Link <unnamedrambler@gmail.com>
  *  Copyright (c) 2008 Leo Franchi <lfranchi@kde.org>
+ *  Copyright (c) 2008-2009 Jeff Mitchell <mitchell@kde.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -39,6 +40,7 @@
 #include <QVariant>
 #include <QTextCodec>
 #include <QXmlStreamAttributes>
+#include <QDir>         //QDir::separator
 
 #include <KMessageBox>
 #include <KStandardDirs>
@@ -65,11 +67,10 @@ ScanManager::ScanManager( SqlCollection *parent )
     DEBUG_BLOCK
 
     // If Amarok is not installed in standard directory
-    const QString binDir = KStandardDirs::findExe( "amarok" );
-    m_amarokCollectionScanDir = binDir.left( binDir.lastIndexOf( '/' ) + 1 );
+    m_amarokCollectionScanDir = App::instance()->applicationDirPath() + QDir::separator();
 
     QTimer *watchFoldersTimer = new QTimer( this );
-    connect( watchFoldersTimer, SIGNAL( timeout() ), SLOT( slotWatchFolders() ) ); 
+    connect( watchFoldersTimer, SIGNAL( timeout() ), SLOT( slotWatchFolders() ) );
     watchFoldersTimer->start( WATCH_INTERVAL );
 }
 
@@ -96,16 +97,7 @@ ScanManager::startFullScan()
         return;
     }
     cleanTables();
-    m_scanner = new AmarokProcess( this );
-    *m_scanner << m_amarokCollectionScanDir + "amarokcollectionscanner" << "--nocrashhandler" << "-p";
-    if( AmarokConfig::scanRecursively() )
-        *m_scanner << "-r";
-    *m_scanner << MountPointManager::instance()->collectionFolders();
-    m_scanner->setOutputChannelMode( KProcess::OnlyStdoutChannel );
-    connect( m_scanner, SIGNAL( readyReadStandardOutput() ), this, SLOT( slotReadReady() ) );
-    connect( m_scanner, SIGNAL( finished( int ) ), SLOT( slotFinished(  ) ) );
-    connect( m_scanner, SIGNAL( error( QProcess::ProcessError ) ), SLOT( slotError( QProcess::ProcessError ) ) );
-    m_scanner->start();
+
     if( m_parser )
     {
         stopParser();
@@ -115,6 +107,27 @@ ScanManager::startFullScan()
     m_isIncremental = false;
     connect( m_parser, SIGNAL( done( ThreadWeaver::Job* ) ), SLOT( slotJobDone() ) );
     ThreadWeaver::Weaver::instance()->enqueue( m_parser );
+
+    QString batchfileLocation( KGlobal::dirs()->saveLocation( "data", QString("amarok/"), false ) + "amarokcollectionscanner_batchfullscan.xml" );
+    debug() << "Checking for batch file in " << batchfileLocation;
+    
+    if( !QFile::exists( batchfileLocation ) || !readBatchFile( batchfileLocation )  )
+    {
+        m_scanner = new AmarokProcess( this );
+        *m_scanner << m_amarokCollectionScanDir + "amarokcollectionscanner" << "-p";
+        if( AmarokConfig::scanRecursively() )
+            *m_scanner << "-r";
+        *m_scanner << "--savelocation" << KGlobal::dirs()->saveLocation( "data", QString("amarok/"), true );
+        debug() << "GOING TO SCAN:";
+        foreach( const QString &dir, MountPointManager::instance()->collectionFolders() )
+            debug() << "    " << dir;
+        *m_scanner << MountPointManager::instance()->collectionFolders();
+        m_scanner->setOutputChannelMode( KProcess::OnlyStdoutChannel );
+        connect( m_scanner, SIGNAL( readyReadStandardOutput() ), this, SLOT( slotReadReady() ) );
+        connect( m_scanner, SIGNAL( finished( int ) ), SLOT( slotFinished(  ) ) );
+        connect( m_scanner, SIGNAL( error( QProcess::ProcessError ) ), SLOT( slotError( QProcess::ProcessError ) ) );
+        m_scanner->start();
+    }
 }
 
 void ScanManager::startIncrementalScan()
@@ -130,33 +143,31 @@ void ScanManager::startIncrementalScan()
         debug() << "scanning currently blocked";
         return;
     }
-    const QStringList dirs = getDirsToScan();
 
-    debug() << "GOING TO SCAN:";
-    foreach( const QString &dir, dirs )
-        debug() << "    " << dir;
+    QString batchfileLocation( KGlobal::dirs()->saveLocation( "data", QString("amarok/"), false ) + "amarokcollectionscanner_batchincrementalscan.xml" );
+    bool batchfileExists = QFile::exists( batchfileLocation );
+    if( batchfileExists )
+        debug() << "Found batchfile in " << batchfileLocation;
 
-    if( dirs.isEmpty() )
+    QStringList dirs;
+    
+    if( !batchfileExists )
     {
-        debug() << "Scanning nothing, return.";
-        return;
+        dirs = getDirsToScan();
+
+        debug() << "GOING TO SCAN:";
+        foreach( const QString &dir, dirs )
+            debug() << "    " << dir;
+
+        if( dirs.isEmpty() )
+        {
+            debug() << "Scanning nothing, return.";
+            writeBatchIncrementalInfoFile();
+            return;
+        }
+
     }
-    if( !m_dbusHandler )
-    {
-        m_dbusHandler = new SqlCollectionDBusHandler( m_collection );
-    }
-    m_scanner = new AmarokProcess( this );
-    *m_scanner << m_amarokCollectionScanDir + "amarokcollectionscanner" << "--nocrashhandler" << "-i" << "--collectionid" << m_collection->collectionId();
-    if( AmarokConfig::scanRecursively() )
-        *m_scanner << "-r";
-    if( pApp->isUniqueInstance() ) 
-        *m_scanner << "--pid" << QString::number( QApplication::applicationPid() );
-    *m_scanner << dirs;
-    m_scanner->setOutputChannelMode( KProcess::OnlyStdoutChannel );
-    connect( m_scanner, SIGNAL( readyReadStandardOutput() ), this, SLOT( slotReadReady() ) );
-    connect( m_scanner, SIGNAL( finished( int ) ), SLOT( slotFinished() ) );
-    connect( m_scanner, SIGNAL( error( QProcess::ProcessError ) ), SLOT( slotError( QProcess::ProcessError ) ) );
-    m_scanner->start();
+
     if( m_parser )
     {
         stopParser();
@@ -166,13 +177,35 @@ void ScanManager::startIncrementalScan()
     m_isIncremental = true;
     connect( m_parser, SIGNAL( done( ThreadWeaver::Job* ) ), SLOT( slotJobDone() ) );
     ThreadWeaver::Weaver::instance()->enqueue( m_parser );
+
+    if( !batchfileExists || !readBatchFile( batchfileLocation )  )
+    {
+        if( !m_dbusHandler )
+        {
+            m_dbusHandler = new SqlCollectionDBusHandler( m_collection );
+        }
+        m_scanner = new AmarokProcess( this );
+        *m_scanner << m_amarokCollectionScanDir + "amarokcollectionscanner" << "-i"
+                << "--collectionid" << m_collection->collectionId() << "-p";
+        if( AmarokConfig::scanRecursively() )
+            *m_scanner << "-r";
+        *m_scanner << "--savelocation" << KGlobal::dirs()->saveLocation( "data", QString("amarok/"), true );
+        if( pApp->isNonUniqueInstance() )
+            *m_scanner << "--pid" << QString::number( QApplication::applicationPid() );
+        *m_scanner << dirs;
+        m_scanner->setOutputChannelMode( KProcess::OnlyStdoutChannel );
+        connect( m_scanner, SIGNAL( readyReadStandardOutput() ), this, SLOT( slotReadReady() ) );
+        connect( m_scanner, SIGNAL( finished( int ) ), SLOT( slotFinished() ) );
+        connect( m_scanner, SIGNAL( error( QProcess::ProcessError ) ), SLOT( slotError( QProcess::ProcessError ) ) );
+        m_scanner->start();        
+    }
 }
 
 bool
 ScanManager::isDirInCollection( QString path )
 {
     // In the database all directories have a trailing slash, so we must add that
-    if ( !path.endsWith( '/' ) )
+    if( !path.endsWith( '/' ) )
         path += '/';
 
     const int deviceid = MountPointManager::instance()->getIdForUrl( path );
@@ -265,6 +298,9 @@ ScanManager::slotFinished( )
         m_dbusHandler->deleteLater();
         m_dbusHandler = 0;
     }
+
+    if( m_isIncremental )
+        writeBatchIncrementalInfoFile();
 }
 
 void
@@ -280,8 +316,53 @@ ScanManager::slotError( QProcess::ProcessError error )
     }
 }
 
+bool
+ScanManager::readBatchFile( QString fileLocation )
+{
+    DEBUG_BLOCK
+    QFile file( fileLocation );
+    if( !file.open( QIODevice::ReadOnly ) )
+    {
+        debug() << "Couldn't open batchscan file, which does exist";
+        return false;
+    }
+    
+    QByteArray data;
+    data = file.readAll();
+
+    if( !data.isEmpty() )
+    {
+        // amarokcollectionscanner outputs UTF-8 regardless of local encoding
+        QString newData = QTextCodec::codecForName( "UTF-8" )->toUnicode( data );
+        QStringList splitData = newData.split( "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>", QString::SkipEmptyParts );
+        debug() << "splitData.size = " << splitData.size();
+        if( splitData.size() > 1 )
+        {
+            splitData.first().chop( 11 );
+            int n = 1;
+            while( n < splitData.size() - 1 )
+            {
+                splitData[n].remove( 0, 9 );
+                splitData[n].chop( 11 );
+                n++;
+            }
+            splitData[splitData.size()-1].remove( 0, 9 );
+        }
+        splitData.first().prepend( "<?xml version=\"1.0\" encoding=\"UTF-8\" ?>" );
+        QString joinedData = splitData.join( QString() );
+        if( m_parser )
+            m_parser->addNewXmlData( joinedData );
+    }
+    else
+        debug() << "Empty read from file!";
+
+    file.close();
+    QFile::remove( fileLocation );
+    return true;
+}
+
 QStringList
-ScanManager::getDirsToScan() const
+ScanManager::getDirsToScan()
 {
     DEBUG_BLOCK
 
@@ -292,12 +373,13 @@ ScanManager::getDirsToScan() const
         if ( !deviceIds.isEmpty() ) deviceIds += ',';
         deviceIds += QString::number( id );
     }
-    
+
     const QStringList values = m_collection->query(
             QString( "SELECT id, deviceid, dir, changedate FROM directories WHERE deviceid IN (%1);" )
             .arg( deviceIds ) );
 
     QList<int> changedFolderIds;
+    QList<int> deletedFolderIds;
 
     QStringList result;
     for( QListIterator<QString> iter( values ); iter.hasNext(); )
@@ -310,6 +392,7 @@ ScanManager::getDirsToScan() const
         QFileInfo info( folder );
         if( info.exists() )
         {
+            m_incrementalDirs << folder;
             if( info.lastModified().toTime_t() != mtime )
             {
                 result << folder;
@@ -319,8 +402,8 @@ ScanManager::getDirsToScan() const
         else
         {
             // this folder has been removed
-            result << folder;
             changedFolderIds << id;
+            deletedFolderIds << id;
         }
     }
     {
@@ -349,6 +432,20 @@ ScanManager::getDirsToScan() const
             }
         }
     }
+    {
+        QString ids;
+        foreach( int id, deletedFolderIds )
+        {
+            if( !ids.isEmpty() )
+                ids += ',';
+            ids += QString::number( id );
+        }
+        if( !ids.isEmpty() )
+        {
+            QString sql = QString( "DELETE FROM directories WHERE id IN ( %1 );" ).arg( ids );
+            m_collection->query( sql );
+        }
+    }
     //debug() << "Scanning the following dirs: " << result;
     return result;
 }
@@ -393,11 +490,11 @@ ScanManager::restartScanner()
     DEBUG_BLOCK
 
     m_scanner = new AmarokProcess( this );
-    *m_scanner << m_amarokCollectionScanDir + "amarokcollectionscanner" << "--nocrashhandler";
+    *m_scanner << m_amarokCollectionScanDir + "amarokcollectionscanner";
     if( m_isIncremental )
     {
         *m_scanner << "-i" << "--collectionid" << m_collection->collectionId();
-        if( pApp->isUniqueInstance() )
+        if( pApp->isNonUniqueInstance() )
             *m_scanner << "--pid" << QString::number( QApplication::applicationPid() );
     }
 
@@ -439,6 +536,21 @@ ScanManager::stopParser()
     }
 }
 
+void
+ScanManager::writeBatchIncrementalInfoFile()
+{
+    DEBUG_BLOCK
+    QString fileName = KGlobal::dirs()->saveLocation( "data", QString("amarok/"), true )
+        + "amarokcollectionscanner_batchincrementalinput.data";
+    QFile incrementalFile( fileName );
+    if( incrementalFile.open( QIODevice::WriteOnly ) )
+    {
+        QTextStream stream( &incrementalFile );
+        stream.setCodec( QTextCodec::codecForName("UTF-8") );
+        stream << m_incrementalDirs.join( "\n" );
+        incrementalFile.close();
+    }
+}
 
 ///////////////////////////////////////////////////////////////////////////////
 // class XmlParseJob
@@ -487,7 +599,6 @@ XmlParseJob::run()
     {
         processor.setScanType( ScanResultProcessor::FullScan );
     }
-    
     do
     {
         m_abortMutex.lock();
@@ -496,7 +607,7 @@ XmlParseJob::run()
 
         if( abort )
             break;
-        
+
         //debug() << "Get new xml data or wait till new xml data is available";
 
         m_mutex.lock();
@@ -504,7 +615,7 @@ XmlParseJob::run()
         {
             m_wait.wait( &m_mutex );
         }
-        
+
         if( m_nextData.isEmpty() )
             break;
 
@@ -525,17 +636,30 @@ XmlParseJob::run()
 
                 if( localname == "itemcount" )
                 {
+//                     debug() << "Got an itemcount with value: " << localname.toString();
                     The::statusBar()->incrementProgressTotalSteps( this, m_reader.attributes().value( "count" ).toString().toInt() );
                 }
                 else if( localname == "tags" )
                 {
-                    //debug() << "Parsing FILE:\n";
+//                     debug() << "Parsing FILE:\n";
                     QXmlStreamAttributes attrs = m_reader.attributes();
                     QList<QXmlStreamAttribute> list = attrs.toList();
 
-                    //foreach( QXmlStreamAttribute l, list )
-                    //    debug() << " TAG: " << l.name().toString() << '\t' << l.value().toString() << '\n';
-                    //debug() << "End FILE";
+//                     foreach( QXmlStreamAttribute l, list )
+//                        debug() << " TAG: " << l.name().toString() << '\t' << l.value().toString() << '\n';
+//                     debug() << "End FILE";
+
+                    //NOTE: compilation may not be used according to the comment below,
+                    //but this is functionality getting moved from the collection scanner
+                    //until (if ever) pure-Qt transations are supported
+                    QString compilationValue = attrs.value( "compilation" ).toString();
+                    if( compilationValue == "checkforvarious" )
+                    {
+                        if( attrs.value( "artist" ).toString() == i18n( "Various Artists" ) )
+                            compilationValue = QString::number( 1 );
+                        else
+                            compilationValue = QString();
+                    }                            
 
                     QVariantMap data;
                     data.insert( Meta::Field::URL, attrs.value( "path" ).toString() );
@@ -550,6 +674,7 @@ XmlParseJob::run()
                     data.insert( Meta::Field::DISCNUMBER, attrs.value( "discnumber" ).toString() );
                     data.insert( Meta::Field::BPM, attrs.value( "bpm" ).toString() );
                     //filetype and uniqueid are missing in the fields, compilation is not used here
+
                     if( attrs.value( "audioproperties" ) == "true" )
                     {
                         data.insert( Meta::Field::BITRATE, attrs.value( "bitrate" ).toString() );
@@ -558,6 +683,20 @@ XmlParseJob::run()
                     }
                     if( !attrs.value( "filesize" ).isEmpty() )
                         data.insert( Meta::Field::FILESIZE, attrs.value( "filesize" ).toString() );
+                    if( !attrs.value( "albumgain" ).isEmpty() )
+                    {
+                        data.insert( Meta::Field::ALBUMGAIN, attrs.value( "albumgain" ).toString().toDouble() );
+                        // peak gain is in decibels
+                        if( !attrs.value( "albumpeakgain" ).isEmpty() )
+                            data.insert( Meta::Field::ALBUMPEAKGAIN, attrs.value( "albumpeakgain" ).toString().toDouble() );
+                    }
+                    if( !attrs.value( "trackgain" ).isEmpty() )
+                    {
+                        data.insert( Meta::Field::TRACKGAIN, attrs.value( "trackgain" ).toString().toDouble() );
+                        // peak gain is in decibels
+                        if( !attrs.value( "trackpeakgain" ).isEmpty() )
+                            data.insert( Meta::Field::TRACKPEAKGAIN, attrs.value( "trackpeakgain" ).toString().toDouble() );
+                    }
 
                     data.insert( Meta::Field::UNIQUEID, attrs.value( "uniqueid" ).toString() );
 
@@ -582,13 +721,13 @@ XmlParseJob::run()
                 }
                 else if( localname == "folder" )
                 {
-                    //debug() << "Parsing FOLDER:\n";
+//                     debug() << "Parsing FOLDER:\n";
                     QXmlStreamAttributes attrs = m_reader.attributes();
                     QList<QXmlStreamAttribute> list = attrs.toList();
 
-                    //foreach( QXmlStreamAttribute l, list )
-                    //    debug() << " ATTRIBUTE: " << l.name().toString() << '\t' << l.value().toString() << '\n';
-                    //debug() << "End FOLDER";
+//                     foreach( QXmlStreamAttribute l, list )
+//                        debug() << " ATTRIBUTE: " << l.name().toString() << '\t' << l.value().toString() << '\n';
+//                     debug() << "End FOLDER";
 
                     const QString folder = attrs.value( "path" ).toString();
                     const QFileInfo info( folder );
@@ -603,18 +742,18 @@ XmlParseJob::run()
                 }
                 else if( localname == "image" )
                 {
-                    debug() << "Parsing IMAGE:\n";
+//                     debug() << "Parsing IMAGE:\n";
                     QXmlStreamAttributes attrs = m_reader.attributes();
                     QList<QXmlStreamAttribute> thisList = attrs.toList();
 
-                    //foreach( QXmlStreamAttribute l, thisList )
-                    //    debug() << " ATTR: " << l.name().toString() << '\t' << l.value().toString() << '\n';
-                    //debug() << "End IMAGE";
+//                     foreach( QXmlStreamAttribute l, thisList )
+//                        debug() << " ATTR: " << l.name().toString() << '\t' << l.value().toString() << '\n';
+//                     debug() << "End IMAGE";
 
                     // Deserialize CoverBundle list
                     QStringList list = attrs.value( "list" ).toString().split( "AMAROK_MAGIC" );
                     QList< QPair<QString, QString> > covers;
-                   
+
                     // Don't iterate if the list only has one element
                     if( list.size() > 1 )
                     {
@@ -626,14 +765,14 @@ XmlParseJob::run()
                 }
             }
         }
-        if( m_reader.error() != QXmlStreamReader::PrematureEndOfDocumentError )
+        if( m_reader.error() != QXmlStreamReader::PrematureEndOfDocumentError && m_reader.error() != QXmlStreamReader::NoError )
         {
             debug() << "do-while done with error: " << m_reader.error();
             //the error cannot be PrematureEndOfDocumentError, so handle an unrecoverable error here
 
             // At this point, most likely the scanner has crashed and is about to get restarted.
             // Reset the XML-reader and try to get new data:
-            debug() << "Trying to restart the QXmlStreamReader.."; 
+            debug() << "Trying to restart the QXmlStreamReader..";
             m_reader.clear();
             continue;
         }
@@ -650,7 +789,7 @@ XmlParseJob::run()
         if( !directoryData.isEmpty() )
             processor.processDirectory( directoryData );
         processor.commit();
-    } 
+    }
 }
 
 void
