@@ -2,6 +2,7 @@
  *  Copyright (c) 2003-2008 Mark Kretschmann <kretschmann@kde.org>
  *  Copyright (c) 2007 Maximilian Kossick <maximilian.kossick@googlemail.com>
  *  Copyright (c) 2007 Casey Link <unnamedrambler@gmail.com>
+ *  Copyright (c) 2008-2009 Jeff Mitchell <mitchell@kde.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -61,11 +62,13 @@ class ScanManager : public QObject
         void restartScanner();
 
     private:
-        QStringList getDirsToScan() const;
+        QStringList getDirsToScan();
         void handleRestart();
         void cleanTables();
         void stopParser();
-
+        void writeBatchIncrementalInfoFile();
+        bool readBatchFile( QString fileLocation );
+        
     private:
         SqlCollection *m_collection;
         SqlCollectionDBusHandler *m_dbusHandler;
@@ -78,6 +81,7 @@ class ScanManager : public QObject
         bool m_isIncremental;
         bool m_blockScan;
         QString m_amarokCollectionScanDir;
+        QStringList m_incrementalDirs;
 };
 
 class XmlParseJob : public ThreadWeaver::Job
