@@ -57,7 +57,8 @@ ScanResultProcessor::setScanType( ScanType type )
 void
 ScanResultProcessor::addDirectory( const QString &dir, uint mtime )
 {
-    if (dir.isEmpty()) {
+    if( dir.isEmpty() )
+    {
         debug() << "got directory with no path from the scanner, not adding";
         return;
     }
@@ -90,7 +91,7 @@ ScanResultProcessor::addDirectory( const QString &dir, uint mtime )
     }
 }
 
-void 
+void
 ScanResultProcessor::addImage( const QString &path, const QList< QPair<QString, QString> > covers )
 {
     QList< QPair<QString,QString> >::ConstIterator it = covers.begin();
@@ -102,7 +103,7 @@ ScanResultProcessor::addImage( const QString &path, const QList< QPair<QString,
 
         int artist = artistId( key.first );
         int album  = albumId( key.second, artist );
-       
+
         // Will automatically add the image path to the database if needed
         imageId( path, album );
     }
@@ -142,7 +143,7 @@ ScanResultProcessor::commit()
     debug() << "Sending changed signal";
     m_collection->sendChangedSignal();
 
-    connect( this, SIGNAL( changedTrackUrls( TrackUrls ) ), 
+    connect( this, SIGNAL( changedTrackUrls( TrackUrls ) ),
              CollectionManager::instance()->primaryCollection(), SLOT( updateTrackUrls( TrackUrls ) ) );
 
     emit changedTrackUrls( m_changedUrls );
@@ -217,13 +218,9 @@ ScanResultProcessor::findAlbumArtist( const QSet<QString> &artists ) const
             else
             {
                 if( artistCount.contains( tmp ) )
-                {
                     artistCount.insert( tmp, artistCount.value( tmp ) + 1 );
-                }
                 else
-                {
                     artistCount.insert( tmp, 1 );
-                }
             }
         }
         else if( artist.contains( "feat." ) )
@@ -239,25 +236,17 @@ ScanResultProcessor::findAlbumArtist( const QSet<QString> &artists ) const
             else
             {
                 if( artistCount.contains( tmp ) )
-                {
                     artistCount.insert( tmp, artistCount.value( tmp ) + 1 );
-                }
                 else
-                {
                     artistCount.insert( tmp, 1 );
-                }
             }
         }
         else
         {
             if( artistCount.contains( artist ) )
-            {
                 artistCount.insert( artist, artistCount.value( artist ) + 1 );
-            }
             else
-            {
                 artistCount.insert( artist, 1 );
-            }
         }
     }
     QString albumArtist;
@@ -276,6 +265,7 @@ ScanResultProcessor::findAlbumArtist( const QSet<QString> &artists ) const
 void
 ScanResultProcessor::addTrack( const QVariantMap &trackData, int albumArtistId )
 {
+//     DEBUG_BLOCK
     //amarok 1 stored all tracks of a compilation in different directories.
     //when using its "Organize Collection" feature
     //try to detect these cases
@@ -283,37 +273,22 @@ ScanResultProcessor::addTrack( const QVariantMap &trackData, int albumArtistId )
     int album = 0;
 
     QString path = trackData.value( Field::URL ).toString();
-    
+
     QFileInfo file( path );
-    
+
     QDir dir = file.dir();
     dir.setFilter( QDir::Files );
-    
+
+    //TODO: find a better way to ignore non-audio files than the extension matching below
     //name filtering should be case-insensitive because we do not use QDir::CaseSensitive
     QStringList filters;
     filters << "*.mp3" << "*.ogg" << "*.oga" << "*.flac" << "*.wma" << "*.m4a";
     dir.setNameFilters( filters );
-    
-    int compilationId = 0;
-    
-    //do not check existing albums if there is more than one file in the directory
-    //see comments in checkExistingAlbums
-    
-    //TODO: find a better way to ignore non-audio files than the extension matching above
-    if( !m_filesInDirs.contains( dir.absolutePath() ) )
-    {
-        m_filesInDirs.insert( dir.absolutePath(), dir.count() );
-    }
 
-    if( dir.count() == 1 )
-    {
-        compilationId = checkExistingAlbums( albumName );
-    }
+    int compilationId = checkAlbumArtists( file, albumName, albumArtistId );
 
-    if( 0 == compilationId )
-    {
+    if( compilationId == 0 )
         album = albumId( albumName, albumArtistId );
-    }
 
     int artist = artistId( trackData.value( Field::ARTIST ).toString() );
     int genre = genreId( trackData.value( Field::GENRE ).toString() );
@@ -328,10 +303,11 @@ ScanResultProcessor::addTrack( const QVariantMap &trackData, int albumArtistId )
     //urlId will take care of the urls table part of AFT
     int url = urlId( path, uid );
 
-    QString sql,sql2;
+    QString sql,sql2,sql3;
     sql = "REPLACE INTO tracks_temp(url,artist,album,genre,composer,year,title,comment,"
                     "tracknumber,discnumber,bitrate,length,samplerate,filesize,filetype,bpm,"
-                    "createdate,modifydate) VALUES ( %1,%2,%3,%4,%5,%6,'%7','%8',%9"; //goes up to tracknumber
+                    "createdate,modifydate,albumgain,albumpeakgain,trackgain,trackpeakgain) "
+                    "VALUES ( %1,%2,%3,%4,%5,%6,'%7','%8',%9"; //goes up to tracknumber
     sql = sql.arg( QString::number( url )
                 , QString::number( artist )
                 , QString::number( compilationId ? compilationId : album )
@@ -342,7 +318,7 @@ ScanResultProcessor::addTrack( const QVariantMap &trackData, int albumArtistId )
                 , m_collection->escape( trackData[ Field::COMMENT ].toString() )
                 , QString::number( trackData[Field::TRACKNUMBER].toInt() ) );
 
-    sql2 = ",%1,%2,%3,%4,%5,%6,%7,%8,%9);";
+    sql2 = ",%1,%2,%3,%4,%5,%6,%7,%8,%9"; // goes up to modifydate
     sql2 = sql2.arg( QString::number( trackData[Field::DISCNUMBER].toInt() )
                 , QString::number( trackData[Field::BITRATE].toInt() )
                 , QString::number( trackData[Field::LENGTH].toInt() )
@@ -352,7 +328,26 @@ ScanResultProcessor::addTrack( const QVariantMap &trackData, int albumArtistId )
                 , "0" // NYI: bpm
                 , QString::number( created )
                 , QString::number( modified ) );
-    sql += sql2;
+
+    // replay gain - only store gain values if we also have peak gain values
+    //               (ie: ignore broken tags)
+    sql3 = ",%1,%2,%3,%4);";
+    if ( trackData.contains( Field::ALBUMGAIN ) && trackData.contains( Field::ALBUMPEAKGAIN ) )
+    {
+        sql3 = sql3.arg( QString::number( trackData[ Field::ALBUMGAIN ].toDouble() ) );
+        sql3 = sql3.arg( trackData[ Field::ALBUMPEAKGAIN ].toDouble() );
+    }
+    else
+        sql3 = sql3.arg( "NULL", "NULL" );
+    if ( trackData.contains( Field::TRACKGAIN ) && trackData.contains( Field::TRACKPEAKGAIN ) )
+    {
+        sql3 = sql3.arg( QString::number( trackData[ Field::TRACKGAIN ].toDouble() ) );
+        sql3 = sql3.arg( trackData[ Field::TRACKPEAKGAIN ].toDouble() );
+    }
+    else
+        sql3 = sql3.arg( "NULL", "NULL" );
+
+    sql += sql2 + sql3;
 
     m_collection->query( sql );
 
@@ -482,7 +477,7 @@ ScanResultProcessor::imageId( const QString &image, int albumId )
     return imageId;
 }
 
-int 
+int
 ScanResultProcessor::albumId( const QString &album, int artistId )
 {
     //artistId == 0 means no albumartist
@@ -491,7 +486,7 @@ ScanResultProcessor::albumId( const QString &album, int artistId )
         return m_albums.value( key );
 
     QString query;
-    if( artistId == 0 ) 
+    if( artistId == 0 )
     {
         query = QString( "SELECT id FROM albums_temp WHERE artist IS NULL AND name = '%1';" )
                     .arg( m_collection->escape( album ) );
@@ -617,54 +612,53 @@ ScanResultProcessor::directoryId( const QString &dir )
 }
 
 int
-ScanResultProcessor::checkExistingAlbums( const QString &album )
+ScanResultProcessor::checkAlbumArtists( const QFileInfo &file, const QString &album, int artistId )
 {
-    // "Unknown" albums shouldn't be handled as compilations
     if( album.isEmpty() )
         return 0;
 
-    //check if this album already exists, ignoring the albumartist
-    //if it does, and if each file of the album is alone in its directory
-    //it's probably a compilation.
-    QString query = "SELECT urls.deviceid,urls.rpath,tracks_temp.id,albums.id,albums.artist FROM urls_temp AS urls "
-                    "LEFT JOIN tracks_temp on urls.id = tracks_temp.url LEFT JOIN albums_temp AS albums ON "
-                    "tracks_temp.album = albums.id WHERE albums.name = '%1';";
-    query = query.arg( m_collection->escape( album ) );
+    //If the album has a track from another artist, it is a compilation
+    QString query = "SELECT u.deviceid, u.rpath, t.id, t.artist "
+                    "FROM tracks_temp t "
+                    "INNER JOIN albums_temp a "
+                      "ON a.id = t.album "
+                    "INNER JOIN urls_temp u "
+                      "ON u.id = t.url "
+                    "WHERE t.album = a.id "
+                      "AND a.name = '%1' "
+                      "AND t.artist <> '%2';";
+    query = query.arg( m_collection->escape( album ), QString::number( artistId ) );
     QStringList result = m_collection->query( query );
-    QList<QString> trackIds;
+
+    //No tracks in the album are from an artist other than the one specified
+    if( result.isEmpty() ) 
+        return 0;
+    
+    //Calculate a list of tracks with a different artist, same album and same directory
+    QStringList trackIds;
     for( QListIterator<QString> iter( result ); iter.hasNext(); )
     {
         int deviceid = iter.next().toInt();
         QString rpath = iter.next();
         QString trackId = iter.next();
-        QString albumId = iter.next();
-        QString albumArtistId = iter.next();
+        QString artistId = iter.next();
         QString currentPath = MountPointManager::instance()->getAbsolutePath( deviceid, rpath );
         QFileInfo info( currentPath );
-        uint dirCount = m_filesInDirs.value( info.dir().absolutePath() );
-        if( dirCount == 1 )
-        {
+        if( info.dir() == file.dir() )
             trackIds << trackId;
-        }
     }
-    
-    if( trackIds.isEmpty() )
-    {
-        return 0;
-    }
-    else
+
+    if( !trackIds.isEmpty() )
     {
         int compilationId = albumId( album, 0 );
-        QString trackIdsSql = "-1";
-        foreach( const QString &trackId, trackIds )
-        {
-            trackIdsSql += ',';
-            trackIdsSql += trackId;
-        }
-        QString update = "UPDATE tracks_temp SET album = %1 where id IN (%2);";
-        m_collection->query( update.arg( QString::number( compilationId ), trackIdsSql ) );
+        const QString trackIdsInAlbum = trackIds.join(",");;
+        query = "UPDATE tracks_temp SET album = %1 where id IN (%2);";
+        query = query.arg( QString::number( compilationId ), trackIdsInAlbum );
+        m_collection->query( query );
         return compilationId;
     }
+
+    return 0;
 }
 
 void
@@ -674,13 +668,9 @@ ScanResultProcessor::setupDatabase()
     {
         m_collection->dbUpdater()->createTemporaryTables();
         if( m_type == IncrementalScan )
-        {
             m_collection->dbUpdater()->prepareTemporaryTables();
-        }
         else
-        {
             m_collection->dbUpdater()->prepareTemporaryTablesForFullScan();
-        }
         m_setupComplete = true;
     }
 }
