@@ -22,7 +22,7 @@
 
 #include "CollectionManager.h"
 
-#include <QHash>
+#include <QFileInfo>
 #include <QList>
 #include <QMap>
 #include <QPair>
@@ -74,7 +74,7 @@ class ScanResultProcessor : public QObject
         void updateAftPermanentTablesUrlString( const QString &url, const QString &uid );
         void updateAftPermanentTablesUidString( const QString &url, const QString &uid );
 
-        int checkExistingAlbums( const QString &album );
+        int checkAlbumArtists( const QFileInfo &file, const QString &album, int artistId );
 
         QString findAlbumArtist( const QSet<QString> &artists ) const;
         void setupDatabase();
@@ -91,8 +91,6 @@ class ScanResultProcessor : public QObject
         QMap<QPair<QString, int>, int> m_images;
         QMap<QString, int> m_directories;
 
-        QHash<QString, uint> m_filesInDirs;
-
         TrackUrls m_changedUrls;
 
         ScanType m_type;
