@@ -28,6 +28,8 @@
 
 #include <QPointer>
 
+#include <KIcon>
+
 class SqlCollectionFactory : public Amarok::CollectionFactory
 {
     Q_OBJECT
@@ -59,6 +61,7 @@ class SqlCollection : public Amarok::Collection, public SqlStorage
         virtual QString uidUrlProtocol() const;
         virtual QString collectionId() const;
         virtual QString prettyName() const;
+        virtual KIcon icon() const { return KIcon("drive-harddisk"); }
 
         SqlRegistry* registry() const;
         DatabaseUpdater* dbUpdater() const;
@@ -75,7 +78,6 @@ class SqlCollection : public Amarok::Collection, public SqlStorage
         virtual bool isWritable() const;
         virtual bool isOrganizable() const;
 
-
         //sqlcollection internal methods
         void sendChangedSignal();
 
@@ -99,8 +101,12 @@ class SqlCollection : public Amarok::Collection, public SqlStorage
 
         virtual void vacuum() const;
 
+        virtual bool hasCapabilityInterface( Meta::Capability::Type type ) const;
+        virtual Meta::Capability* asCapabilityInterface( Meta::Capability::Type type );
+
     public slots:
         void updateTrackUrls( TrackUrls changedUrls );
+        void deleteTracksSlot( Meta::TrackList tracklist );
 
     signals:
         void scanFinished();
