@@ -21,19 +21,25 @@
 
 #include "amarokconfig.h"
 #include "Amarok.h"
+#include "amarokurls/BookmarkMetaActions.h"
 #include "Debug.h"
 #include "MetaUtility.h"
+#include "SqlBookmarkThisCapability.h"
 #include "SqlCollection.h"
 #include "SqlQueryMaker.h"
 #include "SqlRegistry.h"
 #include "context/popupdropper/libpud/PopupDropperAction.h"
 #include "covermanager/CoverFetcher.h"
 #include "covermanager/CoverFetchingActions.h"
-#include "meta/CustomActionsCapability.h"
-#include "meta/EditCapability.h"
-#include "meta/ImportCapability.h"
-#include "meta/OrganiseCapability.h"
-#include "meta/UpdateCapability.h"
+#include "meta/capabilities/CustomActionsCapability.h"
+#include "meta/capabilities/CurrentTrackActionsCapability.h"
+#include "meta/capabilities/EditCapability.h"
+#include "meta/capabilities/StatisticsCapability.h"
+#include "meta/capabilities/TimecodeLoadCapability.h"
+#include "meta/capabilities/TimecodeWriteCapability.h"
+#include "meta/capabilities/OrganiseCapability.h"
+#include "meta/capabilities/UpdateCapability.h"
+#include "amarokurls/PlayUrlRunner.h"
 #include "MountPointManager.h"
 //#include "mediadevice/CopyToDeviceAction.h"
 
@@ -75,11 +81,11 @@ class EditCapabilityImpl : public Meta::EditCapability
         KSharedPtr<SqlTrack> m_track;
 };
 
-class ImportCapabilityImpl : public Meta::ImportCapability
+class StatisticsCapabilityImpl : public Meta::StatisticsCapability
 {
     public:
-        ImportCapabilityImpl( SqlTrack *track )
-            : Meta::ImportCapability()
+        StatisticsCapabilityImpl( SqlTrack *track )
+            : Meta::StatisticsCapability()
             , m_track( track ) {}
 
         virtual void setScore( const int score ) {
@@ -158,6 +164,57 @@ class UpdateCapabilityImpl : public Meta::UpdateCapability
         KSharedPtr<SqlTrack> m_track;
 };
 
+class TimecodeWriteCapabilityImpl : public Meta::TimecodeWriteCapability
+{
+    Q_OBJECT
+    public:
+        TimecodeWriteCapabilityImpl( SqlTrack *track )
+        : Meta::TimecodeWriteCapability()
+        , m_track( track )
+        {}
+
+        virtual bool writeTimecode ( int seconds )
+        {
+            DEBUG_BLOCK
+            return Meta::TimecodeWriteCapability::writeTimecode( seconds, Meta::TrackPtr( m_track.data() ) );
+        }
+
+        virtual bool writeAutoTimecode ( int seconds )
+        {
+            DEBUG_BLOCK
+            return Meta::TimecodeWriteCapability::writeAutoTimecode( seconds, Meta::TrackPtr( m_track.data() ) );
+        }
+
+    private:
+        KSharedPtr<SqlTrack> m_track;
+};
+
+class TimecodeLoadCapabilityImpl : public Meta::TimecodeLoadCapability
+{
+    Q_OBJECT
+    public:
+        TimecodeLoadCapabilityImpl( SqlTrack *track )
+        : Meta::TimecodeLoadCapability()
+        , m_track( track )
+        {}
+
+        virtual bool hasTimecodes()
+        {
+            if ( loadTimecodes().size() > 0 )
+                return true;
+            return false;
+        }
+
+        virtual QList<KSharedPtr<AmarokUrl> > loadTimecodes()
+        {
+            QList<KSharedPtr<AmarokUrl> > list = PlayUrlRunner::bookmarksFromUrl( m_track->playableUrl() );
+            return list;
+        }
+
+    private:
+        KSharedPtr<SqlTrack> m_track;
+};
+
 QString
 SqlTrack::getTrackReturnValues()
 {
@@ -170,6 +227,8 @@ SqlTrack::getTrackReturnValues()
            "tracks.filesize, tracks.samplerate, "
            "statistics.createdate, statistics.accessdate, "
            "statistics.playcount, tracks.filetype, tracks.bpm, "
+           "tracks.albumgain, tracks.albumpeakgain, "
+           "tracks.trackgain, tracks.trackpeakgain, "
            "artists.name, artists.id, "
            "albums.name, albums.id, albums.artist, "
            "genres.name, genres.id, "
@@ -177,6 +236,18 @@ SqlTrack::getTrackReturnValues()
            "years.name, years.id";
 }
 
+QString
+SqlTrack::getTrackJoinConditions()
+{
+    return "LEFT JOIN tracks ON urls.id = tracks.url "
+           "LEFT JOIN statistics ON urls.id = statistics.url "
+           "LEFT JOIN artists ON tracks.artist = artists.id "
+           "LEFT JOIN albums ON tracks.album = albums.id "
+           "LEFT JOIN genres ON tracks.genre = genres.id "
+           "LEFT JOIN composers ON tracks.composer = composers.id "
+           "LEFT JOIN years ON tracks.year = years.id";
+}
+
 int
 SqlTrack::getTrackReturnValueCount()
 {
@@ -187,34 +258,27 @@ SqlTrack::getTrackReturnValueCount()
 TrackPtr
 SqlTrack::getTrack( int deviceid, const QString &rpath, SqlCollection *collection )
 {
-    QString query = "SELECT %1 FROM urls "
-                    "LEFT JOIN tracks ON urls.id = tracks.url "
-                    "LEFT JOIN statistics ON urls.id = statistics.url "
-                    "LEFT JOIN artists ON tracks.artist = artists.id "
-                    "LEFT JOIN albums ON tracks.album = albums.id "
-                    "LEFT JOIN genres ON tracks.genre = genres.id "
-                    "LEFT JOIN composers ON tracks.composer = composers.id "
-                    "LEFT JOIN years ON tracks.year = years.id "
-                    "WHERE urls.deviceid = %2 AND urls.rpath = '%3';";
-    query = query.arg( SqlTrack::getTrackReturnValues(), QString::number( deviceid ), collection->escape( rpath ) );
+    QString query = "SELECT %1 FROM urls %2 "
+                    "WHERE urls.deviceid = %3 AND urls.rpath = '%4';";
+    query = query.arg( getTrackReturnValues(), getTrackJoinConditions(),
+                       QString::number( deviceid ), collection->escape( rpath ) );
     QStringList result = collection->query( query );
     if( result.isEmpty() )
         return TrackPtr();
-    else
-        return TrackPtr( new SqlTrack( collection, result ) );
+    return TrackPtr( new SqlTrack( collection, result ) );
 }
 
 TrackPtr
 SqlTrack::getTrackFromUid( const QString &uid, SqlCollection* collection )
 {
-    QString query = "SELECT urls.deviceid, urls.rpath FROM urls "
-                    "WHERE urls.uniqueid = '%1';";
-    query = query.arg( collection->escape( uid ) );
+    QString query = "SELECT %1 FROM urls %2 "
+                    "WHERE urls.uniqueid = '%3';";
+    query = query.arg( getTrackReturnValues(), getTrackJoinConditions(),
+                       collection->escape( uid ) );
     QStringList result = collection->query( query );
     if( result.isEmpty() )
         return TrackPtr();
-    else
-        return getTrack( result[0].toInt(), result[1], collection );    
+    return TrackPtr( new SqlTrack( collection, result ) );
 }
 
 SqlTrack::SqlTrack( SqlCollection* collection, const QStringList &result )
@@ -245,6 +309,23 @@ SqlTrack::SqlTrack( SqlCollection* collection, const QStringList &result )
     ++iter; //file type
     ++iter; //BPM
 
+    // if there is no track gain, we assume a gain of 0
+    // if there is no album gain, we use the track gain
+    QString albumGain = *(iter++);
+    QString albumPeakGain = *(iter++);
+    m_trackGain = (*(iter++)).toDouble();
+    m_trackPeakGain = (*(iter++)).toDouble();
+    if ( albumGain.isEmpty() )
+    {
+        m_albumGain = m_trackGain;
+        m_albumPeakGain = m_trackPeakGain;
+    }
+    else
+    {
+        m_albumGain = albumGain.toDouble();
+        m_albumPeakGain = albumPeakGain.toDouble();
+    }
+
     SqlRegistry* registry = m_collection->registry();
     QString artist = *(iter++);
     int artistId = (*(iter++)).toInt();
@@ -301,7 +382,7 @@ SqlTrack::fullPrettyName() const
         s = i18n("%1 - %2", m_artist->name(), name() );
 
     //TODO
-    if( s.isEmpty() ) 
+    if( s.isEmpty() )
         s = prettyTitle( m_url.fileName() );
 
     return s;
@@ -338,7 +419,7 @@ SqlTrack::setUrl( const QString &url )
     m_deviceid = MountPointManager::instance()->getIdForUrl( url );
     m_rpath = MountPointManager::instance()->getRelativePath( m_deviceid, url );
     if( m_batchUpdate )
-        m_cache.insert( Meta::Field::URL, MountPointManager::instance()->getAbsolutePath( m_deviceid, m_rpath ) ); 
+        m_cache.insert( Meta::Field::URL, MountPointManager::instance()->getAbsolutePath( m_deviceid, m_rpath ) );
     else
     {
         m_url = url;
@@ -365,6 +446,9 @@ SqlTrack::setUrl( const int deviceid, const QString &rpath )
 void
 SqlTrack::setArtist( const QString &newArtist )
 {
+    if ( m_artist && m_artist->name() == newArtist )
+        return;
+
     if( m_batchUpdate )
         m_cache.insert( Meta::Field::ARTIST, newArtist );
     else
@@ -383,6 +467,9 @@ SqlTrack::setArtist( const QString &newArtist )
 void
 SqlTrack::setGenre( const QString &newGenre )
 {
+    if ( m_genre && m_genre->name() == newGenre )
+        return;
+
     if( m_batchUpdate )
         m_cache.insert( Meta::Field::GENRE, newGenre );
     else
@@ -399,6 +486,9 @@ SqlTrack::setGenre( const QString &newGenre )
 void
 SqlTrack::setComposer( const QString &newComposer )
 {
+    if ( m_composer && m_composer->name() == newComposer )
+        return;
+
     if( m_batchUpdate )
         m_cache.insert( Meta::Field::COMPOSER, newComposer );
     else
@@ -415,6 +505,9 @@ SqlTrack::setComposer( const QString &newComposer )
 void
 SqlTrack::setYear( const QString &newYear )
 {
+    if ( m_year && m_year->name() == newYear )
+        return;
+
     if( m_batchUpdate )
         m_cache.insert( Meta::Field::YEAR, newYear );
     else
@@ -431,6 +524,9 @@ SqlTrack::setYear( const QString &newYear )
 void
 SqlTrack::setAlbum( const QString &newAlbum )
 {
+    if ( m_album && m_album->name() == newAlbum )
+        return;
+
     if( m_batchUpdate )
         m_cache.insert( Meta::Field::ALBUM, newAlbum );
     else
@@ -721,6 +817,8 @@ SqlTrack::updateStatisticsInDb()
     QString query = "SELECT urls.id FROM urls WHERE urls.deviceid = %1 AND urls.rpath = '%2';";
     query = query.arg( QString::number( m_deviceid ), m_collection->escape( m_rpath ) );
     QStringList res = m_collection->query( query );
+    if( res.isEmpty() )
+        return; // No idea why this happens.. but it does
     int urlId = res[0].toInt();
     QStringList count = m_collection->query( QString( "SELECT count(*) FROM statistics WHERE url = %1;" ).arg( urlId ) );
     if( count[0].toInt() == 0 )
@@ -830,11 +928,17 @@ SqlTrack::hasCapabilityInterface( Meta::Capability::Type type ) const
     switch( type )
     {
         case Meta::Capability::CustomActions:
+        case Meta::Capability::Importable:
         case Meta::Capability::Organisable:
         case Meta::Capability::Updatable:
+        case Meta::Capability::CurrentTrackActions:
+        case Meta::Capability::WriteTimecode:
+        case Meta::Capability::LoadTimecode:
             return true;
+
         case Meta::Capability::Editable:
             return isEditable();
+
         default:
             return false;
     }
@@ -849,7 +953,7 @@ SqlTrack::asCapabilityInterface( Meta::Capability::Type type )
             return new EditCapabilityImpl( this );
 
         case Meta::Capability::Importable:
-            return new ImportCapabilityImpl( this );
+            return new StatisticsCapabilityImpl( this );
 
         case Meta::Capability::CustomActions:
         {
@@ -867,6 +971,19 @@ SqlTrack::asCapabilityInterface( Meta::Capability::Type type )
         case Meta::Capability::Updatable:
             return new UpdateCapabilityImpl( this );
 
+        case Meta::Capability::CurrentTrackActions:
+        {
+            QList< PopupDropperAction * > actions;
+            PopupDropperAction* flag = new BookmarkCurrentTrackPositionAction( m_collection );
+            actions << flag;
+            debug() << "returning bookmarkcurrenttrack action";
+            return new Meta::CurrentTrackActionsCapability( actions );
+        }
+        case Meta::Capability::WriteTimecode:
+            return new TimecodeWriteCapabilityImpl( this );
+        case Meta::Capability::LoadTimecode:
+            return new TimecodeLoadCapabilityImpl( this );
+
         default:
             return 0;
     }
@@ -881,10 +998,17 @@ SqlArtist::SqlArtist( SqlCollection* collection, int id, const QString &name ) :
     ,m_tracksLoaded( false )
     ,m_albumsLoaded( false )
     ,m_mutex( QMutex::Recursive )
+    ,m_bookmarkAction( 0 )
 {
     //nothing to do (yet)
 }
 
+Meta::SqlArtist::~SqlArtist()
+{
+    delete m_bookmarkAction;
+}
+
+
 void
 SqlArtist::invalidateCache()
 {
@@ -960,6 +1084,8 @@ SqlArtist::hasCapabilityInterface( Meta::Capability::Type type ) const
 {
     switch( type )
     {
+        case Meta::Capability::BookmarkThis:
+            return true;
         default:
             return false;
     }
@@ -970,6 +1096,12 @@ SqlArtist::asCapabilityInterface( Meta::Capability::Type type )
 {
     switch( type )
     {
+        case Meta::Capability::BookmarkThis:
+        {
+            if ( !m_bookmarkAction )
+                m_bookmarkAction = new BookmarkArtistAction( 0, ArtistPtr( this ) );
+            return new SqlBookmarkThisCapability( m_bookmarkAction );
+        }
         default:
             return 0;
     }
@@ -1018,10 +1150,16 @@ SqlAlbum::SqlAlbum( SqlCollection* collection, int id, const QString &name, int
     , m_tracksLoaded( false )
     , m_artist()
     , m_mutex( QMutex::Recursive )
+    , m_bookmarkAction( 0 )
 {
     //nothing to do
 }
 
+Meta::SqlAlbum::~SqlAlbum()
+{
+    delete m_bookmarkAction;
+}
+
 void
 SqlAlbum::invalidateCache()
 {
@@ -1126,9 +1264,9 @@ SqlAlbum::image( int size )
     // subscribers. We use queueAlbum() because this runs the fetch as a
     // background job and doesn't give an intruding popup asking for confirmation
     if( !m_name.isEmpty() && AmarokConfig::autoGetCoverArt() )
-        CoverFetcher::instance()->queueAlbum( KSharedPtr<Meta::Album>(this) );
+        CoverFetcher::instance()->queueAlbum( AlbumPtr(this) );
 
-    // If the result image is empty then we didn't find any cached image, nor 
+    // If the result image is empty then we didn't find any cached image, nor
     // could we find the original cover to scale to the appropriate size. Hence,
     // the album cannot have a cover, for any size. In this case, we return the
     // default image
@@ -1150,31 +1288,31 @@ SqlAlbum::imageLocation( int size )
     QPixmap i = image( size );
     if( m_images.contains( size ) )
         return KUrl( m_images.value( size ) );
-    
+
     return KUrl();
 }
 
 void
-SqlAlbum::setImage( const QImage &image )
+SqlAlbum::setImage( const QPixmap &pixmap )
 {
-    if( image.isNull() )
+    if( pixmap.isNull() )
         return;
 
-    QByteArray widthKey = QString::number( image.width() ).toLocal8Bit() + '@';
+    QByteArray widthKey = QString::number( pixmap.width() ).toLocal8Bit() + '@';
     QString album = m_name;
     QString artist = hasAlbumArtist() ? albumArtist()->name() : QString();
 
     if( artist.isEmpty() && album.isEmpty() )
         return;
 
-    // removeImage() will destroy all scaled cached versions of the artwork 
+    // removeImage() will destroy all scaled cached versions of the artwork
     // and remove references from the database if required.
     if( hasImage( -1 ) ) // -1 is a dummy
         removeImage();
 
     QByteArray key = md5sum( artist, album, QString() );
     QString path = Amarok::saveLocation( "albumcovers/large/" ) + key;
-    image.save( path, "JPG" );
+    pixmap.save( path, "JPG" );
 
     updateImage( path );
 
@@ -1208,9 +1346,9 @@ SqlAlbum::removeImage()
     }
 
     // TODO: remove directory image ??
-    
+
     // Update the database image path
-    
+
     QString query = "SELECT images.id, count( images.id ) FROM images, albums "
                     "WHERE albums.image = images.id AND albums.id = %1 "
                     "GROUP BY images.id";
@@ -1259,7 +1397,7 @@ SqlAlbum::unsetImageId() const
         return m_unsetImageId;
 
     QString query = "SELECT id FROM images WHERE path = '%1'";
-    QStringList res = m_collection->query( query.arg( AMAROK_UNSET_MAGIC ) ); 
+    QStringList res = m_collection->query( query.arg( AMAROK_UNSET_MAGIC ) );
 
     // We already have the AMAROK_UNSET_MAGIC variable in the database
     if( !res.isEmpty() )
@@ -1349,7 +1487,7 @@ SqlAlbum::findLargeCachedImage() const
     {
         const QString filePath = largeCoverDir.filePath( key );
         updateImage( filePath ); // We found a previously downloaded large image, let's keep it for future reference.
-        return filePath; 
+        return filePath;
     }
     return QString();
 }
@@ -1380,7 +1518,7 @@ SqlAlbum::createScaledImage( QString path, int size ) const
             QImage img( path );
             if( img.isNull() )
                 return QString();
-           
+
             // resize and save the image
             img.scaled( size, size, Qt::KeepAspectRatio, Qt::SmoothTransformation ).save( cachedImagePath, "JPG" );
         }
@@ -1397,9 +1535,9 @@ SqlAlbum::findImage( int size )
         return m_images.value( size );
 
     QString fullsize;
-    
+
     // get the full size path from the cache if we have it
-    if( m_images.contains( 0 ) ) 
+    if( m_images.contains( 0 ) )
     {
         fullsize = m_images.value( 0 );
     }
@@ -1411,7 +1549,7 @@ SqlAlbum::findImage( int size )
         if( !res.isEmpty() )
         {
             fullsize = res.first();
-            if( !fullsize.isEmpty() ) 
+            if( !fullsize.isEmpty() )
                 m_images.insert( 0, fullsize ); // store the full size version
         }
     }
@@ -1486,13 +1624,14 @@ SqlAlbum::setCompilation( bool compilation )
         else
         {
             debug() << "User selected album as non-compilation";
-            
+
             QString select = "SELECT artist FROM tracks WHERE album = %1";
             QStringList artistid = m_collection->query( select.arg( m_id ) );
-            
+
             m_artistId = artistid[0].toInt();
-            m_artist = this->tracks()[0]->artist();
-            
+            if( tracks().size() > 0 )
+                m_artist = this->tracks()[0]->artist();
+
             QString update = "UPDATE albums SET artist = %1 WHERE id = %2;";
             update = update.arg( m_artistId ).arg( m_id );
             m_collection->query( update );
@@ -1509,7 +1648,8 @@ SqlAlbum::hasCapabilityInterface( Meta::Capability::Type type ) const
     {
         case Meta::Capability::CustomActions:
             return true;
-
+        case Meta::Capability::BookmarkThis:
+            return true;
         default:
             return false;
     }
@@ -1524,7 +1664,7 @@ SqlAlbum::asCapabilityInterface( Meta::Capability::Type type )
         {
             QList<PopupDropperAction*> actions;
             actions.append( new CompilationAction( m_collection, this ) );
-            
+
             PopupDropperAction *separator          = new PopupDropperAction( m_collection );
             PopupDropperAction *displayCoverAction = new DisplayCoverAction( m_collection, Meta::AlbumPtr(this) );
             PopupDropperAction *unsetCoverAction   = new UnsetCoverAction( m_collection, Meta::AlbumPtr(this) );
@@ -1542,6 +1682,12 @@ SqlAlbum::asCapabilityInterface( Meta::Capability::Type type )
             actions.append( unsetCoverAction );
             return new CustomActionsCapability( actions );
         }
+        case Meta::Capability::BookmarkThis:
+        {
+            if ( !m_bookmarkAction )
+                m_bookmarkAction = new BookmarkAlbumAction( 0, AlbumPtr( this ) );
+            return new SqlBookmarkThisCapability( m_bookmarkAction );
+        }
 
         default:
             return 0;
@@ -1683,5 +1829,7 @@ SqlYear::tracks()
         return TrackList();
 }
 
+
+
 #include "sqlmeta.moc"
 
