@@ -30,6 +30,7 @@
 #include <QVariant>
 
 class SqlCollection;
+class PopupDropperAction;
 
 namespace Meta
 {
@@ -90,33 +91,38 @@ class SqlTrack : public Meta::Track
 
         virtual QString comment() const { return m_comment; }
         virtual void setComment( const QString &newComment );
-        
+
         virtual double score() const { return m_score; }
         virtual void setScore( double newScore );
-        
+
         virtual int rating() const { return m_rating; }
         virtual void setRating( int newRating );
-        
+
         virtual int length() const { return m_length; }
         virtual int filesize() const { return m_filesize; }
         virtual int sampleRate() const { return m_sampleRate; }
         virtual int bitrate() const { return m_bitrate; }
-        
+
         virtual int trackNumber() const { return m_trackNumber; }
         virtual void setTrackNumber( int newTrackNumber );
-        
+
         virtual int discNumber() const { return m_discNumber; }
         virtual void setDiscNumber( int newDiscNumber );
-        
+
         virtual uint firstPlayed() const { return m_firstPlayed; }
         virtual void setFirstPlayed( const uint newTime );
-        
+
         virtual uint lastPlayed() const { return m_lastPlayed; }
         virtual void setLastPlayed( const uint newTime );
-        
+
         virtual int playCount() const { return m_playCount; }
         virtual void setPlayCount( const int newCount );
-        
+
+        virtual qreal replayGain( ReplayGainMode mode ) const
+        { return ( mode == AlbumReplayGain ) ? m_albumGain : m_trackGain; }
+        virtual qreal replayPeakGain( ReplayGainMode mode ) const
+        { return ( mode == AlbumReplayGain ) ? m_albumPeakGain : m_trackPeakGain; }
+
         virtual void setUidUrl( const QString &uid );
 
         virtual void beginMetaDataUpdate();
@@ -149,6 +155,9 @@ class SqlTrack : public Meta::Track
         void updateStatisticsInDb();
 
     private:
+        /** returns a string of all database joins that are required to fetch all values for a track*/
+        static QString getTrackJoinConditions();
+
         SqlCollection* m_collection;
 
         QString m_title;
@@ -171,6 +180,10 @@ class SqlTrack : public Meta::Track
         double m_score;
         QString m_comment;
         QString m_uid;
+        qreal m_albumGain;
+        qreal m_albumPeakGain;
+        qreal m_trackGain;
+        qreal m_trackPeakGain;
 
         Meta::AlbumPtr m_album;
         Meta::ArtistPtr m_artist;
@@ -187,6 +200,7 @@ class SqlArtist : public Meta::Artist
 {
     public:
         SqlArtist( SqlCollection* collection, int id, const QString &name );
+        ~SqlArtist();
 
         virtual QString name() const { return m_name; }
         virtual QString prettyName() const { return m_name; } //change if necessary
@@ -219,6 +233,7 @@ class SqlArtist : public Meta::Artist
         //see http://www.trolltech.com/developer/task-tracker/index_html?method=entry&id=131880
         //switch to QReadWriteLock as soon as it does!
         QMutex m_mutex;
+        PopupDropperAction * m_bookmarkAction;
 
 };
 
@@ -226,6 +241,7 @@ class SqlAlbum : public Meta::Album
 {
     public:
         SqlAlbum( SqlCollection* collection, int id, const QString &name, int artist );
+        ~SqlAlbum();
 
         virtual QString name() const { return m_name; }
         virtual QString prettyName() const { return m_name; }
@@ -244,7 +260,7 @@ class SqlAlbum : public Meta::Album
         virtual bool canUpdateImage() const { return true; }
         virtual QPixmap image( int size = 1 );
         virtual KUrl imageLocation( int size = 1 );
-        virtual void setImage( const QImage &image );
+        virtual void setImage( const QPixmap &pixmap );
         virtual void removeImage();
 
         virtual bool hasCapabilityInterface( Meta::Capability::Type type ) const;
@@ -285,6 +301,8 @@ class SqlAlbum : public Meta::Album
         //switch to QReadWriteLock as soon as it does!
         QMutex m_mutex;
 
+        PopupDropperAction * m_bookmarkAction;
+
         //TODO: add album artist
 };
 
