@@ -42,6 +42,8 @@ class SqlRegistry : public QObject
 
         Meta::TrackPtr getTrack( const QString &url );
         Meta::TrackPtr getTrack( const QStringList &rowData );
+        Meta::TrackPtr getTrackFromUid( const QString &uid );
+        bool checkUidExists( const QString &uid );
 
         Meta::ArtistPtr getArtist( const QString &name, int id = -1 );
         Meta::GenrePtr getGenre( const QString &name, int id = -1 );
@@ -57,6 +59,7 @@ class SqlRegistry : public QObject
 
         //we don't care about the ordering so use the faster QHash
         QHash<TrackId, Meta::TrackPtr > m_trackMap;
+        QHash<QString, Meta::TrackPtr > m_uidMap;
         QHash<int, Meta::ArtistPtr > m_artistMap;
         QHash<int, Meta::ComposerPtr > m_composerMap;
         QHash<int, Meta::GenrePtr > m_genreMap;
@@ -69,6 +72,7 @@ class SqlRegistry : public QObject
         QMutex m_genreMutex;
         QMutex m_yearMutex;
         QMutex m_albumMutex;
+        QMutex m_uidMutex;
 
         QTimer *m_timer;
 
