@@ -15,13 +15,13 @@
 #define AMAROK_APPLET_H
 
 #include "amarok_export.h"
-#include "plasma/applet.h"
+
+#include <plasma/applet.h>
 
 #include <QFont>
 #include <QRectF>
 #include <QString>
 
-
 namespace Context
 {
 
@@ -29,7 +29,6 @@ class AMAROK_EXPORT Applet : public Plasma::Applet
 {
 
     public:
-
         explicit Applet( QObject* parent, const QVariantList& args = QVariantList() );
 
         //helper functions
@@ -38,6 +37,7 @@ class AMAROK_EXPORT Applet : public Plasma::Applet
         
     public Q_SLOTS:
         virtual void destroy();
+
     private:
         void cleanUpAndDelete();
 
