@@ -17,17 +17,18 @@
 #include "context/Applet.h"
 #include "context/DataEngine.h"
 #include "context/Svg.h"
-#include "plasma/framesvg.h"
-#include "plasma/widgets/iconwidget.h"
+
+#include <plasma/widgets/iconwidget.h>
 
 
 class QGraphicsSimpleTextItem;
 class QGraphicsProxyWidget;
-class QTextEdit;
+class QTextBrowser;
 
 class LyricsApplet : public Context::Applet
 {
     Q_OBJECT
+
 public:
     LyricsApplet( QObject* parent, const QVariantList& args );
     ~LyricsApplet();
@@ -58,11 +59,8 @@ private:
     
     // holds main body
     QGraphicsProxyWidget *m_lyricsProxy;
-    QTextEdit* m_lyrics;
-    qreal m_aspectRatio;
+    QTextBrowser* m_lyrics;
     QGraphicsTextItem* m_suggested;
-
-    Plasma::FrameSvg* m_theme;
 };
 
 K_EXPORT_AMAROK_APPLET( lyrics, LyricsApplet )
