@@ -17,10 +17,16 @@
 #include "Debug.h"
 #include "context/Svg.h"
 #include "EngineController.h"
-#include <plasma/theme.h>
 
+#include <plasma/theme.h>
+#include <plasma/widgets/webview.h>
+#include <plasma/widgets/iconwidget.h>
+
+#include <KIcon>
 #include <KStandardDirs>
 
+#include <QAction>
+#include <QDesktopServices>
 #include <QGraphicsSimpleTextItem>
 #include <QPainter>
 
@@ -33,6 +39,7 @@ WikipediaApplet::WikipediaApplet( QObject* parent, const QVariantList& args )
     , m_size( QSizeF() )
     , m_wikipediaLabel( 0 )
     , m_webView( 0 )
+    , m_reloadIcon( 0 )
 {
     setHasConfigurationInterface( false );
     setBackgroundHints( Plasma::Applet::NoBackground );
@@ -40,12 +47,6 @@ WikipediaApplet::WikipediaApplet( QObject* parent, const QVariantList& args )
 
 WikipediaApplet::~ WikipediaApplet()
 {
-    //hacky stuff to keep QWebView from causing a crash
-   /* m_wikiPage->setWidget( 0 );
-    delete m_wikiPage;
-    m_wikiPage = 0;
-    delete m_webView;*/
-
     delete m_webView;
 }
 
@@ -71,17 +72,12 @@ void WikipediaApplet::init()
     m_wikipediaLabel = new QGraphicsSimpleTextItem( this );
 
     m_webView = new Plasma::WebView( this );
-
+    m_webView->setAttribute( Qt::WA_NoSystemBackground );
+    
     m_webView->page()->settings()->setUserStyleSheetUrl( "file://" + KStandardDirs::locate("data", "amarok/data/WikipediaCustomStyle.css" ) );
     m_webView->page()->setLinkDelegationPolicy ( QWebPage::DelegateAllLinks );
     connect( m_webView->page(), SIGNAL( linkClicked( const QUrl & ) ) , this, SLOT( linkClicked ( const QUrl & ) ) );
 
-    //make background transparent
-    QPalette p = m_webView->palette();
-    p.setColor( QPalette::Dark, QColor( 255, 255, 255, 0)  );
-    p.setColor( QPalette::Window, QColor( 255, 255, 255, 0)  );
-    m_webView->setPalette( p );
-
     QFont labelFont;
     labelFont.setBold( true );
     labelFont.setPointSize( labelFont.pointSize() + 3 );
@@ -89,13 +85,46 @@ void WikipediaApplet::init()
     m_wikipediaLabel->setFont( labelFont );
     m_wikipediaLabel->setText( i18n( "Wikipedia" ) );
 
+    QAction* reloadAction = new QAction( i18n( "Reload" ), this );
+    reloadAction->setIcon( KIcon( "view-refresh" ) );
+    reloadAction->setVisible( true );
+    reloadAction->setEnabled( false );
+    m_reloadIcon = addAction( reloadAction );
+
+    connect( m_reloadIcon, SIGNAL( activated() ), this, SLOT( reloadWikipedia() ) );
+
     connectSource( "wikipedia" );
     connect( dataEngine( "amarok-wikipedia" ), SIGNAL( sourceAdded( const QString & ) ),
              this, SLOT( connectSource( const QString & ) ) );
-            
+
     constraintsEvent();
 }
 
+Plasma::IconWidget *
+WikipediaApplet::addAction( QAction *action )
+{
+    DEBUG_BLOCK
+    if ( !action ) {
+        debug() << "ERROR!!! PASSED INVALID ACTION";
+        return 0;
+    }
+    
+    Plasma::IconWidget *tool = new Plasma::IconWidget( this );
+    tool->setAction( action );
+    tool->setText( "" );
+    tool->setToolTip( action->text() );
+    tool->setDrawBackground( false );
+    tool->setOrientation( Qt::Horizontal );
+    QSizeF iconSize = tool->sizeFromIconSize( 16 );
+    tool->setMinimumSize( iconSize );
+    tool->setMaximumSize( iconSize );
+    tool->resize( iconSize );
+
+    tool->setZValue( zValue() + 1 );
+
+    return tool;
+}
+
 void
 WikipediaApplet::connectSource( const QString &source )
 {
@@ -106,10 +135,8 @@ WikipediaApplet::connectSource( const QString &source )
 void
 WikipediaApplet::linkClicked( const QUrl &url )
 {
-    DEBUG_BLOCK
     debug() << "URL: " << url;
-
-    Amarok::invokeBrowser( url.toString() );
+    QDesktopServices::openUrl( url.toString() );
 }
 
 void WikipediaApplet::constraintsEvent( Plasma::Constraints constraints )
@@ -137,6 +164,8 @@ void WikipediaApplet::constraintsEvent( Plasma::Constraints constraints )
 
     if ( infoSize.isValid() )
         m_webView->resize( infoSize );
+
+    m_reloadIcon->setPos( size().width() - m_reloadIcon->size().width() - MARGIN, MARGIN );
 }
 
 bool WikipediaApplet::hasHeightForWidth() const
@@ -146,7 +175,7 @@ bool WikipediaApplet::hasHeightForWidth() const
 
 qreal WikipediaApplet::heightForWidth( qreal width ) const
 {
-    return width * m_aspectRatio;;
+    return width * m_aspectRatio;
 }
 
 void WikipediaApplet::dataUpdated( const QString& name, const Plasma::DataEngine::Data& data ) // SLOT
@@ -170,6 +199,13 @@ void WikipediaApplet::dataUpdated( const QString& name, const Plasma::DataEngine
         m_title = data[ "title" ].toString();
     else
         m_title.clear();
+
+    if( m_reloadIcon->action() && !m_reloadIcon->action()->isEnabled() )
+    {        
+        m_reloadIcon->action()->setEnabled( true );
+        //for some reason when we enable the action suddenly the icon has the text "..."
+        m_reloadIcon->action()->setText( "" );  
+    }
 }
 
 void WikipediaApplet::paintInterface( QPainter *p, const QStyleOptionGraphicsItem *option, const QRect &contentsRect )
@@ -188,14 +224,20 @@ void WikipediaApplet::paintInterface( QPainter *p, const QStyleOptionGraphicsIte
 
 QSizeF WikipediaApplet::sizeHint( Qt::SizeHint which, const QSizeF & constraint ) const
 {
-    Q_UNUSED( which )
+    // TODO hardcoding for now.
+    // i want to have a system where an applet can ask
+    // for a full "CV pane" of size, but for now this will stop the crash
+    QSizeF size;
+    size.setWidth( QGraphicsWidget::sizeHint( which, constraint ).width() );
+    size.setHeight( 450 );
+    return size;
+}
 
-    if( constraint.height() == -1 && constraint.width() > 0 ) // asking height for given width basically
-    {
-        return QSizeF( constraint.width(), m_aspectRatio * constraint.width() );
-    }
-        
-    return constraint;
+void
+WikipediaApplet::reloadWikipedia()
+{
+    DEBUG_BLOCK
+    dataEngine( "amarok-wikipedia" )->query( "wikipedia:reload" );
 }
 
 #include "WikipediaApplet.moc"
