@@ -20,7 +20,7 @@
 #include "collection/Collection.h"
 #include "collection/CollectionManager.h"
 #include "meta/MetaUtility.h"
-#include "meta/SourceInfoCapability.h"
+#include "meta/capabilities/SourceInfoCapability.h"
 
 #include <QVariant>
 
@@ -31,6 +31,9 @@ CurrentEngine::CurrentEngine( QObject* parent, const QList<QVariant>& args )
     , ContextObserver( ContextView::self() )
     , m_coverWidth( 0 )
     , m_requested( true )
+	, m_qm( 0 )
+	, m_qmTracks( 0 )
+	, m_qmFavTracks( 0 )
     , m_currentArtist( 0 )
 {
     DEBUG_BLOCK
@@ -47,6 +50,15 @@ CurrentEngine::CurrentEngine( QObject* parent, const QList<QVariant>& args )
 CurrentEngine::~CurrentEngine()
 {
     DEBUG_BLOCK
+    if( m_qm )
+        m_qm->abortQuery();
+    delete m_qm;
+    if( m_qmTracks )
+        m_qmTracks->abortQuery();
+    delete m_qmTracks;
+    if( m_qmFavTracks )
+        m_qmFavTracks->abortQuery();
+    delete m_qmFavTracks;
 }
 
 QStringList CurrentEngine::sources() const
@@ -56,7 +68,7 @@ QStringList CurrentEngine::sources() const
     return m_sources; // we don't have sources, if connected, it is enabled.
 }
 
-bool CurrentEngine::sourceRequested( const QString& name )
+bool CurrentEngine::sourceRequestEvent( const QString& name )
 {
     DEBUG_BLOCK
     Q_UNUSED( name );
@@ -105,7 +117,10 @@ CurrentEngine::stoppedState()
     setData( "albums", "headerText", QVariant( i18n( "Recently added albums" ) ) );
     
     Amarok::Collection *coll = CollectionManager::instance()->primaryCollection();
-    m_qm = coll->queryMaker();
+    if( m_qm )
+		m_qm->reset();
+	else
+    	m_qm = coll->queryMaker();
     m_qm->setQueryType( QueryMaker::Album );
     m_qm->excludeFilter( Meta::valAlbum, QString(), true, true ); 
     m_qm->orderBy( Meta::valCreateDate, true );
@@ -120,7 +135,10 @@ CurrentEngine::stoppedState()
 
     // Get the latest tracks played:
 
-    m_qmTracks = coll->queryMaker();
+    if( m_qmTracks )
+		m_qmTracks->reset();
+	else
+    	m_qmTracks = coll->queryMaker();
     m_qmTracks->setQueryType( QueryMaker::Track );
     m_qmTracks->excludeFilter( Meta::valTitle, QString(), true, true );
     m_qmTracks->orderBy( Meta::valLastPlayed, true );
@@ -133,6 +151,23 @@ CurrentEngine::stoppedState()
     connect( m_qmTracks, SIGNAL( queryDone() ), SLOT( setupTracksData() ) );
 
     m_qmTracks->run();
+
+    // Get the favorite tracks:
+
+    if( m_qmFavTracks )
+		m_qmFavTracks->reset();
+	else
+    	m_qmFavTracks = coll->queryMaker();
+    m_qmFavTracks->setQueryType( QueryMaker::Track );
+    m_qmFavTracks->excludeFilter( Meta::valTitle, QString(), true, true );
+    m_qmFavTracks->orderBy( Meta::valScore, true );
+    m_qmFavTracks->limitMaxResultSize( 5 );
+
+    m_qmFavTracks->run();
+
+    connect( m_qmFavTracks, SIGNAL( newResultReady( QString, Meta::TrackList ) ),
+             SLOT( resultReady( QString, Meta::TrackList ) ), Qt::QueuedConnection );
+    connect( m_qmFavTracks, SIGNAL( queryDone() ), SLOT( setupTracksData() ) );
 }
 
 void CurrentEngine::metadataChanged( Meta::AlbumPtr album )
@@ -253,8 +288,16 @@ CurrentEngine::setupTracksData()
     DEBUG_BLOCK
 
     QVariant v;
-    v.setValue( m_latestTracks );
-    setData( "current", "tracks", v );
+    if( sender() == m_qmTracks )
+    {
+        v.setValue( m_latestTracks );
+        setData( "current", "lastTracks", v );
+    }
+    else if( sender() == m_qmFavTracks )
+    {
+        v.setValue( m_favoriteTracks );
+        setData( "current", "favoriteTracks", v );
+    }
 }
 
 void
@@ -272,9 +315,16 @@ CurrentEngine::resultReady( const QString &collectionId, const Meta::TrackList &
 {
     DEBUG_BLOCK
     Q_UNUSED( collectionId )
-
-    m_latestTracks.clear();
-    m_latestTracks << tracks;
+    if( sender() == m_qmTracks )
+    {
+        m_latestTracks.clear();
+        m_latestTracks << tracks;
+    }
+    else if( sender() == m_qmFavTracks )
+    {
+        m_favoriteTracks.clear();
+        m_favoriteTracks << tracks;
+    }
 }
 
 
