@@ -48,14 +48,14 @@ public:
     void newLyrics( QStringList& lyrics );
     void newLyricsHtml( QString& lyrics );
     void newSuggestions( QStringList& suggest );
-    void lyricsMessage( QString& message );
+    void lyricsMessage( QString& key, QString& val );
 
     // reimplemented from Meta::Observer
     using Observer::metadataChanged;
     void metadataChanged( Meta::TrackPtr track );
     
 protected:
-    bool sourceRequested( const QString& name );
+    bool sourceRequestEvent( const QString& name );
     
 private:
     void update();
