@@ -27,7 +27,6 @@ WikipediaEngine::WikipediaEngine( QObject* parent, const QList<QVariant>& /*args
     , ContextObserver( ContextView::self() )
     , m_wikiJob( 0 )
     , m_currentSelection( "artist" )
-    , m_wikiLocale( "en" )
     , m_requested( true )
     , m_sources( "current" )
     , m_triedRefinedSearch( false )
@@ -45,15 +44,23 @@ QStringList WikipediaEngine::sources() const
     return m_sources;
 }
 
-bool WikipediaEngine::sourceRequested( const QString& name )
+bool WikipediaEngine::sourceRequestEvent( const QString& name )
 {
     DEBUG_BLOCK
     Q_UNUSED( name )
 
     m_requested = true; // someone is asking for data, so we turn ourselves on :)
-    removeAllData( name );
-    setData( name, QVariant());
-    update();
+    QStringList tokens = name.split( ':' );
+    if( tokens.contains( "reload" ) )
+    {
+        reloadWikipedia();
+    }
+    else
+    {
+        removeAllData( name );
+        setData( name, QVariant());
+        update();
+    }
 
     return true;
 }
@@ -244,9 +251,11 @@ WikipediaEngine::wikiResult( KJob* job )
     }
 
     // Ok lets remove the top and bottom parts of the page
-    m_wiki = m_wiki.mid( m_wiki.indexOf( "<h1 class=\"firstHeading\">" ) );
+    m_wiki = m_wiki.mid( m_wiki.indexOf( "<!-- start content -->" ) );
     m_wiki = m_wiki.mid( 0, m_wiki.indexOf( "<div class=\"printfooter\">" ) );
-    // Adding back license information
+
+    // Adding back style and license information
+    m_wiki = "<div id=\"bodyContent\"" + m_wiki;
     m_wiki += copyright;
     m_wiki.append( "</div>" );
     m_wiki.remove( QRegExp("<h3 id=\"siteSub\">[^<]*</h3>") );
@@ -276,10 +285,6 @@ WikipediaEngine::wikiResult( KJob* job )
     m_wiki.remove( QRegExp( "<textarea[^>]*>" ) );
     m_wiki.remove( "</textarea>" );
 
-    //first we convert all the links with protocol to external, as they should all be External Links.
-    m_wiki.replace( QRegExp( "href= *\"http:" ), "href=\"externalurl:" );
-    m_wiki.replace( QRegExp( "href= *\"#" ), "href=\"" +m_wikiCurrentUrl + '#' );
-
     // Make sure that the relative links inside the wikipedia HTML is forcibly made into absolute links (yes, this is deep linking, but we're showing wikipedia data as wikipedia data, not stealing any credz here)
     m_wiki.replace( QRegExp( "href= *\"/" ), "href=\"" + wikiSiteUrl() );
     m_wiki.replace( QRegExp( "src= *\"/" ), "src=\"" + wikiSiteUrl() );
@@ -329,7 +334,11 @@ WikipediaEngine::wikiResult( KJob* job )
 inline QString
 WikipediaEngine::wikiLocale() const
 {
-    return !m_wikiLocale.isEmpty() ? m_wikiLocale : QString( "en" );
+    // if there is no language set (QLocale::C) then return english as default
+	if( m_wikiLang.language() == QLocale::C )
+        return "en";
+    else
+        return m_wikiLang.name().split( "_" )[0];
 }
 
 inline QString
@@ -339,6 +348,8 @@ WikipediaEngine::wikiArtistPostfix()
         return " (band)";
     else if( wikiLocale() == "de" )
         return " (Band)";
+    else if( wikiLocale() == "pl" )
+        return " (Kapela)";
     return QString();
 }
 
@@ -372,7 +383,7 @@ WikipediaEngine::reloadWikipedia()
     DEBUG_BLOCK
         
     debug() << "wiki url: " << m_wikiCurrentUrl;
-
+    removeSource( "wikipedia" );
     setData( "wikipedia", "message", i18n( "Fetching content.." ) );
     m_wikiJob = KIO::storedGet( m_wikiCurrentUrl, KIO::NoReload, KIO::HideProgressInfo );
     connect( m_wikiJob, SIGNAL( result( KJob* ) ), SLOT( wikiResult( KJob* ) ) );
