@@ -55,7 +55,7 @@ PopupDropperItemPrivate::PopupDropperItemPrivate( PopupDropperItem *parent )
     , customHoveredFillBrush( false )
     , separator( false )
     , file( QString() )
-    , blankElementRect( 0, 0, 30, 50 )
+    , svgElementRect( 0, 0, 50, 50 )
     , horizontalOffset( 30 )
     , textOffset( 30 )
     , hoverIndicatorShowStyle( PopupDropperItem::Never )
@@ -132,16 +132,19 @@ void PopupDropperItem::setAction( PopupDropperAction *action )
         }
         if( d->svgItem && pudaction->renderer() && pudaction->renderer()->isValid() )
             d->svgItem->setSharedRenderer( pudaction->renderer() );
-        if( d->svgItem && !pudaction->elementId().isEmpty() && d->svgItem->renderer() && d->svgItem->renderer()->elementExists( pudaction->elementId() ) )
-        {
+
+        if( d->svgItem && !pudaction->elementId().isEmpty() )
             d->svgItem->setElementId( pudaction->elementId() );
+
+        if( d->svgItem->renderer() && d->svgItem->renderer()->elementExists( pudaction->elementId() ) )
             d->svgItem->show();
-        }
         else if( d->svgItem )
             d->svgItem->hide();
+
         if( pudaction->isSeparator() )
             d->separator = true;
-        reposSvgItem();
+        
+        scaleAndReposSvgItem();
 
         d->hoverIndicatorRectItem = new QGraphicsRectItem( this );
             
@@ -166,6 +169,8 @@ void PopupDropperItem::setAction( PopupDropperAction *action )
             d->hoverIndicatorRectItem->hide();
 
         d->hoverIndicatorRectFillItem->hide();
+
+        reposHoverFillRects();
     }
 
     if( d->pd )
@@ -197,9 +202,7 @@ PopupDropperItem::Orientation PopupDropperItem::orientation() const
 void PopupDropperItem::setOrientation( const Orientation orientation )
 {
     d->orientation = orientation;
-    reposSvgItem();
-    reposTextItem();
-    reposHoverFillRects();
+    fullUpdate();
 }
 
 QString PopupDropperItem::text() const
@@ -375,127 +378,96 @@ QGraphicsSvgItem* PopupDropperItem::svgItem() const
     return d->svgItem;
 }
 
-void PopupDropperItem::reposSvgItem()
+void PopupDropperItem::scaleAndReposSvgItem()
 {
-    if( !d->svgItem )
+    if( !d->svgItem || !d->borderRectItem )
         return;
-    //qDebug() << "\n\nPUDItem boundingRect().width() = " << boundingRect().width();
-    //qDebug() << "svgItem boundingRect width = " << d->svgItem->boundingRect().width();
+
+    //Need to scale if it is too tall or wide
+    qreal maxheight = d->svgElementRect.height() - ( d->borderRectItem ? ( 2 * d->borderRectItem->pen().width() ) : 0 );
+    qreal maxwidth = d->svgElementRect.width() - ( d->borderRectItem ? ( 2 * d->borderRectItem->pen().width() ) : 0 );
+    qreal vertScaleValue = maxheight / d->svgItem->sceneBoundingRect().height();
+    qreal horizScaleValue = maxwidth / d->svgItem->sceneBoundingRect().width();
+    qreal scaleValue = vertScaleValue < horizScaleValue ? vertScaleValue : horizScaleValue;
+    
+    d->svgItem->scale( scaleValue, scaleValue );
+
+    qreal item_center = ( d->borderRectItem->sceneBoundingRect().height() / 2 ) + d->borderRectItem->pos().y();
+
     if( d->orientation == PopupDropperItem::Left )
     {
-        d->svgItem->setPos( d->horizontalOffset, 0 );
-        //qDebug() << "Left, svgItem pos = " << d->svgItem->pos().x();
+        d->svgItem->setPos( d->horizontalOffset, item_center - ( d->svgElementRect.height() / 2 ) );
     }
     else
     {
-        //int rightside = d->borderRectItem ? d->borderRectItem->boundingRect().width() : boundingRect().width();
         int rightside;
         if( !d->pd || d->pd->viewSize().width() == 0 )
-            rightside = boundingRect().width();
+            rightside = sceneBoundingRect().width();
         else
             rightside = d->pd->viewSize().width();
-        //qDebug() << "right side = " << rightside;
-        d->svgItem->setPos( 
+            d->svgItem->setPos( 
                 rightside
-                - d->svgItem->boundingRect().width()
+                - d->svgItem->sceneBoundingRect().width()
                 - d->horizontalOffset
-                , 0 );
-        //qDebug() << "Right, svgItem pos = " << d->svgItem->pos().x();
+                , item_center - ( d->svgElementRect.height() / 2 ) );
     }
-    //qDebug() << "\n\n";
 }
 
 void PopupDropperItem::reposTextItem()
 {
-    int rightside; 
+    if( !d->textItem || !d->borderRectItem )
+        return;
+    
+    qreal item_vert_center = ( d->borderRectItem->sceneBoundingRect().height() / 2 ) + d->borderRectItem->pos().y();
+
+    int rightside;
     if( !d->pd || d->pd->viewSize().width() == 0 )
-        rightside = boundingRect().width();
+        rightside = sceneBoundingRect().width();
     else
         rightside = d->pd->viewSize().width();
-    if( d->textItem )
-    {
-        if( d->svgItem->elementId().isEmpty() || ( d->svgItem->renderer() && !d->svgItem->renderer()->elementExists( d->svgItem->elementId() ) ) )
-        {
-            int offsetPos = d->horizontalOffset + d->textOffset + d->blankElementRect.width();
-            d->textItem->setPos(
-                    ( d->orientation == PopupDropperItem::Left
-                        ? offsetPos
-                        : ( d->borderRectItem
-                              ? d->borderRectItem->boundingRect().width() - offsetPos - d->textItem->boundingRect().width()
-                              : rightside - offsetPos - d->textItem->boundingRect().width()
-                          )
-                    )
-                , ( d->blankElementRect.height() / 2 ) - ( d->textItem->boundingRect().height() / 2 ) ); 
-        }
-        else
-        {
-            int offsetPos = d->horizontalOffset + d->textOffset + d->svgItem->boundingRect().width();
-            d->textItem->setPos(
-                    ( d->orientation == PopupDropperItem::Left
-                        ? offsetPos
-                        : ( d->borderRectItem
-                            ? d->borderRectItem->boundingRect().width() - offsetPos - d->textItem->boundingRect().width()
-                            : rightside - offsetPos - d->textItem->boundingRect().width()
-                          )
-                    )
-                , ( d->svgItem->boundingRect().height() / 2 ) - ( d->textItem->boundingRect().height() / 2 ) );
-        }
-        d->textItem->setFont( d->font );
-    }
+    
+    int offsetPos = d->horizontalOffset + d->textOffset + d->svgElementRect.width();
+    d->textItem->setPos(
+            ( d->orientation == PopupDropperItem::Left
+                ? offsetPos
+                : d->borderRectItem->sceneBoundingRect().width() - offsetPos - d->textItem->sceneBoundingRect().width()
+            )
+        , item_vert_center - ( d->textItem->sceneBoundingRect().height() / 2 ) ); 
+    d->textItem->setFont( d->font );
 }
 
 void PopupDropperItem::reposHoverFillRects()
 {
-    if( !d->hoverIndicatorRectItem || !d->hoverIndicatorRectFillItem || !d->svgItem || !d->borderRectItem )
+    if( !d->hoverIndicatorRectItem || !d->hoverIndicatorRectFillItem || !d->textItem || !d->borderRectItem )
         return;
 
     //qDebug() << "\n\nPUDItem boundingRect().width() = " << boundingRect().width();
-    qreal startx, starty, endx, endy;
+    qreal startx, starty, endx, endy, item_center;
     int rightside;
         if( !d->pd || d->pd->viewSize().width() == 0 )
-            rightside = boundingRect().width();
+            rightside = sceneBoundingRect().width();
         else
             rightside = d->pd->viewSize().width();
     //int rightside = d->borderRectItem ? d->borderRectItem->boundingRect().width() : boundingRect().width();
     if( d->orientation == PopupDropperItem::Left )
-        startx = ( d->textItem
-                     && ( d->svgItem->elementId().isEmpty()
-                            || ( d->svgItem->renderer()
-                                && !d->svgItem->renderer()->elementExists( d->svgItem->elementId() )
-                               )
-                        )
-                 ? ( d->textItem->pos().x() / 2 )
-                 : ( d->svgItem->pos().x() / 2 )
-                 )
-                 - ( d->hoverIndicatorRectWidth / 2 )
-                 + d->hoverIndicatorRectItem->pen().width();
+        startx = d->horizontalOffset 
+                 - d->hoverIndicatorRectWidth
+                 - ( 2  * d->hoverIndicatorRectItem->pen().width() );
     else
     {
         //qDebug() << "right side = " << rightside;
-        startx = ( d->textItem
-                     && ( d->svgItem->elementId().isEmpty()
-                            || ( d->svgItem->renderer()
-                                && !d->svgItem->renderer()->elementExists( d->svgItem->elementId() )
-                               )
-                         )
-                 ? ( rightside - ( ( rightside - d->textItem->pos().x() - d->textItem->boundingRect().width() ) / 2 ) )
-                 : ( rightside - ( ( rightside - d->svgItem->pos().x() - d->svgItem->boundingRect().width() ) / 2 ) )
-                 )
-                 - ( d->hoverIndicatorRectWidth / 2 )
-                 + d->hoverIndicatorRectItem->pen().width();
+        startx = rightside - d->horizontalOffset
+                 + d->hoverIndicatorRectWidth
+                 - ( 2 * d->hoverIndicatorRectItem->pen().width() );
     }
 
-    starty = ( d->svgItem->elementId().isEmpty() || ( d->svgItem->renderer() && !d->svgItem->renderer()->elementExists( d->svgItem->elementId() ) )
-                ? ( d->borderRectItem->pos().y() + ( 2 * d->borderRectItem->pen().width() ) - ( d->hoverIndicatorRectItem->pen().width() ) )
-                : ( d->svgItem->pos().y() + ( 2 * d->borderRectItem->pen().width() ) ) );
+    item_center = ( d->borderRectItem->sceneBoundingRect().height() / 2 ) + d->borderRectItem->pos().y();
+
+    starty = item_center - ( d->svgElementRect.height() / 2 );
 
     endx = d->hoverIndicatorRectWidth - ( 2 * d->hoverIndicatorRectItem->pen().width() );
 
-    endy = ( d->svgItem->elementId().isEmpty() || ( d->svgItem->renderer() && !d->svgItem->renderer()->elementExists( d->svgItem->elementId() ) )
-                ? ( d->borderRectItem->boundingRect().height() - ( 4 * d->borderRectItem->pen().width() ) - ( 3 * d->hoverIndicatorRectItem->pen().width() ) )
-                : ( d->svgItem->boundingRect().height() - ( 4 * d->borderRectItem->pen().width() ) ) );
-
-    //qDebug() << "startx, endx = " << startx << ", " << endx;
+    endy = d->svgElementRect.height();
 
     QRectF indicatorRect( startx, starty, endx, endy );
     d->hoverIndicatorRectItem->setRect( indicatorRect );
@@ -519,7 +491,14 @@ QSvgRenderer* PopupDropperItem::sharedRenderer() const
 void PopupDropperItem::setSharedRenderer( QSvgRenderer *renderer )
 {
     if( renderer && d->svgItem )
+    {
         d->svgItem->setSharedRenderer( renderer );
+        if( !d->svgItem->elementId().isEmpty() && d->svgItem->renderer()->elementExists( d->svgItem->elementId() ) )
+        {
+            d->svgItem->show();
+            fullUpdate();
+        }
+    }
 }
 
 QString PopupDropperItem::elementId() const
@@ -542,14 +521,14 @@ void PopupDropperItem::setElementId( const QString &id )
     }
 }
 
-QRect PopupDropperItem::blankElementRect() const
+QRect PopupDropperItem::svgElementRect() const
 {
-    return d->blankElementRect;
+    return d->svgElementRect;
 }
 
-void PopupDropperItem::setBlankElementRect( const QRect &rect )
+void PopupDropperItem::setSvgElementRect( const QRect &rect )
 {
-    d->blankElementRect = rect;
+    d->svgElementRect = rect;
 }
 
 int PopupDropperItem::horizontalOffset() const
@@ -584,24 +563,23 @@ void PopupDropperItem::setHoverMsecs( const int msecs )
 
 void PopupDropperItem::hoverEntered()
 {
-    //qDebug() << "Starting hover timer for " << static_cast<QObject*>(this);
     if( d->hoverIndicatorRectItem && d->hoverIndicatorRectFillItem && d->hoverIndicatorShowStyle != PopupDropperItem::Never )
     {
         d->hoverIndicatorRectFillItem->show();
     }
     d->hoverTimer.stop();
     d->hoverTimer.setDirection( QTimeLine::Forward );
-    d->hoverTimer.start();
     d->hoveredOver = true;
+    d->hoverTimer.start();
 }
 
 void PopupDropperItem::hoverLeft()
 {
-    //qDebug() << "Stopping hover timer for " << static_cast<QObject*>(this);
     d->hoverTimer.stop();
     d->hoverTimer.setDirection( QTimeLine::Backward );
-    d->hoverTimer.start();
     d->hoveredOver = false;
+    if( d->hoverTimer.currentFrame() != 0 )
+        d->hoverTimer.start();
 }
 
 int PopupDropperItem::borderWidth() const
@@ -669,11 +647,10 @@ void PopupDropperItem::dropped( QDropEvent *event ) //virtual SLOT
 
 void PopupDropperItem::hoverFinished() //SLOT
 {
-    //qDebug() << "PopupDropperItem timeout";
     if( d->separator )
         return;
 
-    //qDebug() << d->hoverTimer.direction();;
+    //qDebug() << "direction = forwards ? " << ( d->hoverTimer.direction() == QTimeLine::Forward ? "yes" : "no" );
 
     if( d->action && d->hoverTimer.direction() == QTimeLine::Forward )
         d->action->activate( QAction::Hover );
@@ -682,9 +659,9 @@ void PopupDropperItem::hoverFinished() //SLOT
         d->textItem->setDefaultTextColor( d->hoveredTextColor );
     else
         d->textItem->setDefaultTextColor( d->baseTextColor );
-   
+
     //Something is messed up in QTimeLine...I get hoverFinished immediately after doing a hoverLeft, but only sometimes...hence the check
-    //to see if the timeline isn't running 
+    //to see if the timeline isn't running
     if( d->hoverIndicatorRectFillItem && d->hoverTimer.state() == QTimeLine::NotRunning && d->hoverTimer.direction() == QTimeLine::Backward )
     {
         d->hoverIndicatorRectFillItem->hide();
@@ -700,6 +677,7 @@ void PopupDropperItem::hoverFrameChanged( int frame ) //SLOT
 {
     if( d->separator )
         return;
+    //qDebug() << "hoverFrameChanged for " << static_cast<QObject*>(this) << ", frame = " << frame;
     int range = d->hoverTimer.endFrame() - d->hoverTimer.startFrame();
     qreal multiplier = ( 1.0 * frame ) / range;
     int r = (int)( ( d->hoveredTextColor.red() - d->baseTextColor.red() ) * multiplier ) + d->baseTextColor.red();
@@ -743,12 +721,23 @@ void PopupDropperItem::hoverFrameChanged( int frame ) //SLOT
         d->pd->updateAllOverlays();
 }
 
+void PopupDropperItem::fullUpdate()
+{
+    scaleAndReposSvgItem();
+    reposTextItem();
+    reposHoverFillRects();
+    if( d->pd )
+        d->pd->updateAllOverlays();
+}
+
 QRectF PopupDropperItem::boundingRect() const
 {
     if( d->borderRectItem )
         return d->borderRectItem->boundingRect();
+    else if( d->pd && d->pd->viewSize().width() != 0 )
+        return QRectF( 0, 0, d->pd->viewSize().width(), d->svgElementRect.height() );
     else
-        return QRectF( 0, 0, d->blankElementRect.width(), d->blankElementRect.height() );
+        return QRectF( 0, 0, d->svgElementRect.width(), d->svgElementRect.height() );
 }
 
 void PopupDropperItem::paint( QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget )
