@@ -16,8 +16,9 @@
 
 #include "Debug.h"
 
-#include "plasma/applet.h"
-#include "plasma/corona.h"
+#include <plasma/applet.h>
+#include <plasma/corona.h>
+
 #include <kicon.h>
 
 #define ENTRY_HEIGHT  32
@@ -33,6 +34,7 @@ AmarokToolBoxMenu::AmarokToolBoxMenu( QGraphicsItem *parent, bool runningApplets
     , m_containment( 0 )
     , m_removeApplets( false )
     , m_menuSize( 4 )
+    , m_installScriptedApplet( 0 )
     , m_showing( 0 )
     , m_delay( 250 )
 {
@@ -50,7 +52,7 @@ AmarokToolBoxMenu::AmarokToolBoxMenu( QGraphicsItem *parent, bool runningApplets
 
         allApplets.insert( info.name(), info.pluginName() );
         if( !runningAppletsOnly )
-            appletsToShow << info.name();        
+            appletsToShow << info.name();      
     }
     
     if( runningAppletsOnly )
@@ -92,7 +94,7 @@ AmarokToolBoxMenu::init( QMap< QString, QString > allApplets, QStringList applet
     
     m_hideIcon = new ToolBoxIcon( this );
 
-    QAction *hideMenu = new QAction( "", this );
+    QAction *hideMenu = new QAction( this );
     hideMenu->setIcon( KIcon( "window-close" ) );
     hideMenu->setEnabled( true );
     hideMenu->setVisible( true );
@@ -114,6 +116,20 @@ AmarokToolBoxMenu::init( QMap< QString, QString > allApplets, QStringList applet
     m_downArrow = new ToolBoxIcon( this );
     createArrow( m_upArrow, "up" );
     createArrow( m_downArrow, "down" );
+    
+    m_installScriptedApplet = new ToolBoxIcon( this );
+    m_installScriptedApplet->setDrawBackground( true );
+    m_installScriptedApplet->setOrientation( Qt::Horizontal );
+    m_installScriptedApplet->setText( i18n( "Install Applets" ) );
+    const QSizeF size( ENTRY_WIDTH - 60, ENTRY_HEIGHT - 9 );
+    m_installScriptedApplet->setMinimumSize( size );
+    m_installScriptedApplet->setMaximumSize( size );
+    m_installScriptedApplet->resize( size );
+    m_installScriptedApplet->setZValue( zValue() + 1 );
+    m_installScriptedApplet->hide();
+    
+    connect( m_installScriptedApplet, SIGNAL( clicked() ), this, SIGNAL( installApplets() ) );
+    
 }
 
 void
@@ -135,7 +151,6 @@ AmarokToolBoxMenu::setContainment( Containment *newContainment )
             disconnect( containment, SIGNAL( appletRemoved( Plasma::Applet * ) ),
                  this, SLOT( appletRemoved( Plasma::Applet * ) ) );
         }
-        
         m_containment = newContainment;
         initRunningApplets();
         populateMenu();
@@ -296,6 +311,10 @@ AmarokToolBoxMenu::show( bool refreshApplets )
     m_hideIcon->setPos( 5, boundingRect().height() - ( ENTRY_HEIGHT + ENTRY_MARGIN ) * m_menuSize - OFFSET_Y + ENTRY_MARGIN * 2 );
     m_hideIcon->show();
     setZValue( zValue() + 10000 );
+    
+//    m_installScriptedApplet->setPos( 30, boundingRect().height() - ( ENTRY_HEIGHT + ENTRY_MARGIN ) * m_menuSize - OFFSET_Y + ENTRY_MARGIN * 2 );
+//    m_installScriptedApplet->show();
+    
     for( int i = m_currentMenu.count() - 1; i >= 0; i-- )
     {
         ToolBoxIcon *entry = m_currentMenu[m_currentMenu.count() - i - 1];
@@ -355,22 +374,12 @@ AmarokToolBoxMenu::setupMenuEntry( ToolBoxIcon *entry, const QString &appletName
 void
 AmarokToolBoxMenu::addApplet( const QString &pluginName )
 {
-    if( !pluginName.isEmpty() )
+    DEBUG_BLOCK
+    if( !pluginName.isEmpty() && containment() )
     {
-        bool appletFound = false;
-        //First we check if the applet is already running and in that case we just change
-        //to the containment where the applet is otherwise we add the applet to the current containment.
-        foreach( Plasma::Containment *containment, m_runningApplets.keys() )
-        {
-            if( m_runningApplets[containment].contains( pluginName ) )
-            {
-                emit changeContainment( containment );
-                appletFound = true;
-                break;
-            }
-        }
-        if( !appletFound && containment() )
-            containment()->addApplet( pluginName );
+        // we can't add directly b/c we may want to say at what location, so we just report
+        //containment()->addApplet( pluginName );
+        emit addAppletToContainment( pluginName );
     }
 }
 
