@@ -1,5 +1,6 @@
 /*******************************************************************************
 * copyright              : (C) 2008 William Viana Soares <vianasw@gmail.com>   *
+* copyright		 : (C) 2009 Leo Franchi		 <lfranchi@kde.org>    *
 *                                                                              *
 ********************************************************************************/
 
@@ -19,6 +20,10 @@
 #include "meta/MetaUtility.h"
 #include "playlist/PlaylistController.h"
 
+#include <plasma/widgets/iconwidget.h>
+
+#include <KIcon>
+
 #include <QFont>
 #include <QFontMetricsF>
 
@@ -28,6 +33,26 @@ TrackWidget::TrackWidget( QGraphicsItem *parent )
     , m_track( 0 )
     , m_rating( new RatingWidget( this ) )
 {
+    m_scoreLabel = new QGraphicsSimpleTextItem( i18nc( "Score of a track", "Score:" ), this );
+    
+    m_scoreText = new QGraphicsSimpleTextItem( this );
+    m_scoreText->setCursor( Qt::ArrowCursor );
+    
+    QFont font;
+    font.setBold( false );
+    font.setPointSize( font.pointSize() - 2 );
+    font.setStyleStrategy( QFont::PreferAntialias );
+
+    m_scoreText->setFont( font );
+    m_scoreText->setBrush( Amarok::highlightColor().darker( 200 ) );
+    m_scoreText->show();
+
+    m_scoreLabel->setFont( font );
+    m_scoreLabel->setBrush( Amarok::highlightColor().darker( 150 ) );
+    m_scoreLabel->show();
+
+    setBrush( Amarok::highlightColor().darker( 200 ) );
+    
     setDrawBackground( true );
     m_rating->setSpacing( 2 );
 
@@ -62,11 +87,18 @@ TrackWidget::mouseReleaseEvent( QGraphicsSceneMouseEvent *event )
 void
 TrackWidget::paint( QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget )
 {
+    painter->setRenderHint( QPainter::Antialiasing );
+    int ratingXPos = contentsRect().width() - PADDING - m_rating->size().width();
+    m_rating->setPos( ratingXPos, contentsRect().height() / 2 - m_rating->size().height() / 2 - 2 );
     m_rating->setMinimumSize( contentsRect().width() / 5, contentsRect().height() - PADDING );
     m_rating->setMaximumSize( contentsRect().width() / 5, contentsRect().height() - PADDING );
 
-    m_rating->setPos( contentsRect().width() - PADDING - m_rating->size().width(),
-                      contentsRect().height() / 2 - m_rating->size().height() / 2 - 2 );
+    QFontMetrics fm( m_scoreText->font() );
+    m_scoreText->setPos( ratingXPos - m_scoreText->boundingRect().width() - PADDING,
+                         boundingRect().height() / 2 - fm.boundingRect( m_scoreText->text() ).height() / 2 );
+    m_scoreLabel->setPos( m_scoreText->pos().x() - m_scoreLabel->boundingRect().width() - PADDING,
+                          boundingRect().height() / 2 - fm.boundingRect( m_scoreLabel->text() ).height() / 2 );
+    
 
     ToolBoxIcon::paint( painter, option, widget );
 }
@@ -76,6 +108,7 @@ TrackWidget::setTrack( Meta::TrackPtr track )
 {
     m_track = track;
     m_rating->setRating( track->rating() );
+    m_scoreText->setText( QString("%1").arg( int( track->score() ) ) );
 }
 
 void
@@ -85,10 +118,13 @@ TrackWidget::show()
     if( m_track )
     {
         const QString playedLast = Amarok::verboseTimeSince( m_track->lastPlayed() );
-        const QString fullText( i18n( "%1 - %2 ( %3 )", m_track->artist()->prettyName(), m_track->prettyName(), playedLast ) );
+        const QString fullText( i18n( "%1 - %2 (%3)", m_track->artist()->prettyName(), m_track->prettyName(), playedLast ) );
         const QFontMetricsF fm( font() );
 
-        setText( fm.elidedText( fullText, Qt::ElideRight, contentsRect().width() - m_rating->size().width() - PADDING ) );
+        int rightMargin = m_scoreLabel->boundingRect().width() + m_scoreText->boundingRect().width() + \
+                            m_rating->size().width() + PADDING;
+
+        setText( fm.elidedText( fullText, Qt::ElideRight, contentsRect().width() - rightMargin ) );
     }
 
     ToolBoxIcon::show();
