@@ -27,6 +27,7 @@
 #include "amarokconfig.h"
 #include "CollectionManager.h"
 #include "covermanager/CoverFetcher.h"
+#include "covermanager/CoverFetchingActions.h"
 #include "Debug.h"
 #include "EditCapability.h"
 #include "FilenameLayoutDialog.h"
@@ -42,6 +43,7 @@
 #include <KGlobal>
 #include <KHTMLView>
 #include <KLineEdit>
+#include <KMenu>
 #include <KRun>
 
 #include <qdom.h>
@@ -49,19 +51,18 @@
 #include <QPair>
 #include <QTimer>
 
-#include "metadata/tfile_helper.h" //TagLib::File::isWritable
-
 TagDialog::TagDialog( const Meta::TrackList &tracks, QWidget *parent )
     : KDialog( parent )
     , m_currentCover()
     , m_tracks( tracks )
-    , m_currentTrack( tracks.first() )
     , m_trackIterator( m_tracks )
     , m_queryMaker( 0 )
     , ui( new Ui::TagDialogBase() )
 {
     DEBUG_BLOCK
 
+    setCurrentTrack( m_tracks.first() );
+
     ui->setupUi( this );
     init();
     startDataQuery();
@@ -71,13 +72,14 @@ TagDialog::TagDialog( Meta::TrackPtr track, QWidget *parent )
     : KDialog( parent )
     , m_currentCover()
     , m_tracks()
-    , m_currentTrack( track )
     , m_trackIterator( m_tracks )   //makes the compiler happy
     , m_queryMaker( 0 )
     , ui( new Ui::TagDialogBase() )
 {
     DEBUG_BLOCK
 
+    setCurrentTrack( m_tracks.first() );
+
     m_tracks.append( track );
     //we changed the list after creating the iterator, so create a new iterator
     m_trackIterator = QListIterator<Meta::TrackPtr >( m_tracks );
@@ -111,6 +113,9 @@ TagDialog::~TagDialog()
 
     Amarok::config( "TagDialog" ).writeEntry( "CurrentTab", ui->kTabWidget->currentIndex() );
 
+    if( m_currentTrack && m_currentTrack->album() )
+        unsubscribeFrom( m_currentTrack->album() );
+
     // NOTE: temporary crash prevention.  TreeView should _always_ be updated
     // if tags have changed.
 
@@ -128,6 +133,19 @@ TagDialog::setTab( int id )
     ui->kTabWidget->setCurrentIndex( id );
 }
 
+void
+TagDialog::setCurrentTrack( Meta::TrackPtr track )
+{
+    DEBUG_BLOCK
+    if( m_currentTrack && m_currentTrack->album() )
+        unsubscribeFrom( m_currentTrack->album() );
+
+    m_currentTrack = track;
+
+    if( m_currentTrack && m_currentTrack->album() )
+        subscribeTo( m_currentTrack->album() );
+}
+
 
 ////////////////////////////////////////////////////////////////////////////////
 // PRIVATE SLOTS
@@ -156,7 +174,7 @@ TagDialog::queryDone()
     m_trackIterator = QListIterator<Meta::TrackPtr >( m_tracks );
     if( m_tracks.size() )
     {
-        m_currentTrack = m_tracks.first();
+        setCurrentTrack( m_tracks.first() );
         init();
         QTimer::singleShot( 0, this, SLOT( show() ) );
     }
@@ -170,40 +188,56 @@ void
 TagDialog::resultReady( const QString &collectionId, const Meta::AlbumList &albums )
 {
     Q_UNUSED( collectionId )
+
     foreach( Meta::AlbumPtr album, albums )
     {
-        m_albums << album->name();
+        if( !album->name().isEmpty() )
+            m_albums << album->name();
     }
+
+    m_albums.sort();
 }
 
 void
 TagDialog::resultReady( const QString &collectionId, const Meta::ArtistList &artists )
 {
     Q_UNUSED( collectionId )
+
     foreach( Meta::ArtistPtr artist, artists )
     {
-        m_artists << artist->name();
+        if( !artist->name().isEmpty() )
+            m_artists << artist->name();
     }
+
+    m_artists.sort();
 }
 
 void
 TagDialog::resultReady( const QString &collectionId, const Meta::ComposerList &composers )
 {
     Q_UNUSED( collectionId )
+
     foreach( Meta::ComposerPtr composer, composers )
     {
-        m_composers << composer->name();
+        if( !composer->name().isEmpty() )
+            m_composers << composer->name();
     }
+
+    m_composers.sort();
 }
 
 void
 TagDialog::resultReady( const QString &collectionId, const Meta::GenreList &genres )
 {
     Q_UNUSED( collectionId )
+
     foreach( Meta::GenrePtr genre, genres )
     {
-        m_genres << genre->name();
+        if( !genre->name().isEmpty() )  // Where the heck do the empty genres come from?
+            m_genres << genre->name();
     }
+
+    m_genres.sort();
 }
 
 void
@@ -213,10 +247,17 @@ TagDialog::dataQueryDone()
 
     m_dataQueryMaker->deleteLater();
     m_dataQueryMaker = 0;
+
+    // basically we want to ignore the fact that the fields are being
+    // edited because we do it not the user, so it results in empty
+    // tags being saved to files---data loss is BAD!
+    QMap< QString, bool > m_fieldEditedSave = m_fieldEdited;
+
     //we simply clear the completion data of all comboboxes
     //then load the current track again. that's more work than necessary
     //but the performance impact should be negligible
-
+    // we do this because if we insert items and the contents of the textbox
+    // are not in the list, it clears the textbox. which is bad --lfranchi 2.22.09
     QString saveText( ui->kComboBox_artist->lineEdit()->text() );
     ui->kComboBox_artist->clear();
     ui->kComboBox_artist->insertItems( 0, m_artists );
@@ -237,6 +278,8 @@ TagDialog::dataQueryDone()
     ui->kComboBox_genre->insertItems( 0, m_genres );
     ui->kComboBox_genre->lineEdit()->setText( saveText );
 
+   
+
     if( !m_queryMaker )  //track query complete or not necessary
     {
         if( m_perTrack )
@@ -248,6 +291,8 @@ TagDialog::dataQueryDone()
             readMultipleTracks();
         }
     }
+
+    m_fieldEdited = m_fieldEditedSave;
 }
 
 void
@@ -288,7 +333,8 @@ TagDialog::previousTrack()
     {
         if ( m_currentTrack == m_trackIterator.peekPrevious() )
             m_trackIterator.previous();
-        m_currentTrack = m_trackIterator.previous();
+
+        setCurrentTrack( m_trackIterator.previous() );
     }
     loadTags( m_currentTrack );
     enableItems();
@@ -305,7 +351,8 @@ TagDialog::nextTrack()
     {
         if ( m_currentTrack == m_trackIterator.peekNext() )
             m_trackIterator.next();
-        m_currentTrack = m_trackIterator.next();
+
+        setCurrentTrack( m_trackIterator.next() );
     }
     loadTags( m_currentTrack );
     enableItems();
@@ -345,63 +392,64 @@ TagDialog::enableItems()
     ui->pushButton_next->setEnabled( m_perTrack && m_trackIterator.hasNext() );
 }
 
-inline void 
+inline void
 TagDialog::composerModified()
 {
     m_fieldEdited[ "composer" ] = true;
     checkModified();
 }
 
-inline void 
+inline void
 TagDialog::artistModified()
 {
     m_fieldEdited[ "artist" ] = true;
     checkModified();
 }
 
-inline void 
+inline void
 TagDialog::albumModified()
 {
     m_fieldEdited[ "album" ] = true;
     checkModified();
 }
 
-inline void 
+inline void
 TagDialog::genreModified()
 {
     m_fieldEdited[ "genre" ] = true;
     checkModified();
 }
 
-inline void 
+inline void
 TagDialog::ratingModified()
 {
     m_fieldEdited[ "rating" ] = true;
     checkModified();
 }
 
-inline void 
+inline void
 TagDialog::yearModified()
 {
     m_fieldEdited[ "year" ] = true;
     checkModified();
 }
 
-inline void 
+inline void
 TagDialog::scoreModified()
 {
     m_fieldEdited[ "score" ] = true;
     checkModified();
 }
 
-inline void 
+inline void
 TagDialog::commentModified()
 {
+    DEBUG_BLOCK
     m_fieldEdited[ "comment" ] = true;
     checkModified();
 }
 
-inline void 
+inline void
 TagDialog::discNumberModified()
 {
     m_fieldEdited[ "discNumber" ] = true;
@@ -421,11 +469,12 @@ TagDialog::loadCover()
     if( !m_currentTrack->album() )
         return;
 
-    ui->pixmap_cover->setPixmap( m_currentTrack->album()->image( AmarokConfig::coverPreviewSize() ) );
+    const int s = 100; // Image preview size
+
+    ui->pixmap_cover->setPixmap( m_currentTrack->album()->image(s) );
     QString artist = m_currentTrack->artist() ? m_currentTrack->artist()->name() : QString();
     ui->pixmap_cover->setInformation( artist, m_currentTrack->album()->name() );
 
-    const int s = AmarokConfig::coverPreviewSize();
     ui->pixmap_cover->setMinimumSize( s, s );
     ui->pixmap_cover->setMaximumSize( s, s );
 }
@@ -441,7 +490,7 @@ TagDialog::guessFromFilename() //SLOT
     dialog->setButtons( KDialog::Ok | KDialog::Cancel );
     FilenameLayoutDialog *widget = new FilenameLayoutDialog( dialog );
     dialog->setMainWidget( widget );
-    
+
     const int dcode = dialog->exec();
     QString schemeFromDialog; //note to self: see where to put it from an old revision
     debug() << "FilenameLayoutDialog finished.";
@@ -458,8 +507,8 @@ TagDialog::guessFromFilename() //SLOT
                             //So there's a bunch of rotting unused code but I suggest leaving it like this for now so we can rollback to the old dialog if I don't manage to fix the
                             //new one before 2.0.
     schemes += schemeFromDialog;
-    
-    
+
+
     if( schemeFromDialog.isEmpty() )
     {
         //FIXME: remove this before release
@@ -575,14 +624,6 @@ void TagDialog::init()
     // set an icon for the open-in-konqui button
     ui->pushButton_open->setIcon( KIcon( "folder-amarok" ) );
 
-    //Update cover
-    //FIXME: Port 2.0
-//     connect( CollectionDB::instance(), SIGNAL( coverFetched( const QString&, const QString& ) ),
-//             this, SLOT( loadCover( const QString&, const QString& ) ) );
-//     connect( CollectionDB::instance(), SIGNAL( coverChanged( const QString&, const QString& ) ),
-//              this, SLOT( loadCover( const QString&, const QString& ) ) );
-
-
     connect( ui->pushButton_guessTags, SIGNAL(clicked()), SLOT( guessFromFilename() ) );
 
     if( m_tracks.count() > 1 )
@@ -603,7 +644,7 @@ void TagDialog::init()
         loadLabels( m_currentTrack );
         readTags();
     }
-    
+
     // Connects for modification check
     // only set to overwrite-on-save if the text has changed
     connect( ui->kLineEdit_title,     SIGNAL( textChanged( const QString& ) ),     SLOT(checkModified()) );
@@ -630,6 +671,9 @@ void TagDialog::init()
     connect( ui->pushButton_previous, SIGNAL( clicked() ), SLOT( previousTrack() ) );
     connect( ui->pushButton_next,     SIGNAL( clicked() ), SLOT( nextTrack() ) );
     connect( ui->checkBox_perTrack,   SIGNAL( clicked() ), SLOT( perTrack() ) );
+
+    ui->pixmap_cover->setContextMenuPolicy( Qt::CustomContextMenu );
+    connect( ui->pixmap_cover, SIGNAL( customContextMenuRequested(const QPoint &) ), SLOT( showCoverMenu(const QPoint &) ) );
 }
 
 void
@@ -666,6 +710,48 @@ inline const QString TagDialog::unknownSafe( QString s )
            : s;
 }
 
+void
+TagDialog::metadataChanged( Meta::AlbumPtr album )
+{
+    DEBUG_BLOCK
+
+    if( !m_currentTrack || !m_currentTrack->album() )
+        return;
+
+    debug() << "setting album cover?";
+
+    if( album == m_currentTrack->album() )
+    {
+        debug() << "yep";
+        loadCover();
+    }
+}
+
+void
+TagDialog::showCoverMenu( const QPoint &pos )
+{
+    Meta::AlbumPtr album = m_currentTrack->album();
+    if( !album )
+        return; // TODO: warning or something?
+
+    QAction *displayCoverAction = new DisplayCoverAction( this, album );
+    QAction *unsetCoverAction   = new UnsetCoverAction( this, album );
+
+    if( !album->hasImage() )
+    {
+        displayCoverAction->setEnabled( false );
+        unsetCoverAction->setEnabled( false );
+    }
+
+    KMenu *menu = new KMenu( this );
+    menu->addAction( displayCoverAction );
+    menu->addAction( new FetchCoverAction( this, album ) );
+    menu->addAction( new SetCustomCoverAction( this, album ) );
+    menu->addAction( unsetCoverAction );
+
+    menu->exec( ui->pixmap_cover->mapToGlobal(pos) );
+}
+
 const QStringList TagDialog::statisticsData()
 {
     AMAROK_NOTIMPLEMENTED
@@ -749,16 +835,16 @@ void TagDialog::readTags()
     const int len = ui->trackArtistAlbumLabel->width();
     QString curTrackAlbName;
     QString curArtistName;
-    
+
     QString curTrackName = fnt.elidedText( Qt::escape( m_currentTrack->name() ), Qt::ElideRight, len );
     QString curTrackPretName = fnt.elidedText( Qt::escape( m_currentTrack->prettyName() ), Qt::ElideRight, len );
-    
+
     if( m_currentTrack->album() )
         curTrackAlbName = fnt.elidedText( Qt::escape( m_currentTrack->album()->name() ), Qt::ElideRight, len );
     if( m_currentTrack->artist() )
         curArtistName = fnt.elidedText( Qt::escape( m_currentTrack->artist()->name() ), Qt::ElideRight, len );
-    
-    
+
+
     if( m_currentTrack->album() && m_currentTrack->album()->name().isEmpty() )
     {
         if( !m_currentTrack->name().isEmpty() )
@@ -769,11 +855,11 @@ void TagDialog::readTags()
                 niceTitle = QString( "<b>%1</b>").arg( curTrackName );
         }
         else
-                niceTitle = curTrackPretName;
-    } else if( m_currentTrack->album() )
-    {
+            niceTitle = curTrackPretName;
+    }
+    else if( m_currentTrack->album() )
         niceTitle = i18n( "<b>%1</b> by <b>%2</b> on <b>%3</b>" , curTrackName, curArtistName, curTrackAlbName );
-    } else if( m_currentTrack->artist() )
+    else if( m_currentTrack->artist() )
         niceTitle = i18n( "<b>%1</b> by <b>%2</b>" , curTrackName, curArtistName );
     else
         niceTitle = i18n( "<b>%1</b>" , curTrackName );
@@ -806,8 +892,8 @@ void TagDialog::readTags()
 
     summaryText = "<table width=100%><tr><td width=50%><table>";
     summaryText += body2cols.arg( i18n("Length:"), unknownSafe( Meta::secToPrettyTime( m_currentTrack->length() ) ) );
-    summaryText += body2cols.arg( i18n("Bitrate:"), unknownSafe( Meta::prettyBitrate( m_currentTrack->bitrate() ) ) );
-    summaryText += body2cols.arg( i18n("Samplerate:"), unknownSafe( QString::number( m_currentTrack->sampleRate() ) ) );
+    summaryText += body2cols.arg( i18n("Bit rate:"), unknownSafe( Meta::prettyBitrate( m_currentTrack->bitrate() ) ) );
+    summaryText += body2cols.arg( i18n("Sample rate:"), unknownSafe( QString::number( m_currentTrack->sampleRate() ) ) );
     summaryText += body2cols.arg( i18n("Size:"), unknownSafe( Meta::prettyFilesize( m_currentTrack->filesize() ) ) );
     summaryText += body2cols.arg( i18n("Format:"), unknownSafe( m_currentTrack->type() ) );
 
@@ -845,16 +931,7 @@ void TagDialog::readTags()
     //lyrics
     ui->kTextEdit_lyrics->setHtml( m_lyrics );
 
-    if( m_currentTrack->album() )
-    {
-        ui->pixmap_cover->setPixmap( m_currentTrack->album()->image( AmarokConfig::coverPreviewSize() ) );
-        if( m_currentTrack->artist() )
-            ui->pixmap_cover->setInformation( m_currentTrack->artist()->name(), m_currentTrack->album()->name() );
-    }
-    const int s = AmarokConfig::coverPreviewSize();
-    ui->pixmap_cover->setMinimumSize( s, s );
-    ui->pixmap_cover->setMaximumSize( s, s );
-
+    loadCover();
 
     // enable only for editable files
 #define enableOrDisable( X ) \
@@ -876,6 +953,9 @@ void TagDialog::readTags()
     ui->kTextEdit_comment->setEnabled( editable );
     ui->kTextEdit_selectedLabels->setEnabled( editable );
     m_labelCloud->view()->setEnabled( editable );
+    ui->ratingWidget->setEnabled( editable );
+    ui->qSpinBox_score->setEnabled( editable );
+    ui->pushButton_guessTags->setEnabled( editable );
 
     if( local )
         ui->pushButton_guessTags->show();
@@ -1012,18 +1092,21 @@ TagDialog::readMultipleTracks()
     {
         cur_item = ui->kComboBox_artist->currentIndex();
         m_currentData.insert( Meta::Field::ARTIST, first.value( Meta::Field::ARTIST ) );
+        ui->kComboBox_artist->completionObject()->insertItems( m_artists );
         selectOrInsertText( first.value( Meta::Field::ARTIST ).toString(), ui->kComboBox_artist );
     }
     if( album )
     {
         cur_item = ui->kComboBox_album->currentIndex();
         m_currentData.insert( Meta::Field::ALBUM, first.value( Meta::Field::ALBUM ) );
+        ui->kComboBox_album->completionObject()->insertItems( m_albums );
         selectOrInsertText( first.value( Meta::Field::ALBUM ).toString(), ui->kComboBox_album );
     }
     if( genre )
     {
         cur_item = ui->kComboBox_genre->currentIndex();
         m_currentData.insert( Meta::Field::GENRE, first.value( Meta::Field::GENRE ) );
+        ui->kComboBox_genre->completionObject()->insertItems( m_genres );
         selectOrInsertText( first.value( Meta::Field::GENRE ).toString(), ui->kComboBox_genre );
     }
     if( comment )
@@ -1035,6 +1118,7 @@ TagDialog::readMultipleTracks()
     {
         cur_item = ui->kComboBox_composer->currentIndex();
         m_currentData.insert( Meta::Field::COMPOSER, first.value( Meta::Field::COMPOSER ) );
+        ui->kComboBox_composer->completionObject()->insertItems( m_composers );
         selectOrInsertText( first.value( Meta::Field::COMPOSER ).toString(), ui->kComboBox_composer );
     }
     if( year )
@@ -1059,7 +1143,8 @@ TagDialog::readMultipleTracks()
     }
 
     m_trackIterator.toFront();
-    m_currentTrack = m_tracks.first();
+
+    setCurrentTrack( m_tracks.first() );
 
     ui->trackArtistAlbumLabel2->setText( i18np( "Editing 1 file", "Editing %1 files", songCount ) );
 
@@ -1173,15 +1258,27 @@ TagDialog::storeTags( const Meta::TrackPtr &track )
         debug() << "TagDialog::TAGSCHANGED";
         QVariantMap map( m_currentData );
 
-        map.insert( Meta::Field::TITLE, ui->kLineEdit_title->text() );
-        map.insert( Meta::Field::COMPOSER, ui->kComboBox_composer->currentText() );
-        map.insert( Meta::Field::ARTIST, ui->kComboBox_artist->currentText() );
-        map.insert( Meta::Field::ALBUM, ui->kComboBox_album->currentText() );
-        map.insert( Meta::Field::COMMENT, ui->kTextEdit_comment->toPlainText() );
-        map.insert( Meta::Field::GENRE, ui->kComboBox_genre->currentText() );
-        map.insert( Meta::Field::TRACKNUMBER, ui->qSpinBox_track->value() );
-        map.insert( Meta::Field::YEAR, ui->qSpinBox_year->value() );
-        map.insert( Meta::Field::DISCNUMBER, ui->qSpinBox_discNumber->value() );
+        //do not nedlessly update everything, as theat wrecks havoc with grouping in the playlist....
+
+        if ( ui->kLineEdit_title->text() != track->name() )
+            map.insert( Meta::Field::TITLE, ui->kLineEdit_title->text() );
+        if ( !track->composer() || ui->kComboBox_composer->currentText() != track->composer()->name() )
+            map.insert( Meta::Field::COMPOSER, ui->kComboBox_composer->currentText() );
+        if ( !track->artist() || ui->kComboBox_artist->currentText() != track->artist()->name() )
+            map.insert( Meta::Field::ARTIST, ui->kComboBox_artist->currentText() );
+        if ( !track->album() || ui->kComboBox_album->currentText() != track->album()->name() )
+            map.insert( Meta::Field::ALBUM, ui->kComboBox_album->currentText() );
+        if ( ui->kTextEdit_comment->toPlainText() != track->comment() )
+            map.insert( Meta::Field::COMMENT, ui->kTextEdit_comment->toPlainText() );
+        if ( !track->genre() || ui->kComboBox_genre->currentText() != track->genre()->name() )
+            map.insert( Meta::Field::GENRE, ui->kComboBox_genre->currentText() );
+        if ( ui->qSpinBox_track->value() != track->trackNumber() )
+            map.insert( Meta::Field::TRACKNUMBER, ui->qSpinBox_track->value() );
+        if ( !track->year() || QString::number( ui->qSpinBox_year->value() ) != track->year()->name() )
+            map.insert( Meta::Field::YEAR, ui->qSpinBox_year->value() );
+        if ( ui->qSpinBox_discNumber->value() != track->discNumber() )
+            map.insert( Meta::Field::DISCNUMBER, ui->qSpinBox_discNumber->value() );
+
         storedTags.remove( track );
         storedTags.insert( track, map );
     }
@@ -1358,30 +1455,45 @@ TagDialog::saveTags()
         if( !ec->isEditable() )
         {
             debug() << "Track not editable. Aborting loop.";
+            The::statusBar()->shortMessage( i18n( "Writing to file failed. Please check permissions and available disc space." ) );
             continue;
         }
 
         QVariantMap data = storedTags[ track ];
-        ec->beginMetaDataUpdate();
-        if( data.contains( Meta::Field::TITLE ) )
-            ec->setTitle( data.value( Meta::Field::TITLE ).toString() );
-        if( data.contains( Meta::Field::COMMENT ) )
-            ec->setComment( data.value( Meta::Field::COMMENT ).toString() );
-        if( data.contains( Meta::Field::ARTIST ) )
-            ec->setArtist( data.value( Meta::Field::ARTIST ).toString() );
-        if( data.contains( Meta::Field::ALBUM ) )
-            ec->setAlbum( data.value( Meta::Field::ALBUM ).toString() );
-        if( data.contains( Meta::Field::GENRE ) )
-            ec->setGenre( data.value( Meta::Field::GENRE ).toString() );
-        if( data.contains( Meta::Field::COMPOSER ) )
-            ec->setComposer( data.value( Meta::Field::COMPOSER ).toString() );
-        if( data.contains( Meta::Field::YEAR ) )
-            ec->setYear( data.value( Meta::Field::YEAR ).toString() );
-        if( data.contains( Meta::Field::TRACKNUMBER ) )
-            ec->setTrackNumber( data.value( Meta::Field::TRACKNUMBER ).toInt() );
-        if( data.contains( Meta::Field::DISCNUMBER ) )
-            ec->setDiscNumber( data.value( Meta::Field::DISCNUMBER ).toInt() );
-        ec->endMetaDataUpdate();
+
+        //there is really no need to write to the file if only info stored in the db has changed
+
+        //the if from hell
+        if ( data.contains( Meta::Field::TITLE ) || data.contains( Meta::Field::COMMENT ) ||
+             data.contains( Meta::Field::ARTIST ) || data.contains( Meta::Field::ALBUM ) ||
+             data.contains( Meta::Field::GENRE ) || data.contains( Meta::Field::COMPOSER ) ||
+             data.contains( Meta::Field::YEAR ) || data.contains( Meta::Field::TRACKNUMBER ) ||
+             data.contains( Meta::Field::TRACKNUMBER ) || data.contains( Meta::Field::DISCNUMBER ) )
+        {
+
+            debug() << "File info changed....";
+
+            ec->beginMetaDataUpdate();
+            if( data.contains( Meta::Field::TITLE ) )
+                ec->setTitle( data.value( Meta::Field::TITLE ).toString() );
+            if( data.contains( Meta::Field::COMMENT ) )
+                ec->setComment( data.value( Meta::Field::COMMENT ).toString() );
+            if( data.contains( Meta::Field::ARTIST ) )
+                ec->setArtist( data.value( Meta::Field::ARTIST ).toString() );
+            if( data.contains( Meta::Field::ALBUM ) )
+                ec->setAlbum( data.value( Meta::Field::ALBUM ).toString() );
+            if( data.contains( Meta::Field::GENRE ) )
+                ec->setGenre( data.value( Meta::Field::GENRE ).toString() );
+            if( data.contains( Meta::Field::COMPOSER ) )
+                ec->setComposer( data.value( Meta::Field::COMPOSER ).toString() );
+            if( data.contains( Meta::Field::YEAR ) )
+                ec->setYear( data.value( Meta::Field::YEAR ).toString() );
+            if( data.contains( Meta::Field::TRACKNUMBER ) )
+                ec->setTrackNumber( data.value( Meta::Field::TRACKNUMBER ).toInt() );
+            if( data.contains( Meta::Field::DISCNUMBER ) )
+                ec->setDiscNumber( data.value( Meta::Field::DISCNUMBER ).toInt() );
+            ec->endMetaDataUpdate();
+        }
     }
 
     // build a map, such that at least one track represents a unique collection
@@ -1397,7 +1509,7 @@ TagDialog::saveTags()
         }
 
         QString collId = track->collection()->collectionId();
-        
+
         if( !collectionsToUpdateMap.contains( collId ) )
             collectionsToUpdateMap.insert( collId, track );
     }
@@ -1416,6 +1528,7 @@ void
 TagDialog::applyToAllTracks()
 {
     DEBUG_BLOCK
+            debug() << m_fieldEdited;
 
     generateDeltaForLabelList( labelListFromText( ui->kTextEdit_selectedLabels->toPlainText() ) );
 
@@ -1432,55 +1545,55 @@ TagDialog::applyToAllTracks()
             data.insert( Meta::Field::ARTIST, ui->kComboBox_artist->currentText() );
             changed |= TagDialog::TAGSCHANGED;
         }
-        
+
         if( m_fieldEdited.contains( "album" ) && m_fieldEdited[ "album" ] )
         {
             data.insert( Meta::Field::ALBUM, ui->kComboBox_album->currentText() );
             changed |= TagDialog::TAGSCHANGED;
         }
-        
+
         if( m_fieldEdited.contains( "genre" ) && m_fieldEdited[ "genre" ] )
         {
             data.insert( Meta::Field::GENRE, ui->kComboBox_genre->currentText() );
             changed |= TagDialog::TAGSCHANGED;
         }
-        
+
         if( m_fieldEdited.contains( "comment" ) && m_fieldEdited[ "comment" ] )
         {
             data.insert( Meta::Field::COMMENT, ui->kTextEdit_comment->toPlainText() );
             changed |= TagDialog::TAGSCHANGED;
         }
-        
+
         if( m_fieldEdited.contains( "composer" ) && m_fieldEdited[ "composer" ] )
         {
             data.insert( Meta::Field::COMPOSER, ui->kComboBox_composer->currentText() );
             changed |= TagDialog::TAGSCHANGED;
         }
-        
+
         if( m_fieldEdited.contains( "year" ) && m_fieldEdited[ "year" ] )
         {
             data.insert( Meta::Field::YEAR, ui->qSpinBox_year->value() );
             changed |= TagDialog::TAGSCHANGED;
         }
-        
+
         if( m_fieldEdited.contains( "discNumber" ) && m_fieldEdited[ "discNumber" ] )
         {
             data.insert( Meta::Field::DISCNUMBER, ui->qSpinBox_discNumber->value() );
             changed |= TagDialog::TAGSCHANGED;
         }
-        
+
         if( m_fieldEdited.contains( "score" ) && m_fieldEdited[ "score" ] )
         {
             data.insert( Meta::Field::SCORE, ui->qSpinBox_score->value() );
             changed |= TagDialog::SCORECHANGED;
         }
-        
+
         if( m_fieldEdited.contains( "rating" ) && m_fieldEdited[ "rating" ] )
         {
             data.insert( Meta::Field::RATING, ui->ratingWidget->rating() );
             changed |= TagDialog::RATINGCHANGED;
         }
-        
+
         storeTags( track, changed, data );
 
         QStringList tmpLabels = labelsForTrack( track );
