@@ -23,6 +23,7 @@
 
 #include "Amarok.h"
 #include "Debug.h"
+#include "CaseConverter.h"
 
 #include <KConfig>
 #include <KConfigGroup>
@@ -30,8 +31,8 @@
 #include <KMacroExpanderBase>
 
 FileNameScheme::FileNameScheme( const QString &s )
-    : m_cod( s )                    
-    , m_titleField( -1 )               
+    : m_cod( s )
+    , m_titleField( -1 )
     , m_artistField( -1 )
     , m_albumField( -1 )
     , m_trackField( -1 )
@@ -304,7 +305,7 @@ void TagGuesser::guess( const QString &absFileName, FilenameLayoutDialog *dialog
                 composer = capitalizeWords( composer, caseOptions);
                 genre = capitalizeWords( genre, caseOptions);
             }
-            
+
             m_title = title;
             m_artist = artist;
             m_album = album;
@@ -341,43 +342,16 @@ QString TagGuesser::capitalizeWords( const QString &s, const int &caseOptions )
     else if( caseOptions == 3 )
     {
         debug() << "UPPER/LOWER CASE OPTIONS: 3 - First letter of every word uppercase";
-        QString result = s;
-        result[ 0 ] = result[ 0 ].toUpper();
-
-        const QRegExp wordRegExp( "\\s\\w" );
-        int i = result.indexOf( wordRegExp );
-        while ( i > -1 ) {
-            result[ i + 1 ] = result[ i + 1 ].toUpper();
-            i = result.indexOf( wordRegExp, ++i );
-        }
-        return result;
+        return Amarok::CaseConverter::toCapitalizedCase( s );
     }
     else if( caseOptions == 4 )
     {
         debug() << "UPPER/LOWER CASE OPTIONS: 4 - Title case.";
-        QString result = s;
-        result[ 0 ] = result[ 0 ].toUpper();
-
-        const QRegExp wordRegExp( "\\s\\w" );
-        int i = result.indexOf( wordRegExp );
-        const QRegExp littleWordRegExp( "(a|an|as|at|by|for|if|and|of|or|to|the|in)" ); //a an and as at but by en for if in of on or the to v[.]? via vs[.]?
-        while ( i > -1 ) {
-            if( result.mid( i + 1, 20).section( " ", 0, 0 ) == result.mid( i + 1, 20).section( " ", 0, 0 ).toLower() )        //if the word has some capitalization we suppose it's capitalized correctly. This solves issues with proper nouns and acronyms
-            {
-                if( !result.mid( i + 1, 20).section( " ", 0, 0 ).contains( littleWordRegExp ) )
-                {
-                    result[ i + 1 ] = result[ i + 1 ].toUpper();
-                    i = result.indexOf( wordRegExp, ++i );
-                }
-                else    //we have a little word
-                    i = result.indexOf( wordRegExp, ++i );
-            }
-        }
-        return result;   
+        return Amarok::CaseConverter::toTitleCase( s );
     }
     else
     {
-        debug() << "OUCH!";
+        error() << "Case option out of range! (" << caseOptions << ")";
         return s;
     }
 }
