@@ -1,7 +1,7 @@
 /* This file is part of the Amarok project
    Copyright (C) 2007 Maximilian Kossick <maximilian.kossick@googlemail.com>
    Copyright (C) 2007 Ian Monroe <ian@monroe.nu>
-   Copyright (C) 2008 Mark Kretschmann <kretschmann@kde.org> 
+   Copyright (C) 2008 Mark Kretschmann <kretschmann@kde.org>
 
    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
@@ -273,6 +273,20 @@ Meta::Track::addMatchTo( QueryMaker *qm )
     qm->addMatch( TrackPtr( this ) );
 }
 
+double
+Meta::Track::replayGain( Meta::Track::ReplayGainMode mode ) const
+{
+    Q_UNUSED( mode )
+    return 0.0;
+}
+
+double
+Meta::Track::replayPeakGain( Meta::Track::ReplayGainMode mode ) const
+{
+    Q_UNUSED( mode )
+    return 0.0;
+}
+
 void
 Meta::Track::prepareToPlay()
 {
@@ -392,23 +406,23 @@ Meta::Album::image( int size )
     // Return "nocover" until it's fetched.
     QDir cacheCoverDir = QDir( Amarok::saveLocation( "albumcovers/cache/" ) );
     if ( size <= 1 )
-        size = AmarokConfig::coverPreviewSize();
+        size = 100;
     QString sizeKey = QString::number( size ) + '@';
 
-    QImage img;
+    QPixmap pixmap;
     if( cacheCoverDir.exists( sizeKey + "nocover.png" ) )
-         img = QImage( cacheCoverDir.filePath( sizeKey + "nocover.png" ) );
+         pixmap = QPixmap( cacheCoverDir.filePath( sizeKey + "nocover.png" ) );
     else
     {
-        QImage orgImage = QImage( KStandardDirs::locate( "data", "amarok/images/nocover.png" ) ); //optimise this!
+        QPixmap orgPixmap = QPixmap( KStandardDirs::locate( "data", "amarok/images/nocover.png" ) ); //optimise this!
         //scaled() does not change the original image but returns a scaled copy
-        img = orgImage.scaled( size, size, Qt::KeepAspectRatio, Qt::SmoothTransformation );
-        img.save( cacheCoverDir.filePath( sizeKey + "nocover.png" ), "PNG" );
+        pixmap = orgPixmap.scaled( size, size, Qt::KeepAspectRatio, Qt::SmoothTransformation );
+        pixmap.save( cacheCoverDir.filePath( sizeKey + "nocover.png" ), "PNG" );
     }
 
     m_noCoverImage = true;
-    
-    return QPixmap::fromImage( img );
+
+    return pixmap;
 }
 
 
@@ -418,7 +432,7 @@ Meta::Album::imageWithBorder( int size, int borderWidth )
     QPixmap coverWithBorders;
 
     m_noCoverImage = false;
-    
+
     const int imageSize = size - borderWidth * 2;
     QPixmap cover = image( imageSize );
 
