@@ -169,6 +169,17 @@ namespace Meta
     {
         public:
 
+            /**
+             * The Replay Gain mode.
+             */
+            enum ReplayGainMode
+            {
+                /** All tracks should be equally loud.  Also known as Radio mode. */
+                TrackReplayGain,
+                /** All albums should be equally loud.  Also known as Audiophile mode. */
+                AlbumReplayGain
+            };
+
             virtual ~Track() {}
             /** used to display the trackname, should never be empty, returns prettyUrl() by default if name() is empty */
             virtual QString prettyName() const = 0;
@@ -218,10 +229,21 @@ namespace Meta
             virtual uint firstPlayed() const;
             /** Returns the number of times the track was played (what about unknown?)*/
             virtual int playCount() const = 0;
+            /**
+             * Returns the gain adjustment for a given replay gain mode.
+             *
+             * Should return @c 0 if no replay gain value is known.
+             *
+             * Should return the track replay gain if the album gain is requested but
+             * is not available.
+             */
+            virtual qreal replayGain( ReplayGainMode mode ) const;
+            /** Returns the peak (after gain adjustment) for a given replay gain mode */
+            virtual qreal replayPeakGain( ReplayGainMode mode ) const;
 
-            /** Returns the type of this track, e.g. "ogg", "mp3", "Stream" */
+            /** Returns the type of this track, e.g. "ogg", "mp3", "stream" */
             virtual QString type() const = 0;
-            
+
             /** tell the track to perform any prerequisite
              * operations before playing */
             virtual void prepareToPlay();
@@ -298,6 +320,10 @@ namespace Meta
             /** returns all tracks on this album */
             virtual TrackList tracks() = 0;
 
+            /**
+             * A note about image sizes:
+             *  when size is <= 1, return the full size image
+             */
             /** returns true if the album has a cover set */
             virtual bool hasImage( int size = 1 ) const { Q_UNUSED( size ); return false; }
             /** returns the cover of the album */
@@ -309,7 +335,7 @@ namespace Meta
             /** Returns true if it is possible to update the cover of the album */
             virtual bool canUpdateImage() const { return false; }
             /** updates the cover of the album */
-            virtual void setImage( const QImage &image) { Q_UNUSED( image ); } //TODO: choose parameter
+            virtual void setImage( const QPixmap &pixmap ) { Q_UNUSED( pixmap ); } //TODO: choose parameter
             /** removes the album art */
             virtual void removeImage() { }
 
