@@ -59,9 +59,10 @@ namespace Meta
             virtual ~Playlist() {}
             virtual QString name() const = 0;
             virtual QString prettyName() const = 0;
+            virtual QString description() const { return QString(); }
 
             /**override showing just the filename */
-            void setName( QString name ) { m_name = name; }
+            virtual void setName( const QString &name ) { m_name = name; }
 
             /** returns all tracks in this playlist */
             virtual TrackList tracks() = 0;
@@ -71,6 +72,8 @@ namespace Meta
             virtual void unsubscribe( PlaylistObserver *observer )
                     { m_observers.remove( observer ); };
 
+            virtual QStringList groups() { return QStringList(); }
+            virtual void setGroups( const QStringList &groups ) { Q_UNUSED(groups) };
 
             /* the following has been copied from Meta.h
             * it is my hope that we can integrate Playlists
