@@ -23,32 +23,27 @@
 #include "Debug.h"
 #include "meta/stream/Stream.h"
 #include "SqlStorage.h"
-
-#include "SqlPlaylistGroup.h"
+#include "playlistmanager/PlaylistManager.h"
+#include "playlistmanager/sql/SqlPlaylistGroup.h"
+#include "playlistmanager/sql/SqlUserPlaylistProvider.h"
 
 #include <typeinfo>
 
-using namespace Meta;
-
-SqlPlaylist::SqlPlaylist( const QString & name, const Meta::TrackList & tracks, SqlPlaylistGroupPtr parent, const QString &urlId )
-    : SqlPlaylistViewItem()
-    , Playlist()
-    , m_dbId( -1 )
+Meta::SqlPlaylist::SqlPlaylist( const QString & name, const Meta::TrackList
+        &tracks, Meta::SqlPlaylistGroupPtr parent, const QString &urlId )
+    : m_dbId( -1 )
     , m_parent( parent )
     , m_tracks( tracks )
     , m_name( name )
     , m_description( QString() )
     , m_urlId( urlId )
     , m_tracksLoaded( true )
-
 {
     saveToDb();
 }
 
-SqlPlaylist::SqlPlaylist( const QStringList & resultRow, SqlPlaylistGroupPtr parent )
-    : SqlPlaylistViewItem()
-    , Playlist()
-    , m_parent( parent )
+Meta::SqlPlaylist::SqlPlaylist( const QStringList & resultRow, Meta::SqlPlaylistGroupPtr parent )
+    : m_parent( parent )
     , m_tracksLoaded( false )
 {
     m_dbId = resultRow[0].toInt();
@@ -62,14 +57,43 @@ SqlPlaylist::SqlPlaylist( const QStringList & resultRow, SqlPlaylistGroupPtr par
 }
 
 
-SqlPlaylist::~SqlPlaylist()
+Meta::SqlPlaylist::~SqlPlaylist()
 {
 }
 
-bool SqlPlaylist::saveToDb( bool tracks )
+Meta::SqlPlaylistGroupPtr
+Meta::SqlPlaylist::parent() const
+{
+    return m_parent;
+}
+
+QStringList
+Meta::SqlPlaylist::groups()
+{
+    QStringList groups;
+    if( m_parent )
+        groups << m_parent->name();
+    return groups;
+}
+
+void
+Meta::SqlPlaylist::setGroups( const QStringList &groups )
 {
     DEBUG_BLOCK
-            
+    debug() << groups;
+    //HACK: fix this to use m_provider;
+    SqlUserPlaylistProvider *provider = dynamic_cast<SqlUserPlaylistProvider *>(The::playlistManager()->defaultUserPlaylists());
+    m_parent = provider->group( groups.first() );
+    saveToDb();
+
+    m_parent->clear();
+}
+
+bool
+Meta::SqlPlaylist::saveToDb( bool tracks )
+{
+    DEBUG_BLOCK
+
     int parentId = -1;
     if ( m_parent )
         parentId = m_parent->id();
@@ -79,19 +103,19 @@ bool SqlPlaylist::saveToDb( bool tracks )
     //figure out if we have a urlId and if this id is already in the db, if so, update it instead of creating a new one.
     if ( !m_urlId.isEmpty() ) {
 
-        debug() << "Checking " << m_urlId << " against db"; 
+        debug() << "Checking " << m_urlId << " against db";
 
         //check if urlId exists
         QString query = "SELECT id from playlists WHERE urlid='%1'";
         query = query.arg( sql->escape( m_urlId ) );
         QStringList result = sql->query( query );
-        
+
         if ( !result.isEmpty() ) {
 
             //set this id to the already existing one
             m_dbId =  result.at( 0 ).toInt();
             debug() << "Got existing playlist with id " << m_dbId;
-            
+
         }
 
     }
@@ -130,11 +154,12 @@ bool SqlPlaylist::saveToDb( bool tracks )
         m_tracks.clear();
         m_tracksLoaded = false;
     }
-    
+
     return true;
 }
 
-void SqlPlaylist::saveTracks()
+void
+Meta::SqlPlaylist::saveTracks()
 {
     int trackNum = 1;
     SqlStorage * sql =  CollectionManager::instance()->sqlStorage();
@@ -144,10 +169,13 @@ void SqlPlaylist::saveTracks()
         if ( trackPtr )
         {
             QString query = "INSERT INTO playlist_tracks ( playlist_id, track_num, url, title, album, artist, length, uniqueid ) VALUES ( %1, %2, '%3', '%4', '%5', '%6', %7, '%8' );";
-            query = query.arg( QString::number( m_dbId ), QString::number( trackNum ), sql->escape( trackPtr->playableUrl().url() ),
-                                sql->escape( trackPtr->prettyName() ), sql->escape( trackPtr->album()->prettyName() ),
-                                sql->escape( trackPtr->artist()->prettyName() ), QString::number( trackPtr->length() ),
-                                sql->escape( trackPtr->uidUrl() ) );
+            query = query.arg( QString::number( m_dbId ), QString::number( trackNum ),
+                        sql->escape( trackPtr->playableUrl().url() ),
+                        sql->escape( trackPtr->prettyName() ),
+                        sql->escape( trackPtr->album()->prettyName() ),
+                        sql->escape( trackPtr->artist()->prettyName() ),
+                        QString::number( trackPtr->length() ),
+                        sql->escape( trackPtr->uidUrl() ) );
             sql->insert( query, NULL );
 
             trackNum++;
@@ -155,7 +183,8 @@ void SqlPlaylist::saveTracks()
     }
 }
 
-TrackList SqlPlaylist::tracks()
+Meta::TrackList
+Meta::SqlPlaylist::tracks()
 {
     DEBUG_BLOCK
     if ( !m_tracksLoaded )
@@ -165,7 +194,8 @@ TrackList SqlPlaylist::tracks()
     return m_tracks;
 }
 
-void SqlPlaylist::loadTracks()
+void
+Meta::SqlPlaylist::loadTracks()
 {
     DEBUG_BLOCK
     QString query = "SELECT playlist_id, track_num, url, title, album, artist, length FROM playlist_tracks WHERE playlist_id=%1 ORDER BY track_num";
@@ -191,9 +221,11 @@ void SqlPlaylist::loadTracks()
 
                 MetaStream::Track * streamTrack = dynamic_cast<MetaStream::Track *> ( trackPtr.data() );
 
-                streamTrack->setTitle( row[3] );
-                streamTrack->setAlbum( row[4] );
-                streamTrack->setArtist( row[5] );
+                if ( streamTrack ) {
+                    streamTrack->setTitle( row[3] );
+                    streamTrack->setAlbum( row[4] );
+                    streamTrack->setArtist( row[5] );
+                }
 
             }
 
@@ -207,14 +239,15 @@ void SqlPlaylist::loadTracks()
     m_tracksLoaded = true;
 }
 
-
-void Meta::SqlPlaylist::rename(const QString & name)
+void
+Meta::SqlPlaylist::setName( const QString & name )
 {
     m_name = name;
     saveToDb( false ); //no need to resave all tracks
 }
 
-void Meta::SqlPlaylist::removeFromDb()
+void
+Meta::SqlPlaylist::removeFromDb()
 {
     DEBUG_BLOCK
     QString query = "DELETE FROM playlist_tracks WHERE playlist_id=%1";
@@ -226,15 +259,16 @@ void Meta::SqlPlaylist::removeFromDb()
     CollectionManager::instance()->sqlStorage()->query( query );
 }
 
-int Meta::SqlPlaylist::id()
+int
+Meta::SqlPlaylist::id()
 {
     return m_dbId;
 }
 
-
-void Meta::SqlPlaylist::reparent( SqlPlaylistGroupPtr parent )
+/*
+void
+Meta::SqlPlaylist::reparent( SqlPlaylistGroupPtr parent )
 {
     m_parent = parent;
     saveToDb( false );
-}
-
+}*/
