@@ -25,7 +25,7 @@
 #include "MainWindow.h"
 #include "meta/proxy/MetaProxy.h"
 #include "meta/MetaUtility.h"
-#include "meta/StreamInfoCapability.h"
+#include "meta/capabilities/StreamInfoCapability.h"
 #include "meta/stream/Stream.h"
 #include "meta/file/File.h"
 #include "PlaylistManager.h"
@@ -106,19 +106,24 @@ XSPFPlaylist::XSPFPlaylist( Meta::TrackList list )
 }
 
 XSPFPlaylist::~XSPFPlaylist()
-{
-}
+{}
 
 bool
 XSPFPlaylist::save( const QString &location, bool relative )
 {
     DEBUG_BLOCK
     Q_UNUSED( relative );
+
+    QFile::remove( location );
     QFile file( location );
 
     if( !file.open( QIODevice::WriteOnly ) )
     {
-        KMessageBox::sorry( The::mainWindow(), i18n( "Cannot write playlist (%1).", file.fileName() ) );
+        if( The::mainWindow() ) // MainWindow might already be destroyed at this point (at program shutdown)
+            KMessageBox::sorry( The::mainWindow(), i18n( "Cannot write playlist (%1).", file.fileName() ) );
+        else
+            warning() << QString( "Cannot write playlist (%1)." ).arg( file.fileName() ); 
+
         return false;
     }
 
@@ -158,29 +163,26 @@ XSPFPlaylist::tracks()
 
     foreach( const XSPFTrack &track, xspfTracks )
     {
-
-        TrackPtr trackPtr = CollectionManager::instance()->trackForUrl( track.location );
-        if ( trackPtr ){
+        TrackPtr trackPtr;
+        if( !track.identifier.isEmpty() )
+            trackPtr = CollectionManager::instance()->trackForUrl( track.identifier );
+        else
+            trackPtr = CollectionManager::instance()->trackForUrl( track.location );
+        if ( trackPtr )
+        {
             if( !trackPtr->isPlayable() )
-            {
                 trackPtr = CollectionManager::instance()->trackForUrl( track.identifier );
-            }
         }
 
-        if ( trackPtr ) {
-
-            if ( typeid( * trackPtr.data() ) == typeid( MetaStream::Track ) )  {
-
-                debug() << "got stream from trackForUrl, setting album to " << track.album;
-
+        if ( trackPtr )
+        {
+            if ( typeid( * trackPtr.data() ) == typeid( MetaStream::Track ) )
+            {
                 MetaStream::Track * streamTrack = dynamic_cast<MetaStream::Track *> ( trackPtr.data() );
-
                 streamTrack->setTitle( track.title );
                 streamTrack->setAlbum( track.album );
                 streamTrack->setArtist( track.creator );
-
             } 
-
             tracks << trackPtr;
         }
 
@@ -460,6 +462,7 @@ XSPFPlaylist::setLink( const KUrl &link )
 XSPFTrackList
 XSPFPlaylist::trackList()
 {
+    DEBUG_BLOCK
     XSPFTrackList list;
 
     QDomNode trackList = documentElement().namedItem( "trackList" );
@@ -484,10 +487,9 @@ XSPFPlaylist::trackList()
                     track.duration = subSubNode.firstChild().nodeValue().toInt();
                 else if ( subSubNode.nodeName() == "annotation" )
                     track.annotation = subSubNode.firstChild().nodeValue();
-                else if ( subSubNode.nodeName() == "album" ) {
+                else if ( subSubNode.nodeName() == "album" )
                     track.album = subSubNode.firstChild().nodeValue();
-                    debug() << "loaded album name from xspf: " << subSubNode.firstChild().nodeValue();
-                } else if ( subSubNode.nodeName() == "trackNum" )
+                else if ( subSubNode.nodeName() == "trackNum" )
                     track.trackNum = (uint)subSubNode.firstChild().nodeValue().toInt();
                 else if ( subSubNode.nodeName() == "identifier" )
                     track.identifier = subSubNode.firstChild().nodeValue();
