@@ -20,11 +20,18 @@
 #include "File.h"
 #include "File_p.h"
 
+#include "BookmarkMetaActions.h"
 #include "Meta.h"
-#include "meta/EditCapability.h"
-#include "meta/ImportCapability.h"
+#include "meta/capabilities/CurrentTrackActionsCapability.h"
+#include "meta/capabilities/EditCapability.h"
+#include "meta/capabilities/StatisticsCapability.h"
+#include "meta/capabilities/TimecodeWriteCapability.h"
+#include "meta/capabilities/TimecodeLoadCapability.h"
 #include "MetaUtility.h"
+#include "amarokurls/PlayUrlRunner.h"
+#include "context/popupdropper/libpud/PopupDropperAction.h"
 
+#include <QList>
 #include <QPointer>
 #include <QString>
 
@@ -57,11 +64,11 @@ class EditCapabilityImpl : public Meta::EditCapability
         KSharedPtr<MetaFile::Track> m_track;
 };
 
-class ImportCapabilityImpl : public Meta::ImportCapability
+class StatisticsCapabilityImpl : public Meta::StatisticsCapability
 {
     public:
-        ImportCapabilityImpl( MetaFile::Track *track )
-            : Meta::ImportCapability()
+        StatisticsCapabilityImpl( MetaFile::Track *track )
+            : Meta::StatisticsCapability()
             , m_track( track )
         {}
 
@@ -78,6 +85,55 @@ class ImportCapabilityImpl : public Meta::ImportCapability
         KSharedPtr<MetaFile::Track> m_track;
 };
 
+class TimecodeWriteCapabilityImpl : public Meta::TimecodeWriteCapability
+{
+    public:
+        TimecodeWriteCapabilityImpl( MetaFile::Track *track )
+            : Meta::TimecodeWriteCapability()
+            , m_track( track )
+        {}
+
+    virtual bool writeTimecode ( int seconds )
+    {
+        DEBUG_BLOCK
+        return Meta::TimecodeWriteCapability::writeTimecode( seconds, Meta::TrackPtr( m_track.data() ) );
+    }
+
+    virtual bool writeAutoTimecode ( int seconds )
+    {
+        DEBUG_BLOCK
+        return Meta::TimecodeWriteCapability::writeAutoTimecode( seconds, Meta::TrackPtr( m_track.data() ) );
+    }
+
+    private:
+        KSharedPtr<MetaFile::Track> m_track;
+};
+
+class TimecodeLoadCapabilityImpl : public Meta::TimecodeLoadCapability
+{
+    public:
+        TimecodeLoadCapabilityImpl( MetaFile::Track *track )
+        : Meta::TimecodeLoadCapability()
+        , m_track( track )
+        {}
+
+        virtual bool hasTimecodes()
+        {
+            if ( loadTimecodes().size() > 0 )
+                return true;
+            return false;
+        }
+
+        virtual BookmarkList loadTimecodes()
+        {
+            BookmarkList list = PlayUrlRunner::bookmarksFromUrl( m_track->playableUrl() );
+            return list;
+        }
+
+    private:
+        KSharedPtr<MetaFile::Track> m_track;
+};
+
 Track::Track( const KUrl &url )
     : Meta::Track()
     , d( new Track::Private( this ) )
@@ -420,6 +476,22 @@ Track::setPlayCount( int newCount )
     d->playCount = newCount;
 }
 
+qreal
+Track::replayGain( Meta::Track::ReplayGainMode mode ) const
+{
+    if ( mode == Meta::Track::AlbumReplayGain )
+        return d->m_data.trackGain;
+    return d->m_data.albumGain;
+}
+
+qreal
+Track::replayPeakGain( Meta::Track::ReplayGainMode mode ) const
+{
+    if ( mode == Meta::Track::AlbumReplayGain )
+        return d->m_data.trackPeak;
+    return d->m_data.albumPeak;
+}
+
 QString
 Track::type() const
 {
@@ -471,7 +543,11 @@ Track::collection() const
 bool
 Track::hasCapabilityInterface( Meta::Capability::Type type ) const
 {
-    return type == Meta::Capability::Editable || type == Meta::Capability::Importable;
+    return type == Meta::Capability::Editable || 
+           type == Meta::Capability::Importable || 
+           type == Meta::Capability::CurrentTrackActions || 
+           type == Meta::Capability::WriteTimecode || 
+           type == Meta::Capability::LoadTimecode;
 }
 
 Meta::Capability*
@@ -481,9 +557,20 @@ Track::asCapabilityInterface( Meta::Capability::Type type )
     {
         case Meta::Capability::Editable:
             return new EditCapabilityImpl( this );
-            break;
         case Meta::Capability::Importable:
-            return new ImportCapabilityImpl( this );
+            return new StatisticsCapabilityImpl( this );
+        case Meta::Capability::CurrentTrackActions:
+            {
+            QList< PopupDropperAction * > actions;
+            PopupDropperAction* flag = new BookmarkCurrentTrackPositionAction( 0 );
+            actions << flag;
+            debug() << "returning bookmarkcurrenttrack action";
+            return new Meta::CurrentTrackActionsCapability( actions );
+            }
+        case Meta::Capability::WriteTimecode:
+            return new TimecodeWriteCapabilityImpl( this );
+        case Meta::Capability::LoadTimecode:
+            return new TimecodeLoadCapabilityImpl( this );
         default:
             return 0;
     }
