@@ -27,6 +27,7 @@
 #define DEBUG_PREFIX "Playlist::GroupingProxy"
 
 #include "GroupingProxy.h"
+#include "playlist/navigators/NavigatorFilterProxyModel.h"
 
 #include "Debug.h"
 #include "meta/Meta.h"
@@ -51,10 +52,10 @@ Playlist::GroupingProxy::destroy()
     }
 }
 
-Playlist::GroupingProxy::GroupingProxy() : QAbstractProxyModel( 0 ) , m_model( NavigatorFilterProxyModel::instance() )
+Playlist::GroupingProxy::GroupingProxy()
+    : QAbstractProxyModel( 0 )
+    , m_model( NavigatorFilterProxyModel::instance() )
 {
-    DEBUG_BLOCK
-
     setSourceModel( m_model );
     // signal proxies
     connect( m_model, SIGNAL( dataChanged( const QModelIndex&, const QModelIndex& ) ), this, SLOT( modelDataChanged( const QModelIndex&, const QModelIndex& ) ) );
@@ -65,9 +66,8 @@ Playlist::GroupingProxy::GroupingProxy() : QAbstractProxyModel( 0 ) , m_model( N
 
     int max = m_model->rowCount();
     for ( int i = 0; i < max; i++ )
-    {
         m_rowGroupMode.append( None );
-    }
+    
     regroupRows( 0, max - 1 );
 
     s_instance = this;
@@ -75,20 +75,14 @@ Playlist::GroupingProxy::GroupingProxy() : QAbstractProxyModel( 0 ) , m_model( N
 
 Playlist::GroupingProxy::~GroupingProxy()
 {
-    DEBUG_BLOCK
 }
 
 QModelIndex
 Playlist::GroupingProxy::index( int r, int c, const QModelIndex& ) const
 {
     if ( m_model->rowExists( r ) )
-    {
         return createIndex( r, c );
-    }
-    else
-    {
-        return QModelIndex();
-    }
+    return QModelIndex();
 }
 
 QModelIndex
@@ -130,21 +124,15 @@ Playlist::GroupingProxy::data( const QModelIndex& index, int role ) const
     int row = index.row();
 
     if ( role == Playlist::GroupRole )
-    {
         return m_rowGroupMode.at( row );
-    }
+    
     else if ( role == Playlist::GroupedTracksRole )
-    {
         return groupRowCount( row );
-    }
+    
     else if ( role == Playlist::GroupedAlternateRole )
-    {
         return ( row % 2 == 1 );
-    }
-    else
-    {
-        return m_model->data( index, role );
-    }
+    
+    return m_model->data( index, role );
 }
 
 int
@@ -156,7 +144,6 @@ Playlist::GroupingProxy::activeRow() const
 void
 Playlist::GroupingProxy::setActiveRow( int row ) const
 {
-    DEBUG_BLOCK
     m_model->setActiveRow( row );
 }
 
@@ -205,7 +192,6 @@ Playlist::GroupingProxy::setCollapsed( int, bool ) const
 int
 Playlist::GroupingProxy::firstInGroup( int row ) const
 {
-    AMAROK_DEPRECATED
     if ( m_rowGroupMode.at( row ) == None )
         return row;
 
@@ -222,7 +208,6 @@ Playlist::GroupingProxy::firstInGroup( int row ) const
 int
 Playlist::GroupingProxy::lastInGroup( int row ) const
 {
-    AMAROK_DEPRECATED
     if ( m_rowGroupMode.at( row ) == None )
         return row;
 
@@ -273,7 +258,6 @@ Playlist::GroupingProxy::regroupAll()
 void
 Playlist::GroupingProxy::regroupRows( int first, int last )
 {
-
     /* This function maps row numbers to one of the GroupMode enums, according
      * to the following truth matrix:
      *
@@ -335,7 +319,10 @@ Playlist::GroupingProxy::groupRowCount( int row ) const
 bool
 Playlist::GroupingProxy::shouldBeGrouped( Meta::TrackPtr track1, Meta::TrackPtr track2 )
 {
-    return track1->album() == track2->album();
+    if( track1 && track1->album() && track2 && track2->album() )
+        return ( *track1->album().data() ) == ( *track2->album().data() );
+
+    return false;
 }
 
 int Playlist::GroupingProxy::find( const QString &searchTerm, int fields  )
@@ -373,8 +360,33 @@ int Playlist::GroupingProxy::currentSearchFields()
     return m_model->currentSearchFields();
 }
 
+int Playlist::GroupingProxy::tracksInGroup( int row ) const
+{
+    //unfortunately we need to map this to row from source as it will
+    //otherwise mess up ( and crash ) when a filter is applied
+    row = NavigatorFilterProxyModel::instance()->rowFromSource( row );
+
+    return ( lastInGroup( row ) - firstInGroup( row ) ) + 1;
+}
+
+int Playlist::GroupingProxy::lengthOfGroup( int row ) const
+{
+    //unfortunately we need to map this to row from source as it will
+    //otherwise mess up ( and crash ) when a filter is applied
+    row = NavigatorFilterProxyModel::instance()->rowFromSource( row );
+    
+    int totalLenght = 0;
+    for ( int i = firstInGroup( row ); i <= lastInGroup( row ); i++ ) {
+        totalLenght += m_model->trackAt( i )->length();
+    }
+
+    return totalLenght;
+}
+
 #endif
 
 
 
 
+
+
