@@ -47,21 +47,20 @@ Playlist::Actions* Playlist::Actions::s_instance = 0;
 
 Playlist::Actions* Playlist::Actions::instance()
 {
-    return ( s_instance ) ? s_instance : new Actions( 0 );
+    if( !s_instance )
+        s_instance = new Actions();
+    return s_instance;
 }
 
 void
 Playlist::Actions::destroy()
 {
-    if ( s_instance )
-    {
-        delete s_instance;
-        s_instance = 0;
-    }
+    delete s_instance;
+    s_instance = 0;
 }
 
-Playlist::Actions::Actions( QObject* parent )
-        : QObject( parent )
+Playlist::Actions::Actions()
+        : QObject()
         , EngineObserver( The::engineController() )
         , m_nextTrackCandidate( 0 )
         , m_currentTrack( 0 )
@@ -72,38 +71,35 @@ Playlist::Actions::Actions( QObject* parent )
         , m_waitingForNextTrack( false )
 {
     DEBUG_BLOCK
-    s_instance = this;
     playlistModeChanged(); // sets m_navigator.
     m_nextTrackCandidate = m_navigator->requestNextTrack();
 }
 
-
 Playlist::Actions::~Actions()
 {
     DEBUG_BLOCK
     m_navigator->deleteLater();
 }
 
-
 void
 Playlist::Actions::requestNextTrack()
 {
-    if ( m_nextTrackCandidate!=0 )return;
+    if ( m_nextTrackCandidate != 0 )
+        return;
+    if( m_trackError )
+        return;
     m_trackError = false;
-    if ( stopAfterMode()==StopAfterQueue && m_currentTrack==m_trackToBeLast )
-    {
-            setStopAfterMode( StopAfterCurrent );
-    }
+    m_currentTrack = Model::instance()->activeId();
+    if ( stopAfterMode() == StopAfterQueue && m_currentTrack == m_trackToBeLast )
+        setStopAfterMode( StopAfterCurrent );
+    
     m_nextTrackCandidate = m_navigator->requestNextTrack();
-    m_currentTrack=m_nextTrackCandidate;
-    if ( stopAfterMode()==StopAfterCurrent )  //stop after current / stop after track starts here
-    {
+    m_currentTrack = m_nextTrackCandidate;
+    
+    if ( stopAfterMode() == StopAfterCurrent )  //stop after current / stop after track starts here
         setStopAfterMode( StopNever );
-    }
     else
-    {
         play( m_nextTrackCandidate, false );
-    }
 }
 
 void
@@ -273,9 +269,9 @@ Playlist::Actions::queue( QList<int> rows )
 {
     foreach( int row, rows )
     {
-        quint64 id = Model::instance()->idAt( row );
+        quint64 id = The::playlistModel()->idAt( row );
         m_navigator->queueId( id );
-        Model::instance()->setRowQueued( row );
+        The::playlistModel()->setRowQueued( row );
     }
 }
 
@@ -284,9 +280,9 @@ Playlist::Actions::dequeue( QList<int> rows )
 {
     foreach( int row, rows )
     {
-        quint64 id = Model::instance()->idAt( row );
+        quint64 id = The::playlistModel()->idAt( row );
         m_navigator->dequeueId( id );
-        Model::instance()->setRowDequeued( row );
+        The::playlistModel()->setRowDequeued( row );
     }
 }
 
@@ -333,7 +329,12 @@ Playlist::Actions::engineNewTrackPlaying()
         else {
             warning() << "engineNewTrackPlaying:" << track->prettyName() << "does not match what the playlist controller thought it should be";
             if ( model->activeTrack() != track )
-                model->setActiveRow( model->rowForTrack( track ) ); // this will set active row to -1 if the track isn't in the playlist at all
+            {
+                if ( AmarokConfig::lastPlaying() > -1 )
+                    model->setActiveRow( AmarokConfig::lastPlaying() );
+                else
+                    model->setActiveRow( model->rowForTrack( track ) ); // this will set active row to -1 if the track isn't in the playlist at all
+            }
         }
     }
     else
