@@ -1,6 +1,6 @@
 /***************************************************************************
  * copyright        : (C) 2007-2008 Ian Monroe <ian@monroe.nu>
- *                    (C) 2007-2008 Nikolaj Hald Nielsen <nhnFreespirit@gmail.com>
+ *                    (C) 2007-2009 Nikolaj Hald Nielsen <nhnFreespirit@gmail.com>
  *                    (C) 2008 Seb Ruiz <ruiz@kde.org>
  *                    (C) 2008 Soren Harward <stharward@gmail.com>
  *
@@ -31,6 +31,7 @@
 #include "Debug.h"
 #include "DirectoryLoader.h"
 #include "EngineController.h"
+#include "MetaUtility.h"
 #include "PlaylistActions.h"
 #include "PlaylistController.h"
 #include "PlaylistItem.h"
@@ -38,11 +39,16 @@
 #include "UndoCommands.h"
 #include "playlistmanager/PlaylistManager.h"
 #include "services/ServicePluginManager.h" // used in constructor
+#include "GroupingProxy.h"
 
+#include <KGlobal>
+#include <KUrl>
+
+#include <QDate>
 #include <QStringList>
 #include <QTextDocument>
 
-#include <KUrl>
+
 
 #include <typeinfo>
 
@@ -148,19 +154,7 @@ Playlist::Model::headerData( int section, Qt::Orientation orientation, int role
     if ( role != Qt::DisplayRole )
         return QVariant();
 
-    switch ( section )
-    {
-    case 0:
-        return "title";
-    case 1:
-        return "album";
-    case 2:
-        return "artist";
-    case 3:
-        return "custom";
-    default:
-        return QVariant();
-    }
+    return columnNames[section];
 }
 
 QVariant
@@ -186,43 +180,193 @@ Playlist::Model::data( const QModelIndex& index, int role ) const
     else if ( role == QueuePositionRole )
         return Actions::instance()->queuePosition( idAt( row ) ) + 1;
 
+    else if ( role == InCollectionRole )
+        return  m_items.at( row )->track()->inCollection();
+
+    else if ( role == MultiSourceRole )
+        return  m_items.at( row )->track()->hasCapabilityInterface( Meta::Capability::MultiSource );
+
     else if ( role == Qt::DisplayRole || role == Qt::ToolTipRole )
     {
         switch ( index.column() )
         {
-        case 0:
-        {
-            return Qt::escape( m_items.at( row )->track()->name() );
-        }
-        case 1:
-        {
-            if ( m_items.at( row )->track()->album() )
-                return Qt::escape( m_items.at( row )->track()->album()->name() );
-            return QString();
-        }
-        case 2:
-        {
-            if ( m_items.at( row )->track()->artist() )
-                return Qt::escape( m_items.at( row )->track()->artist()->name() );
-            return QString();
-        }
-        case 3:
-        {
-            QString artist;
-            QString album;
-            QString track;
-
-            if ( m_items.at( row )->track() )
+            case PlaceHolder:
+                return QString();
+            case Album:
             {
-                track = m_items.at( row )->track()->name();
-                if ( m_items.at( row )->track()->artist() )
-                    artist = m_items.at( row )->track()->artist()->name();
-                if ( m_items.at( row )->track()->artist() )
-                    album = m_items.at( row )->track()->album()->name();
+                if ( m_items.at( row )->track()->album() )
+                    return m_items.at( row )->track()->album()->name();
+                return QString();
             }
+            case AlbumArtist:
+            {
+                if ( m_items.at( row )->track()->album() )
+                    if (  m_items.at( row )->track()->album()->albumArtist() )
+                        return m_items.at( row )->track()->album()->albumArtist()->name();
+                return QString();
+            }
+            case Artist:
+            {
+                if ( m_items.at( row )->track()->artist() )
+                    return m_items.at( row )->track()->artist()->name();
+                return QString();
+            }
+            case Bitrate:
+            {
+                return Meta::prettyBitrate( m_items.at( row )->track()->bitrate() );
+            }
+            case Bpm:
+            {
+                return 0;
+            }
+            case Comment:
+            {
+                return m_items.at( row )->track()->comment();
+            }
+            case Composer:
+            {
+                if ( m_items.at( row )->track()->composer() )
+                    return m_items.at( row )->track()->composer()->name();
+                return QString();
+            }
+            case CoverImage:
+            {
+                if ( m_items.at( row )->track()->album() )
+                    return m_items.at( row )->track()->album()->imageWithBorder( 100 ); //FIXME:size?
+                return QImage();
+            }
+            case Directory:
+            {
+                if ( m_items.at( row )->track()->playableUrl().isLocalFile() )
+                    return m_items.at( row )->track()->playableUrl().directory();
+                else
+                    return QString();
+            }
+            case DiscNumber:
+            {
+                return m_items.at( row )->track()->discNumber();
+            }
+            case Filename:
+            {
+
+                if ( m_items.at( row )->track()->playableUrl().isLocalFile() )
+                    return m_items.at( row )->track()->playableUrl().fileName();
+                else
+                    return QString();
+            }
+            case Filesize:
+            {
+                return Meta::prettyFilesize( m_items.at( row )->track()->filesize() );
+            }
+            case Genre:
+            {
+                if ( m_items.at( row )->track()->genre() )
+                    return m_items.at( row )->track()->genre()->name();
+                return QString();
+            }
+            case GroupLength:
+            {
+                return Meta::secToPrettyTime( GroupingProxy::instance()->lengthOfGroup( row ) );
+            }
+            case GroupTracks:
+            {
+                return i18np ( "1 track", "%1 tracks", GroupingProxy::instance()->tracksInGroup( row ) );
+            }
+            case LastPlayed:
+            {
+                if ( m_items.at( row )->track()->lastPlayed() == 0 )
+                    return i18n( "Never" );
+                else {
+                    QDateTime date = QDateTime::fromTime_t( m_items.at( row )->track()->lastPlayed() );
+                    return Amarok::verboseTimeSince( date );
+                }
+            }
+            case Length:
+            {
+                return  Meta::secToPrettyTime( m_items.at( row )->track()->length() );
+            }
+            case Mood:
+            {
+                return QString(); //FIXME
+            }
+            case PlayCount:
+            {
+                return m_items.at( row )->track()->playCount();
+            }
+            case Rating:
+            {
+                return m_items.at( row )->track()->rating();
+            }
+            case SampleRate:
+            {
+                return m_items.at( row )->track()->sampleRate();
+            }
+            case Score:
+            {
+                return (int)m_items.at( row )->track()->score(); // Cast to int, as we don't need to show the decimals in the view..
+            }
+            case Source:
+            {
+                QString sourceName;
+                Meta::SourceInfoCapability *sic = m_items.at( row )->track()->as<Meta::SourceInfoCapability>();
+                if ( sic )
+                {
+                    sourceName = sic->sourceName();
+                    delete sic;
+                }
+                else
+                {
+                    sourceName = m_items.at( row )->track()->collection() ? m_items.at( row )->track()->collection()->prettyName() : QString();
+                }
+                return sourceName;
+            }
+            case SourceEmblem:
+            {
+                QPixmap emblem;
+                Meta::SourceInfoCapability *sic = m_items.at( row )->track()->as<Meta::SourceInfoCapability>();
+                if ( sic )
+                {
+                    QString source = sic->sourceName();
+                    if ( !source.isEmpty() )
+                        emblem = sic->emblem();;
+                    delete sic;
+                }
+                return emblem;
+            }
+            case Title:
+            {
+                return m_items.at( row )->track()->prettyName();
+            }
+            case TitleWithTrackNum:
+            {
+                QString trackString;
+                QString trackName = m_items.at( row )->track()->prettyName();
+                if ( m_items.at( row )->track()->trackNumber() > 0 )
+                {
+                    QString trackNumber = QString::number( m_items.at( row )->track()->trackNumber() );
+                    trackString =  QString( trackNumber + " - " + trackName );
+                } else
+                    trackString = trackName;
+
+                return trackString;
+            }
+            case TrackNumber:
+            {
+                return m_items.at( row )->track()->trackNumber();
+            }
+            case Type:
+            {
+                return m_items.at( row )->track()->type();
+            }
+            case Year:
+            {
+                if ( m_items.at( row )->track()->year() )
+                    return m_items.at( row )->track()->year()->name();
+                return QString();
+            }
+            default:
+                return QString();
 
-            return Qt::escape( QString( "%1 - %2 - %3" ).arg( artist, album, track ) );
-        }
         }
     }
     // else
@@ -268,13 +412,8 @@ Playlist::Model::mimeData( const QModelIndexList &indexes ) const
 bool
 Playlist::Model::dropMimeData( const QMimeData* data, Qt::DropAction action, int row, int, const QModelIndex& parent )
 {
-    DEBUG_BLOCK
-
     if ( action == Qt::IgnoreAction )
-    {
-        debug() << "ignoring drop";
         return true;
-    }
 
     int beginRow;
     if ( row != -1 )
@@ -336,7 +475,6 @@ Playlist::Model::dropMimeData( const QMimeData* data, Qt::DropAction action, int
         dl->init( data->urls() );
         return true;
     }
-    debug() << "ignoring unknown drop data";
     return false;
 }
 
@@ -482,8 +620,6 @@ Playlist::Model::stateOfId( quint64 id ) const
 void
 Playlist::Model::metadataChanged( Meta::TrackPtr track )
 {
-    DEBUG_BLOCK
-
     const int size = m_items.size();
     for ( int i = 0; i < size; i++ )
     {
@@ -498,13 +634,9 @@ Playlist::Model::metadataChanged( Meta::TrackPtr track )
 void
 Playlist::Model::metadataChanged( Meta::AlbumPtr album )
 {
-    DEBUG_BLOCK
-
     Meta::TrackList tracks = album->tracks();
     foreach( Meta::TrackPtr track, tracks )
-    {
         metadataChanged( track );
-    }
 }
 
 bool
@@ -585,8 +717,6 @@ Playlist::Model::prettyColumnName( Column index ) //static
 void
 Playlist::Model::insertTracksCommand( const InsertCmdList& cmds )
 {
-    DEBUG_BLOCK
-
     if ( cmds.size() < 1 )
         return;
 
@@ -599,6 +729,7 @@ Playlist::Model::insertTracksCommand( const InsertCmdList& cmds )
         }
     }
 
+    int newActiveRow = m_activeRow;
     int min = m_items.size() + cmds.size();
     int max = 0;
     QList<quint64> newIds;
@@ -606,7 +737,7 @@ Playlist::Model::insertTracksCommand( const InsertCmdList& cmds )
     {
         min = qMin( min, ic.second );
         max = qMax( max, ic.second );
-        debug() << "inserting" << ic.first->prettyName() << "at" << ic.second;
+        newActiveRow += ( ic.second <= m_activeRow ) ? 1 : 0;
     }
 
     // actually do the insertion
@@ -616,6 +747,7 @@ Playlist::Model::insertTracksCommand( const InsertCmdList& cmds )
         Meta::TrackPtr track = ic.first;
         m_totalLength += track->length();
         subscribeTo( track );
+
         if ( track->album() )
             subscribeTo( track->album() );
 
@@ -642,6 +774,11 @@ Playlist::Model::insertTracksCommand( const InsertCmdList& cmds )
         }
     }
 
+    if( m_activeRow >= 0 )
+        m_activeRow = newActiveRow;
+    else
+        m_activeRow = -1;
+
     Amarok::actionCollection()->action( "playlist_clear" )->setEnabled( !m_items.isEmpty() );
     //Amarok::actionCollection()->action( "play_pause" )->setEnabled( !activeTrack().isNull() );
 }
@@ -649,8 +786,6 @@ Playlist::Model::insertTracksCommand( const InsertCmdList& cmds )
 void
 Playlist::Model::removeTracksCommand( const RemoveCmdList& cmds )
 {
-    DEBUG_BLOCK
-
     if ( cmds.size() < 1 )
         return;
 
@@ -664,12 +799,8 @@ Playlist::Model::removeTracksCommand( const RemoveCmdList& cmds )
         min = qMin( min, rc.second );
         max = qMax( max, rc.second );
         activeShift += ( rc.second < m_activeRow ) ? 1 : 0;
-        debug() << "removing" << rc.first->prettyName() << "from" << rc.second;
         if ( rc.second == m_activeRow )
-        {
-            debug() << "deleting the active track";
             activeDeleted = true;
-        }
     }
 
     /* This next bit is probably more complicated that you expected it to be.
@@ -744,8 +875,6 @@ Playlist::Model::removeTracksCommand( const RemoveCmdList& cmds )
 void
 Playlist::Model::moveTracksCommand( const MoveCmdList& cmds, bool reverse )
 {
-    DEBUG_BLOCK
-
     if ( cmds.size() < 1 )
         return;
 
@@ -757,7 +886,6 @@ Playlist::Model::moveTracksCommand( const MoveCmdList& cmds, bool reverse )
         min = qMin( min, rc.second );
         max = qMax( max, rc.first );
         max = qMax( max, rc.second );
-        debug() << "moving" << rc.first << "to" << rc.second;
     }
 
     int newActiveRow = m_activeRow;
@@ -788,27 +916,27 @@ Playlist::Model::moveTracksCommand( const MoveCmdList& cmds, bool reverse )
 
 namespace The
 {
-AMAROK_EXPORT Playlist::Model* playlistModel()
-{
-    return Playlist::Model::instance();
+    AMAROK_EXPORT Playlist::Model* playlistModel()
+    {
+        return Playlist::Model::instance();
+    }
 }
-}
-
 
 
 int Playlist::Model::find( const QString & searchTerm, int searchFields )
 {
-
     DEBUG_BLOCK
+
     m_currentSearchTerm = searchTerm;
     m_currentSearchFields = searchFields;
     int matchRow = -1;
     int row = 0;
-    foreach( Item* item, m_items ) {
-
+    foreach( Item* item, m_items )
+    {
         Meta::TrackPtr track = item->track();
 
-        if ( trackMatch( track, searchTerm, searchFields ) ) {
+        if ( trackMatch( track, searchTerm, searchFields ) )
+        {
             matchRow = row;
             break;
         }
@@ -817,28 +945,27 @@ int Playlist::Model::find( const QString & searchTerm, int searchFields )
     }
 
     return matchRow;
-
 }
 
 int Playlist::Model::findNext( const QString & searchTerm, int selectedRow, int searchFields )
 {
     DEBUG_BLOCK
+
     m_currentSearchTerm = searchTerm;
     m_currentSearchFields = searchFields;
-    int matchRow = -1;
     int row = 0;
     int firstMatch = -1;
-    foreach( Item* item, m_items ) {
-
+    foreach( Item* item, m_items )
+    {
         Meta::TrackPtr track = item->track();
 
-        if ( trackMatch( track, searchTerm, searchFields ) ) {
+        if ( trackMatch( track, searchTerm, searchFields ) )
+        {
             if ( firstMatch == -1 )
                 firstMatch = row;
 
-            if ( row > selectedRow ) {
+            if ( row > selectedRow )
                 return row;
-            }
         }
 
         row++;
@@ -852,32 +979,32 @@ int Playlist::Model::findNext( const QString & searchTerm, int selectedRow, int
 int Playlist::Model::findPrevious( const QString & searchTerm, int selectedRow, int searchFields )
 {
     DEBUG_BLOCK
+
     m_currentSearchTerm = searchTerm;
     m_currentSearchFields = searchFields;
-    int matchRow = -1;
     int row = m_items.count() -1;
     int lastMatch = -1;
 
     QList<Item*> tempItems = m_items;
-    while( tempItems.size() > 0 ) {
-
+    while( tempItems.size() > 0 )
+    {
         Item* item = tempItems.takeLast();
 
         Meta::TrackPtr track = item->track();
 
-        if ( trackMatch( track, searchTerm, searchFields ) ) {
+        if ( trackMatch( track, searchTerm, searchFields ) )
+        {
             if ( lastMatch == -1 )
                 lastMatch = row;
 
-            if ( row < selectedRow ) {
+            if ( row < selectedRow )
                 return row;
-            }
         }
 
         row--;
     }
 
-    //we have searche through everything and not found anything that matched _below_
+    //we have searched through everything and not found anything that matched _below_
     //the selected index. So return the first one found above it ( wrap around )
     return lastMatch;
 }
@@ -925,7 +1052,7 @@ bool Playlist::Model::trackMatch( Meta::TrackPtr track, const QString &searchTer
 void Playlist::Model::clearSearchTerm()
 {
     DEBUG_BLOCK
-    m_currentSearchTerm = QString();
+    m_currentSearchTerm.clear();
     m_currentSearchFields = 0;
 }
 
@@ -935,11 +1062,7 @@ bool Playlist::Model::matchesCurrentSearchTerm( int row ) const
     {
         if ( m_currentSearchTerm.isEmpty() )
             return true;
-        else
-            return trackMatch( m_items.at( row )->track(), m_currentSearchTerm, m_currentSearchFields );
-    }
-    else
-    {
-        return false;
+        return trackMatch( m_items.at( row )->track(), m_currentSearchTerm, m_currentSearchFields );
     }
+    return false;
 }
