@@ -22,15 +22,18 @@
 
 #include "ActionClasses.h"
 #include "App.h"
+#include "Debug.h"
 #include "MainWindow.h"
-#include "view/graphic/PlaylistGraphicsView.h"
-#include "PlaylistController.h"
-#include "view/listview/PrettyListView.h"
-#include "PlaylistHeader.h"
 #include "ToolBar.h"
+#include "PlaylistController.h"
+#include "PlaylistDefines.h"
+#include "PlaylistHeader.h"
 #include "PlaylistModel.h"
-#include "widgets/Widget.h"
+#include "layouts/LayoutManager.h"
+#include "navigators/NavigatorFilterProxyModel.h"
 #include "widgets/ProgressiveSearchWidget.h"
+#include "layouts/LayoutConfigAction.h"
+
 
 #include <KToolBarSpacerAction>
 
@@ -42,37 +45,60 @@ Playlist::Widget::Widget( QWidget* parent )
     setContentsMargins( 1, 1, 1, 1 );
 
     m_searchWidget = new ProgressiveSearchWidget( this );
+
+    //this is really only useful for debugging at the moment, so dont show it to users and testers
+    /*m_sortBox = new QComboBox( this );
+    m_sortBox->insertItem( 0, "Album", Album);
+    m_sortBox->insertItem( 1, "AlbumArtist", Album);
+    m_sortBox->insertItem( 2, "Artist", Artist );
+    m_sortBox->insertItem( 3, "Bitrate", Bitrate );
+    m_sortBox->insertItem( 4, "Bpm", Bpm );
+    m_sortBox->insertItem( 5, "Comment", Comment );
+    m_sortBox->insertItem( 6, "Composer", Composer );
+    m_sortBox->insertItem( 7, "Directory", Directory );
+    m_sortBox->insertItem( 8, "DiscNumber", DiscNumber );
+    m_sortBox->insertItem( 9, "Filename", Filename );
+    m_sortBox->insertItem( 10, "Filesize", Filesize );
+    m_sortBox->insertItem( 11, "Genre", Genre );
+    m_sortBox->insertItem( 12, "LastPlayed", LastPlayed );
+    m_sortBox->insertItem( 13, "Length", Length );
+    m_sortBox->insertItem( 14, "Mood", Mood );
+    m_sortBox->insertItem( 15, "PlayCount", PlayCount );
+    m_sortBox->insertItem( 16, "Rating", Rating );
+    m_sortBox->insertItem( 17, "SampleRate", SampleRate );
+    m_sortBox->insertItem( 18, "Score", Score );
+    m_sortBox->insertItem( 29, "Source", Source );
+    m_sortBox->insertItem( 30, "Title", Title );
+    m_sortBox->insertItem( 31, "TrackNumber", TrackNumber );
+    m_sortBox->insertItem( 32, "Type", Type );
+    m_sortBox->insertItem( 33, "Year", Year );
+
+    connect( m_sortBox, SIGNAL( activated( int ) ), this, SLOT( sort( int ) ) );*/
     
-    Amarok::Widget * layoutHolder = new Amarok::Widget( this );
+    QWidget * layoutHolder = new QWidget( this );
 
     layoutHolder->setMinimumWidth( 100 );
     layoutHolder->setMinimumHeight( 200 );
-
+    
     QVBoxLayout* mainPlaylistlayout = new QVBoxLayout( layoutHolder );
     mainPlaylistlayout->setContentsMargins( 0, 0, 0, 0 );
 
-    PrettyListView* playView = new PrettyListView( this );
-    playView->show();
-    m_playlistView = qobject_cast<QWidget*>( playView );
+    m_playlistView = new PrettyListView( this );
+    m_playlistView->show();
 
-    connect( m_searchWidget, SIGNAL( filterChanged( const QString &, int ) ), playView, SLOT( find( const QString &, int ) ) );
-    connect( m_searchWidget, SIGNAL( next( const QString &, int ) ), playView, SLOT( findNext( const QString &, int ) ) );
-    connect( m_searchWidget, SIGNAL( previous( const QString &, int ) ), playView, SLOT( findPrevious( const QString &, int ) ) );
-    connect( m_searchWidget, SIGNAL( filterCleared() ), playView, SLOT( clearSearchTerm() ) );
-    connect( m_searchWidget, SIGNAL( showOnlyMatches( bool ) ), playView, SLOT( showOnlyMatches( bool ) ) );
+    connect( m_searchWidget, SIGNAL( filterChanged( const QString &, int, bool ) ), m_playlistView, SLOT( find( const QString &, int, bool ) ) );
+    connect( m_searchWidget, SIGNAL( next( const QString &, int ) ), m_playlistView, SLOT( findNext( const QString &, int ) ) );
+    connect( m_searchWidget, SIGNAL( previous( const QString &, int ) ), m_playlistView, SLOT( findPrevious( const QString &, int ) ) );
+    connect( m_searchWidget, SIGNAL( filterCleared() ), m_playlistView, SLOT( clearSearchTerm() ) );
+    connect( m_searchWidget, SIGNAL( showOnlyMatches( bool ) ), m_playlistView, SLOT( showOnlyMatches( bool ) ) );
 
-    connect( playView, SIGNAL( found() ), m_searchWidget, SLOT( match() ) );
-    connect( playView, SIGNAL( notFound() ), m_searchWidget, SLOT( noMatch() ) );
+    connect( m_playlistView, SIGNAL( found() ), m_searchWidget, SLOT( match() ) );
+    connect( m_playlistView, SIGNAL( notFound() ), m_searchWidget, SLOT( noMatch() ) );
 
+    connect( LayoutManager::instance(), SIGNAL( activeLayoutChanged() ), m_playlistView, SLOT( reset() ) );
 
     mainPlaylistlayout->setSpacing( 0 );
-    mainPlaylistlayout->addWidget( playView );
-
-    m_stackedWidget = new Amarok::StackedWidget( this );
-
-    m_stackedWidget->addWidget( layoutHolder );
-
-    m_stackedWidget->setCurrentIndex( 0 );
+    mainPlaylistlayout->addWidget( m_playlistView );
 
     KHBox *barBox = new KHBox( this );
     barBox->setMargin( 0 );
@@ -82,10 +108,6 @@ Playlist::Widget::Widget( QWidget* parent )
 
     Model::instance();
 
-    KAction *action = new KAction( KIcon( "view-media-playlist-amarok" ), i18nc( "switch view", "Switch Playlist &View" ), this );
-    connect( action, SIGNAL( triggered( bool ) ), this, SLOT( switchView() ) );
-    Amarok::actionCollection()->addAction( "playlist_switch", action );
-
     // the Controller ctor creates the undo/redo actions that we use below, so we want
     // to make sure that it's been constructed and the the actions registered
     Controller::instance();
@@ -101,7 +123,7 @@ Playlist::Widget::Widget( QWidget* parent )
         
         //FIXME this action should go in ActionController, but we don't have any visibility to the view
         KAction *action = new KAction( KIcon( "music-amarok" ), i18n("Show active track"), this );
-        connect( action, SIGNAL( triggered( bool ) ), playView, SLOT( scrollToActiveTrack() ) );
+        connect( action, SIGNAL( triggered( bool ) ), m_playlistView, SLOT( scrollToActiveTrack() ) );
         plBar->addAction( action );
 
         plBar->addSeparator();
@@ -110,10 +132,14 @@ Playlist::Widget::Widget( QWidget* parent )
         plBar->addSeparator();
         plBar->addAction( Amarok::actionCollection()->action( "playlist_save" ) );
         plBar->addAction( Amarok::actionCollection()->action( "playlist_export" ) );
+        plBar->addSeparator();
+
+        Playlist::LayoutConfigAction *layoutConfigAction = new Playlist::LayoutConfigAction( this );
+        plBar->addAction( layoutConfigAction );
+        QToolButton *tbutton = qobject_cast<QToolButton*>(plBar->widgetForAction( layoutConfigAction ) );
+        if( tbutton )
+            tbutton->setPopupMode( QToolButton::InstantPopup );
 
-        // Alternate playlist view disabled for 2.0
-        //plBar->addSeparator();
-        //plBar->addAction( Amarok::actionCollection()->action( "playlist_switch") );
         plBar->addAction( new KToolBarSpacerAction( this ) );
 
     } //END Playlist Toolbar
@@ -128,10 +154,13 @@ Playlist::Widget::sizeHint() const
     return QSize( static_cast<QWidget*>( parent() )->size().width() / 4 , 300 );
 }
 
-void
-Playlist::Widget::switchView()
+void Playlist::Widget::sort( int index )
 {
-    m_stackedWidget->setCurrentIndex(( m_stackedWidget->currentIndex() + 1 ) % 2 );
+    DEBUG_BLOCK
+    int field = m_sortBox->itemData( index ).toInt();
+    debug() << "Field: " << field;
+    //The::playlistModel()->sort( field );
+    NavigatorFilterProxyModel::instance()->sort( field );
 }
 
 
