@@ -22,17 +22,31 @@
 
 #include "PrettyListView.h"
 
+#include "amarokconfig.h"
+#include "context/ContextView.h"
+#include "context/popupdropper/libpud/PopupDropperAction.h"
+#include "context/popupdropper/libpud/PopupDropperItem.h"
+#include "context/popupdropper/libpud/PopupDropper.h"
 #include "Debug.h"
+#include "EngineController.h"
 #include "PrettyItemDelegate.h"
 #include "dialogs/TagDialog.h"
+#include "GlobalCurrentTrackActions.h"
+#include "meta/capabilities/CurrentTrackActionsCapability.h"
+#include "meta/capabilities/MultiSourceCapability.h"
 #include "meta/Meta.h"
 #include "PaletteHandler.h"
 #include "playlist/GroupingProxy.h"
 #include "playlist/PlaylistActions.h"
 #include "playlist/PlaylistController.h"
 #include "playlist/view/PlaylistViewCommon.h"
+#include "playlist/navigators/NavigatorFilterProxyModel.h"
+#include "PopupDropperFactory.h"
+#include "SvgHandler.h"
+#include "SourceSelectionPopup.h"
 
 #include <KApplication>
+#include <KMenu>
 
 #include <QContextMenuEvent>
 #include <QDropEvent>
@@ -48,8 +62,11 @@
 
 Playlist::PrettyListView::PrettyListView( QWidget* parent )
         : QListView( parent )
+        , EngineObserver( The::engineController() )
         , m_headerPressIndex( QModelIndex() )
         , m_mousePressInHeader( false )
+        , m_skipAutoScroll( false )
+        , m_pd( 0 )
 {
     setModel( GroupingProxy::instance() );
     setItemDelegate( new PrettyItemDelegate( this ) );
@@ -65,7 +82,7 @@ Playlist::PrettyListView::PrettyListView( QWidget* parent )
     setAlternatingRowColors( true) ;
     The::paletteHandler()->updateItemView( this );
     connect( The::paletteHandler(), SIGNAL( newPalette( const QPalette & ) ), SLOT( newPalette( const QPalette & ) ) );
-    
+
     setAutoFillBackground( false );
 
     // signal connections
@@ -79,6 +96,13 @@ Playlist::PrettyListView::PrettyListView( QWidget* parent )
 
 Playlist::PrettyListView::~PrettyListView() {}
 
+void
+Playlist::PrettyListView::engineNewTrackPlaying()
+{
+    if( AmarokConfig::autoScrollPlaylist() )
+        scrollToActiveTrack();
+}
+
 void
 Playlist::PrettyListView::editTrackInformation()
 {
@@ -105,8 +129,13 @@ void
 Playlist::PrettyListView::removeSelection()
 {
     QList<int> sr = selectedRows();
+    qSort( sr );
+    int firstRow = sr.first();
     Controller::instance()->removeRows( sr );
-    selectionModel()->clearSelection();
+
+    //select the track immediately above the cleared are as this is the one that ow has internal focus.
+    firstRow = qBound( 0, firstRow, model()->rowCount() -1 );
+    selectionModel()->select( model()->index( firstRow, 0, QModelIndex() ), QItemSelectionModel::Select );
 }
 
 void
@@ -121,28 +150,69 @@ Playlist::PrettyListView::dequeueSelection()
     Actions::instance()->dequeue( selectedRows() );
 }
 
+void Playlist::PrettyListView::selectSource()
+{
+
+    DEBUG_BLOCK
+
+    QList<int> rows = selectedRows();
+
+    //for now, bail out of more than 1 row...
+    if ( rows.count() != 1 )
+        return;
+
+    //get the track...
+    QModelIndex index = GroupingProxy::instance()->index( rows.at( 0 ) );
+    Meta::TrackPtr track = index.data( Playlist::TrackRole ).value< Meta::TrackPtr >();
+
+    //get multiSource capability:
+
+    Meta::MultiSourceCapability *msc = track->as<Meta::MultiSourceCapability>();
+    if ( msc )
+    {
+        debug() << "sources: " << msc->sources();
+        SourceSelectionPopup * sourceSelector = new SourceSelectionPopup( this, msc );
+        sourceSelector->show();
+        //dialog deletes msc when done with it.
+    }
+
+}
+
+
 void
 Playlist::PrettyListView::scrollToActiveTrack()
 {
+    if( m_skipAutoScroll )
+    {
+        m_skipAutoScroll = false;
+        return;
+    }
     QModelIndex activeIndex = model()->index( GroupingProxy::instance()->activeRow(), 0, QModelIndex() );
     if ( activeIndex.isValid() )
         scrollTo( activeIndex, QAbstractItemView::PositionAtCenter );
 }
 
 void
-Playlist::PrettyListView::trackActivated( const QModelIndex& idx ) const
+Playlist::PrettyListView::trackActivated( const QModelIndex& idx )
 {
+    m_skipAutoScroll = true; // we don't want to do crazy view changes when selecting an item in the view
     Actions::instance()->play( idx );
 }
 
 void
 Playlist::PrettyListView::contextMenuEvent( QContextMenuEvent* event )
 {
-    QModelIndex index = indexAt( event->pos() );
+    DEBUG_BLOCK
+    QModelIndex filteredIndex = indexAt( event->pos() );
     
+    //translate to real model as we might be looking at a filered list:
+    int sourceRow = NavigatorFilterProxyModel::instance()->rowToSource( filteredIndex.row() );
+
+    QModelIndex index = The::playlistModel()->index( sourceRow );
+
     if ( !index.isValid() )
         return;
-    
+
     //Ctrl + Right Click is used for queuing
     if( event->modifiers() & Qt::ControlModifier )
         return;
@@ -249,18 +319,16 @@ Playlist::PrettyListView::mousePressEvent( QMouseEvent* event )
     {
         m_mousePressInHeader = true;
         QModelIndex index = indexAt( event->pos() );
-        //only do this dance if header is not already selected
-        if( !selectionModel()->isSelected( index ) )
-        {
-            m_headerPressIndex = QPersistentModelIndex( index );
-            int rows = index.data( GroupedTracksRole ).toInt();
-            QModelIndex bottomIndex = model()->index( index.row() + rows - 1, 0 );
-            QItemSelection selItems( index, bottomIndex );
-            QItemSelectionModel::SelectionFlags command = headerPressSelectionCommand( index, event );
-            selectionModel()->select( selItems, command );
-            // TODO: if you're doing shift-select on rows above the header, then the rows following the header will be lost from the selection
-            selectionModel()->setCurrentIndex( index, QItemSelectionModel::NoUpdate );
-        }
+        m_headerPressIndex = QPersistentModelIndex( index );
+        int rows = index.data( GroupedTracksRole ).toInt();
+        QModelIndex bottomIndex = model()->index( index.row() + rows - 1, 0 );
+
+        //offset by 1 as the actual header item is selected in QListView::mousePressEvent( event ); and is otherwise deselected again
+        QItemSelection selItems( model()->index( index.row() + 1, 0 ), bottomIndex );
+        QItemSelectionModel::SelectionFlags command = headerPressSelectionCommand( index, event );
+        selectionModel()->select( selItems, command );
+        // TODO: if you're doing shift-select on rows above the header, then the rows following the header will be lost from the selection
+        selectionModel()->setCurrentIndex( index, QItemSelectionModel::NoUpdate );
     }
     else
     {
@@ -269,8 +337,8 @@ Playlist::PrettyListView::mousePressEvent( QMouseEvent* event )
 
     // This should always be forwarded, as it is used to determine the offset
     // relative to the mouse of the selection we are dragging!
-    QListView::mousePressEvent( event ); 
-    
+    QListView::mousePressEvent( event );
+
     // This must go after the call to the super class as the current index is not yet selected otherwise
     // Queueing support for Ctrl Right click
     if( event->button() == Qt::RightButton && event->modifiers() & Qt::ControlModifier )
@@ -311,10 +379,7 @@ Playlist::PrettyListView::mouseEventInHeader( const QMouseEvent* event ) const
         mousePressPos.ry() += verticalOffset();
         return PrettyItemDelegate::insideItemHeader( mousePressPos, rectForIndex( index ) );
     }
-    else
-    {
-        return false;
-    }
+    return false;
 }
 
 void
@@ -331,6 +396,49 @@ Playlist::PrettyListView::paintEvent( QPaintEvent* event )
     QListView::paintEvent( event );
 }
 
+void
+Playlist::PrettyListView::startDrag( Qt::DropActions supportedActions )
+{
+    DEBUG_BLOCK
+
+    //Waah? when a parent item is dragged, startDrag is called a bunch of times
+    static bool ongoingDrags = false;
+    if( ongoingDrags )
+        return;
+    ongoingDrags = true;
+
+    if( !m_pd )
+        m_pd = The::popupDropperFactory()->createPopupDropper( Context::ContextView::self() );
+
+    if( m_pd && m_pd->isHidden() )
+    {
+
+        m_pd->setSvgRenderer( The::svgHandler()->getRenderer( "amarok/images/pud_items.svg" ) );
+        qDebug() << "svgHandler SVG renderer is " << (QObject*)(The::svgHandler()->getRenderer( "amarok/images/pud_items.svg" ));
+        qDebug() << "m_pd SVG renderer is " << (QObject*)(m_pd->svgRenderer());
+        qDebug() << "does play exist in renderer? " << ( The::svgHandler()->getRenderer( "amarok/images/pud_items.svg" )->elementExists( "load" ) );
+        QModelIndexList indices = selectedIndexes();
+
+        QList<PopupDropperAction*> actions =  ViewCommon::actionsFor( this, &indices.first(), true );
+
+        foreach( PopupDropperAction * action, actions )
+            m_pd->addItem( The::popupDropperFactory()->createItem( action ), true );
+
+        m_pd->show();
+    }
+
+    QListView::startDrag( supportedActions );
+    debug() << "After the drag!";
+
+    if( m_pd )
+    {
+        debug() << "clearing PUD";
+        connect( m_pd, SIGNAL( fadeHideFinished() ), m_pd, SLOT( clear() ) );
+        m_pd->hide();
+    }
+    ongoingDrags = false;
+}
+
 QItemSelectionModel::SelectionFlags
 Playlist::PrettyListView::headerPressSelectionCommand( const QModelIndex& index, const QMouseEvent* event ) const
 {
@@ -341,13 +449,13 @@ Playlist::PrettyListView::headerPressSelectionCommand( const QModelIndex& index,
     //const bool controlKeyPressed = event->modifiers() & Qt::ControlModifier;
     const bool indexIsSelected = selectionModel()->isSelected( index );
     const bool controlKeyPressed = event->modifiers() & Qt::ControlModifier;
-    
+
     if ( shiftKeyPressed )
         return QItemSelectionModel::SelectCurrent;
 
     if ( indexIsSelected && controlKeyPressed ) //make this consistent with how single items work. This also makes it possible to drag the header
         return QItemSelectionModel::Deselect;
-    
+
     return QItemSelectionModel::Select;
 }
 
@@ -371,7 +479,8 @@ Playlist::PrettyListView::selectedRows() const
     QList<int> rows;
     foreach( const QModelIndex &idx, selectedIndexes() )
     {
-        rows.append( idx.row() );
+        int sourceRow = NavigatorFilterProxyModel::instance()->rowToSource( idx.row() );
+        rows.append( sourceRow );
     }
     return rows;
 }
@@ -385,31 +494,33 @@ void Playlist::PrettyListView::newPalette( const QPalette & palette )
 
 
 
-void Playlist::PrettyListView::find( const QString &searchTerm, int fields  )
+void Playlist::PrettyListView::find( const QString &searchTerm, int fields, bool filter  )
 {
     DEBUG_BLOCK
 
     bool updateProxy = false;
     if ( ( GroupingProxy::instance()->currentSearchFields() != fields ) || ( GroupingProxy::instance()->currentSearchTerm() != searchTerm ) )
         updateProxy = true;
-            
-    int row = GroupingProxy::instance()->find( searchTerm, fields );
-    if( row != -1 ) {
-        //select this track
-        debug() << "Got match at row: " << row;
-        
-        QModelIndex index = model()->index( row, 0 );
-        QItemSelection selItems( index, index );
-        selectionModel()->select( selItems, QItemSelectionModel::SelectCurrent );
 
-        QModelIndex foundIndex = model()->index( row, 0, QModelIndex() );
-        if ( foundIndex.isValid() )
-            scrollTo( foundIndex, QAbstractItemView::PositionAtCenter );
+    int row = GroupingProxy::instance()->find( searchTerm, fields );
+    if( row != -1 )
+    {
+        //select this track
+
+        if ( !filter ) {
+            QModelIndex index = model()->index( row, 0 );
+            QItemSelection selItems( index, index );
+            selectionModel()->select( selItems, QItemSelectionModel::SelectCurrent );
+
+            QModelIndex foundIndex = model()->index( row, 0, QModelIndex() );
+            if ( foundIndex.isValid() )
+                scrollTo( foundIndex, QAbstractItemView::PositionAtCenter );
+        }
 
         emit( found() );
-    } else {
-        emit( notFound() );
     }
+    else
+        emit( notFound() );
 
 
     //instead of kicking the proxy right away, start a 500msec timeout.
@@ -419,7 +530,6 @@ void Playlist::PrettyListView::find( const QString &searchTerm, int fields  )
     //before this  timeout. Only start count if values have actually changed!
     if ( updateProxy )
         startProxyUpdateTimeout();
-
 }
 
 void Playlist::PrettyListView::findNext( const QString & searchTerm, int fields )
@@ -436,12 +546,12 @@ void Playlist::PrettyListView::findNext( const QString & searchTerm, int fields
         currentRow = selected.last();
 
     debug() << "current row is: " << currentRow;
-    
+
     int row = GroupingProxy::instance()->findNext( searchTerm, currentRow, fields );
-    if( row != -1 ) {
+    if( row != -1 )
+    {
         //select this track
-        debug() << "Got match at row: " << row;
-        
+
         QModelIndex index = model()->index( row, 0 );
         QItemSelection selItems( index, index );
         selectionModel()->select( selItems, QItemSelectionModel::SelectCurrent );
@@ -449,11 +559,11 @@ void Playlist::PrettyListView::findNext( const QString & searchTerm, int fields
         QModelIndex foundIndex = model()->index( row, 0, QModelIndex() );
         if ( foundIndex.isValid() )
             scrollTo( foundIndex, QAbstractItemView::PositionAtCenter );
-        
+
         emit( found() );
-    } else {
-        emit( notFound() );
     }
+    else
+        emit( notFound() );
 
     if ( updateProxy )
         NavigatorFilterProxyModel::instance()->filterUpdated();
@@ -473,12 +583,12 @@ void Playlist::PrettyListView::findPrevious( const QString & searchTerm, int fie
         currentRow = selected.first();
 
     debug() << "current row is: " << currentRow;
-    
+
     int row = GroupingProxy::instance()->findPrevious( searchTerm, currentRow, fields );
-    if( row != -1 ) {
+    if( row != -1 )
+    {
         //select this track
-        debug() << "Got match at row: " << row;
-        
+
         QModelIndex index = model()->index( row, 0 );
         QItemSelection selItems( index, index );
         selectionModel()->select( selItems, QItemSelectionModel::SelectCurrent );
@@ -486,11 +596,11 @@ void Playlist::PrettyListView::findPrevious( const QString & searchTerm, int fie
         QModelIndex foundIndex = model()->index( row, 0, QModelIndex() );
         if ( foundIndex.isValid() )
             scrollTo( foundIndex, QAbstractItemView::PositionAtCenter );
-        
+
         emit( found() );
-    } else {
-        emit( notFound() );
     }
+    else
+        emit( notFound() );
 
     if ( updateProxy )
         NavigatorFilterProxyModel::instance()->filterUpdated();
@@ -498,8 +608,26 @@ void Playlist::PrettyListView::findPrevious( const QString & searchTerm, int fie
 
 void Playlist::PrettyListView::clearSearchTerm()
 {
+    DEBUG_BLOCK
+            
+    //We really do not want to reset the view to the top when the search/filter is cleared, so
+    //we store the first shown row and scroll to that once the term is removed.
+    QModelIndex index = indexAt( QPoint( 0, 0 ) );
+
+     //ah... but we want the source row and not the one reported by the filter model(s)
+    int row = NavigatorFilterProxyModel::instance()->rowToSource( index.row() );
+
+    debug() << "first row in filtered list: " << index.row();
+    debug() << "source row: " << row;
+
     NavigatorFilterProxyModel::instance()->filterUpdated();
     GroupingProxy::instance()->clearSearchTerm();
+
+    //now scroll to the selected row again
+
+    QModelIndex sourceIndex = model()->index( row, 0, QModelIndex() );
+    if ( sourceIndex.isValid() )
+        scrollTo( sourceIndex, QAbstractItemView::PositionAtTop );
 }
 
 void Playlist::PrettyListView::startProxyUpdateTimeout()
@@ -524,4 +652,5 @@ void Playlist::PrettyListView::showOnlyMatches( bool onlyMatches )
 }
 
 
+
 #include "PrettyListView.moc"
