@@ -21,11 +21,15 @@
 #ifndef PRETTYLISTVIEW_H
 #define PRETTYLISTVIEW_H
 
+#include "EngineObserver.h"
+
 #include <QListView>
 #include <QModelIndex>
 #include <QPersistentModelIndex>
 #include <QRect>
 
+class PopupDropper;
+class PopupDropperAction;
 class QContextMenuEvent;
 class QDragLeaveEvent;
 class QDragMoveEvent;
@@ -37,7 +41,7 @@ class QTimer;
 
 namespace Playlist
 {
-class PrettyListView : public QListView
+class PrettyListView : public QListView, public EngineObserver
 {
     Q_OBJECT
 
@@ -47,7 +51,7 @@ public:
 
 signals:
     void found();
-    void notFound(); 
+    void notFound();
 
     // these slots are used by the ContextMenu
 public slots:
@@ -58,18 +62,21 @@ public slots:
     void removeSelection();
     void stopAfterTrack();
     void scrollToActiveTrack();
+    void selectSource();
 
-    void find( const QString & searchTerm, int fields );
+    void find( const QString & searchTerm, int fields, bool filter );
     void findNext( const QString & searchTerm, int fields  );
     void findPrevious( const QString & searchTerm, int fields  );
     void clearSearchTerm();
     void showOnlyMatches( bool onlyMatches );
 
+    virtual void engineNewTrackPlaying(); // from EngineObserver
+
 protected slots:
     void newPalette( const QPalette & palette );
 
 private slots:
-    void trackActivated( const QModelIndex& ) const;
+    void trackActivated( const QModelIndex& );
     void updateProxyTimeout();
 
 private:
@@ -81,20 +88,24 @@ private:
     void mousePressEvent( QMouseEvent* );
     void mouseReleaseEvent( QMouseEvent* );
     void paintEvent( QPaintEvent* );
+    void startDrag( Qt::DropActions supportedActions );
 
     bool mouseEventInHeader( const QMouseEvent* ) const;
     QItemSelectionModel::SelectionFlags headerPressSelectionCommand( const QModelIndex&, const QMouseEvent* ) const;
     QItemSelectionModel::SelectionFlags headerReleaseSelectionCommand( const QModelIndex&, const QMouseEvent* ) const;
 
-    QList<int> selectedRows() const;
-
     void startProxyUpdateTimeout();
 
-    QRect m_dropIndicator;
+    QRect                 m_dropIndicator;
     QPersistentModelIndex m_headerPressIndex;
-    bool m_mousePressInHeader;
+    bool                  m_mousePressInHeader;
+    bool                  m_skipAutoScroll;
 
-    QTimer * m_proxyUpdateTimer;
+    QTimer       *m_proxyUpdateTimer;
+    PopupDropper *m_pd;
+
+public:
+    QList<int> selectedRows() const;
 };
 }
 #endif
