@@ -25,14 +25,15 @@
 #include "PlaylistFileSupport.h"
 #include "PodcastProvider.h"
 #include "sql/SqlPodcastProvider.h"
+#include "sql/SqlUserPlaylistProvider.h"
 #include "Debug.h"
 #include "MainWindow.h"
 #include "meta/M3UPlaylist.h"
 #include "meta/PLSPlaylist.h"
 #include "meta/XSPFPlaylist.h"
-#include "meta/SqlPlaylist.h"
+// #include "meta/SqlPlaylist.h"
 #include "browsers/playlistbrowser/UserPlaylistModel.h"
-#include "browsers/playlistbrowser/SqlPlaylistGroup.h"
+// #include "browsers/playlistbrowser/SqlPlaylistGroup.h"
 
 #include <kdirlister.h>
 #include <kio/jobclasses.h>
@@ -71,11 +72,15 @@ PlaylistManager::PlaylistManager()
     m_defaultPodcastProvider = new SqlPodcastProvider();
     addProvider( m_defaultPodcastProvider, PlaylistManager::PodcastChannel );
     CollectionManager::instance()->addTrackProvider( m_defaultPodcastProvider );
+
+    m_defaultUserPlaylistProvider = new SqlUserPlaylistProvider();
+    addProvider( m_defaultUserPlaylistProvider, UserPlaylist );
 }
 
 PlaylistManager::~PlaylistManager()
 {
     delete m_defaultPodcastProvider;
+    delete m_defaultUserPlaylistProvider;
 }
 
 bool
@@ -155,6 +160,12 @@ PlaylistManager::playlistsOfCategory( int playlistCategory )
     return list;
 }
 
+PlaylistProviderList
+PlaylistManager::providersForCategory( int playlistCategory )
+{
+    return m_map.values( playlistCategory );
+}
+
 PlaylistProvider *
 PlaylistManager::playlistProvider(int category, QString name)
 {
@@ -228,27 +239,19 @@ PlaylistManager::typeName( int playlistCategory )
 bool
 PlaylistManager::save( Meta::TrackList tracks, const QString & name, bool editNow, const QString &fromLocation )
 {
-    Meta::SqlPlaylist playlist( name, tracks, SqlPlaylistGroupPtr(), fromLocation );
-    const int newId = playlist.id();
+    Q_UNUSED( name )
+    Q_UNUSED( fromLocation )
+    SqlUserPlaylistProvider *sqlProvider = dynamic_cast<SqlUserPlaylistProvider *>(m_defaultUserPlaylistProvider);
+    bool saveSuccessful = sqlProvider ? sqlProvider->save( tracks, name ) : false;
 
     if ( editNow )
     {
-        //jolt the playlist browser model to reload so the newly added item is shown
-        //talk about over-coupling... :|
-        // That might be so, but it is needed when manually saving a list, otherwise
-        // the user will not know that anything has happend and will end up with
-        // 20 entries just named "playlist" in the view
-        PlaylistBrowserNS::UserModel::instance()->reloadFromDb();
-
-        // NOTE this doesn't really make sense, especially when batch adding
-        //      during a collection scan ---lfranchi 9/5/08
-
         The::mainWindow()->showBrowser( "PlaylistBrowser" );
         emit( showCategory( UserPlaylist - 1 ) );
-        PlaylistBrowserNS::UserModel::instance()->editPlaylist( newId );
+//         PlaylistBrowserNS::UserModel::instance()->editPlaylist( newId );
     }
 
-    return true; //FIXME what's this supposed to return?
+    return saveSuccessful;
 }
 
 bool
@@ -336,7 +339,7 @@ PlaylistManager::canExpand( Meta::TrackPtr track )
 {
     if( !track )
         return false;
-        
+
     return Meta::getFormat( track->uidUrl() ) != Meta::NotPlaylist;
 }
 
