@@ -26,10 +26,12 @@
 #include <QMultiMap>
 #include <QList>
 
+class KJob;
 class PlaylistManager;
 class PlaylistProvider;
+typedef QList<PlaylistProvider *> PlaylistProviderList;
 class PodcastProvider;
-class KJob;
+class UserPlaylistProvider;
 
 namespace The {
     AMAROK_EXPORT PlaylistManager* playlistManager();
@@ -92,6 +94,11 @@ class PlaylistManager : public QObject
          */
         Meta::PlaylistList playlistsOfCategory( int playlistCategory );
 
+        /**
+        * returns all PlaylistProviders that provider a certain playlist category.
+        **/
+        PlaylistProviderList providersForCategory( int playlistCategory );
+
         /**
          * Add a PlaylistProvider that contains Playlists of a category defined
          * in the PlaylistCategory enum.
@@ -126,14 +133,13 @@ class PlaylistManager : public QObject
 
         bool exportPlaylist( Meta::TrackList tracks, const QString &location );
 
-
         //the next two functions are needed to support some services that have no other way of presenting data to the user
         //than wrapping the url to a playlist in a track.
-
         bool canExpand( Meta::TrackPtr track );
         Meta::PlaylistPtr expand( Meta::TrackPtr track );
 
         PodcastProvider *defaultPodcasts() { return m_defaultPodcastProvider; };
+        UserPlaylistProvider *defaultUserPlaylists() { return m_defaultUserPlaylistProvider; };
 
     signals:
         void updated();
@@ -150,6 +156,7 @@ class PlaylistManager : public QObject
         ~PlaylistManager();
 
         PodcastProvider *m_defaultPodcastProvider;
+        UserPlaylistProvider *m_defaultUserPlaylistProvider;
 
         QMultiMap<int, PlaylistProvider*> m_map; //Map PlaylistCategories to providers
         QMap<int, QString> m_customCategories;
@@ -177,10 +184,6 @@ class AMAROK_EXPORT PlaylistProvider : public QObject, public Amarok::Plugin
 
         virtual Meta::PlaylistList playlists() = 0;
 
-        virtual bool canSavePlaylists() { return false; };
-
-        virtual void save( Meta::PlaylistPtr playlist ) { Q_UNUSED( playlist ) };
-
     signals:
         virtual void updated();
 
