@@ -25,7 +25,7 @@
 #include <KLocale>
 #include <KUrl>
 
-#include <QDate>
+#include <QDateTime>
 #include <QSharedData>
 #include <QString>
 #include <QStringList>
@@ -95,12 +95,28 @@ class PodcastEpisode : public PodcastMetaCommon, public Track
         PodcastEpisode()
             : PodcastMetaCommon()
             , Track()
+            , m_channel( 0 )
+            , m_guid()
+            , m_mimeType()
+            , m_pubDate()
+            , m_duration( 0 )
+            , m_fileSize( 0 )
+            , m_sequenceNumber( 0 )
+            , m_isNew( true )
         {}
 
         PodcastEpisode( PodcastChannelPtr channel )
             : PodcastMetaCommon()
             , Track()
-            , m_channel( channel ) {}
+            , m_channel( channel )
+            , m_guid()
+            , m_mimeType()
+            , m_pubDate()
+            , m_duration( 0 )
+            , m_fileSize( 0 )
+            , m_sequenceNumber( 0 )
+            , m_isNew( true )
+        {}
 
         virtual ~PodcastEpisode() {}
 
@@ -163,12 +179,12 @@ class PodcastEpisode : public PodcastMetaCommon, public Track
         //PodcastEpisode methods
         KUrl localUrl() const { return m_localUrl; }
         void setLocalUrl( const KUrl &url ) { m_localUrl = url; }
-        QString pubDate() const { return m_pubDate; }
+        QDateTime pubDate() const { return m_pubDate; }
         int duration() const { return m_duration; }
         QString guid() const { return m_guid; }
 
         void setUidUrl( const KUrl &url ) { m_url = url; }
-        void setPubDate( const QString &pubDate ) { m_pubDate = pubDate; }
+        void setPubDate( const QDateTime &pubDate ) { m_pubDate = pubDate; }
         void setDuration( int duration ) { m_duration = duration; }
         void setGuid( const QString &guid ) { m_guid = guid; }
 
@@ -185,8 +201,7 @@ class PodcastEpisode : public PodcastMetaCommon, public Track
         KUrl m_url; //remote url of the file
         KUrl m_localUrl; //the localUrl, only valid if downloaded
         QString m_mimeType; //the mimetype of the enclosure
-        //TODO: convert to QDateTime from a RFC822 format
-        QString m_pubDate; //the pubDate from the feed
+        QDateTime m_pubDate; //the pubDate from the feed
         int m_duration; //the playlength in seconds
         int m_fileSize; //the size tag from the enclosure
         int m_sequenceNumber; //number of the episode
@@ -205,7 +220,15 @@ class PodcastChannel : public PodcastMetaCommon, public Playlist
 
         PodcastChannel()
             : PodcastMetaCommon()
-            , Playlist() { }
+            , Playlist()
+            , m_image()
+            , m_subscribeDate()
+            , m_copyright()
+            , m_autoScan( false )
+            , m_fetchType( DownloadWhenAvailable )
+            , m_purge( false )
+            , m_purgeCount( 0 )
+        { }
 
         virtual ~PodcastChannel() {}
 
@@ -219,6 +242,8 @@ class PodcastChannel : public PodcastMetaCommon, public Playlist
 
         //PodcastMetaCommon methods
         int podcastType() { return ChannelType; }
+        //override this since it's ambigous in PodcastMetaCommon and Playlist
+        virtual QString description() const { return m_description; }
 
         //PodcastChannel specific methods
         KUrl url() const { return m_url; }
