@@ -18,8 +18,10 @@
 
 #include "SqlPodcastMeta.h"
 
+#include "amarokurls/BookmarkMetaActions.h"
 #include "CollectionManager.h"
 #include "Debug.h"
+#include "meta/capabilities/CurrentTrackActionsCapability.h"
 #include "SqlPodcastProvider.h"
 #include "SqlStorage.h"
 
@@ -76,6 +78,45 @@ Meta::SqlPodcastEpisode::~SqlPodcastEpisode()
 {
 }
 
+bool
+Meta::SqlPodcastEpisode::hasCapabilityInterface( Meta::Capability::Type type ) const
+{
+    switch( type )
+    {
+        //TODO: for download, delete, etc
+        //case Meta::Capability::CustomActions:
+        case Meta::Capability::CurrentTrackActions:
+            //only downloaded episodes can be position marked
+            return !localUrl().isEmpty();
+
+            //TODO: downloaded episodes can be edited
+//         case Meta::Capability::Editable:
+//             return isEditable();
+
+        default:
+            return false;
+    }
+}
+
+Meta::Capability*
+Meta::SqlPodcastEpisode::asCapabilityInterface( Meta::Capability::Type type )
+{
+    switch( type )
+    {
+        case Meta::Capability::CurrentTrackActions:
+        {
+            QList< PopupDropperAction * > actions;
+            PopupDropperAction* flag = new BookmarkCurrentTrackPositionAction( 0 );
+            actions << flag;
+            debug() << "returning bookmarkcurrenttrack action";
+            return new Meta::CurrentTrackActionsCapability( actions );
+        }
+
+        default:
+            return 0;
+    }
+}
+
 void
 Meta::SqlPodcastEpisode::updateInDb()
 {
@@ -185,6 +226,16 @@ Meta::SqlPodcastChannel::~SqlPodcastChannel()
     m_episodes.clear();
 }
 
+Meta::TrackList
+Meta::SqlPodcastChannel::sqlEpisodesToTracks( Meta::SqlPodcastEpisodeList episodes )
+{
+    Meta::TrackList tracks;
+    foreach( Meta::SqlPodcastEpisodePtr sqlEpisode, episodes )
+        tracks << Meta::TrackPtr::dynamicCast( sqlEpisode );
+
+    return tracks;
+}
+
 void
 Meta::SqlPodcastChannel::addEpisode( PodcastEpisodePtr episode )
 {
@@ -292,4 +343,3 @@ Meta::SqlPodcastChannel::loadEpisodes()
     }
 }
 
-#include "SqlPodcastMeta.moc"
