@@ -24,24 +24,27 @@
 #include "playlist/PlaylistActions.h"
 #include "playlist/PlaylistController.h"
 #include "playlist/PlaylistModel.h"
+#include "playlist/view/listview/PrettyListView.h"
+#include "playlist/PlaylistWidget.h"
 
+#include <QObject>
 
 namespace AmarokScript
 {
     AmarokPlaylistScript::AmarokPlaylistScript( QScriptEngine* scriptEngine, QList<QObject*>* wrapperList )
-    : QObject( kapp )
-    , m_wrapperList( wrapperList )
-    , m_scriptEngine( scriptEngine )
+        : QObject( kapp )
+        , m_wrapperList( wrapperList )
+        , m_scriptEngine( scriptEngine )
     {
         m_wrapperList = wrapperList;
-        connect( The::playlistModel(), SIGNAL( rowsInserted( const QModelIndex&, int, int ) ), this, SLOT ( slotCountChanged( ) ) );
-        connect( The::playlistModel(), SIGNAL( rowsRemoved( const QModelIndex&, int, int ) ), this, SLOT ( slotCountChanged( ) ) );
+        connect( The::playlistModel(), SIGNAL( rowsInserted( const QModelIndex&, int, int ) ), this, SLOT ( slotCountChanged() ) );
+        connect( The::playlistModel(), SIGNAL( rowsRemoved( const QModelIndex&, int, int ) ), this, SLOT ( slotCountChanged() ) );
         connect( The::playlistModel(), SIGNAL( activeRowChanged( int ) ), this, SIGNAL( activeRowChanged( int ) ) );
     }
 
     AmarokPlaylistScript::~AmarokPlaylistScript()
-    {
-    }
+    {}
+
     int AmarokPlaylistScript::activeIndex()
     {
         return The::playlistModel()->activeRow();
@@ -120,7 +123,7 @@ namespace AmarokScript
     QStringList AmarokPlaylistScript::filenames()
     {
         QStringList fileNames;
-        for (int i=0; i < The::playlistModel()->rowCount(); i++)
+        for( int i=0; i < The::playlistModel()->rowCount(); i++ )
             fileNames << The::playlistModel()->trackAt(i)->prettyUrl();
         return fileNames;
     }
@@ -132,9 +135,30 @@ namespace AmarokScript
         return QVariant::fromValue( track );;
     }
 
-    void AmarokPlaylistScript::slotCountChanged( )
+    void AmarokPlaylistScript::slotCountChanged()
     {
-        emit CountChanged ( The::playlistModel()->rowCount() );
+        emit CountChanged( The::playlistModel()->rowCount() );
+    }
+	
+    QList<int> AmarokPlaylistScript::selectedIndexes()
+    {
+        DEBUG_BLOCK
+
+        Playlist::PrettyListView* list = qobject_cast<Playlist::PrettyListView*>( The::mainWindow()->playlistWidget()->currentView() );
+        return list->selectedRows();
+    }
+
+    QStringList AmarokPlaylistScript::selectedFilenames()
+    {
+        DEBUG_BLOCK
+
+        QStringList fileNames;
+        const QList<int> indexes = selectedIndexes();
+
+        for( int i=0; i < indexes.size(); i++ )
+            fileNames << The::playlistModel()->trackAt( indexes[i] )->prettyUrl();
+
+        return fileNames;
     }
 }
 
