@@ -28,13 +28,12 @@
 #include "ServiceInfoProxy.h"
 
 #include <khbox.h>
+#include <KLineEdit>
 #include <KMenuBar>
 
 #include <QFrame>
 #include <QLabel>
-#include <QPainter>
 
-#include <QDirModel>
 
 ServiceFactory::ServiceFactory() : m_initialized( false )
 {
@@ -59,7 +58,7 @@ ServiceFactory::trackForUrl(const KUrl & url)
     /*Meta::ServiceTrack * serviceTrack = new Meta::ServiceTrack();
     serviceTrack->setUidUrl( url.url() );
     Meta::TrackPtr track( serviceTrack );*/
-    
+
     Meta::TrackPtr track;
     foreach( ServiceBase * service, m_activeServices )
     {
@@ -73,7 +72,7 @@ ServiceFactory::trackForUrl(const KUrl & url)
             debug() << "Service Ready. Collection is: " << service->collection();
             track = service->collection()->trackForUrl( url );
         }
-        
+
         if ( track )
             return track;
     }
@@ -99,16 +98,20 @@ void ServiceFactory::slotServiceReady()
 
 ServiceBase *ServiceBase::s_instance = 0;
 
-ServiceBase::ServiceBase( const QString &name, ServiceFactory *parent )
-        : KVBox( 0)
+ServiceBase::ServiceBase( const QString &name, ServiceFactory *parent, bool useCollectionTreeView, const QString &prettyName )
+        : KVBox( 0 )
+        , m_contentView ( 0 )
         , m_parentFactory( parent )
         , m_polished( false )
         , m_serviceready( false )
+        , m_useCollectionTreeView( useCollectionTreeView )
         , m_infoParser( 0 )
+        , m_model( 0 )
 {
     DEBUG_BLOCK
 
     m_name = name;
+    m_prettyName = prettyName;
 
     setContentsMargins( 1, 1, 1, 1 );
     setSpacing( 1 );
@@ -118,7 +121,7 @@ ServiceBase::ServiceBase( const QString &name, ServiceFactory *parent )
     m_topPanel->setLineWidth( 2 );
     m_topPanel->setSpacing( 2 );
     m_topPanel->setMargin( 2 );
-    
+
     //m_topPanel->setFixedHeight( 50 );
 
     KHBox * commonPanel = new KHBox ( m_topPanel );
@@ -131,31 +134,34 @@ ServiceBase::ServiceBase( const QString &name, ServiceFactory *parent )
 
     QLabel * nameLabel = new QLabel( commonPanel );
     nameLabel->setMinimumSize( 230 , 28 );
-    nameLabel->setText( m_name );
+    if( !m_prettyName.isEmpty() )
+        nameLabel->setText( m_prettyName );
+    else
+        nameLabel->setText( m_name );
+
     QFont nameLabelFont = nameLabel->font();
     nameLabelFont.setBold( true );
     nameLabelFont.setPointSize( nameLabelFont.pointSize() + 2 );
     nameLabel->setFont( nameLabelFont );
     nameLabel->setAlignment(Qt::AlignCenter | Qt::AlignHCenter);
 
-    m_contentView = new ServiceCollectionTreeView( this );
-    m_contentView->setFrameShape( QFrame::NoFrame );
-
+    if( useCollectionTreeView ) {
+        m_contentView = new ServiceCollectionTreeView( this );
+        m_contentView->setFrameShape( QFrame::NoFrame );
+        m_contentView->setSortingEnabled( true );
+        m_contentView->sortByColumn ( 0, Qt::AscendingOrder );
+        m_contentView->setDragEnabled ( true );
+        m_contentView->setDragDropMode ( QAbstractItemView::DragOnly );
+        connect( m_contentView, SIGNAL( itemSelected ( CollectionTreeItem * )  ), this, SLOT( itemSelected( CollectionTreeItem * ) ) );
+    }
     //m_contentView->setAlternatingRowColors ( true );
     //m_contentView->setAnimated( true );
-    m_contentView->setSortingEnabled( true );
-    m_contentView->sortByColumn ( 0, Qt::AscendingOrder );
 
-    m_contentView->setDragEnabled ( true );
-    m_contentView->setDragDropMode ( QAbstractItemView::DragOnly );
 
 
     //connect( m_contentView, SIGNAL( pressed ( const QModelIndex & ) ), this, SLOT( treeItemSelected( const QModelIndex & ) ) );
     //connect( m_contentView, SIGNAL( doubleClicked ( const QModelIndex & ) ), this, SLOT( itemActivated ( const QModelIndex & ) ) );
 
-    connect( m_contentView, SIGNAL( itemSelected ( CollectionTreeItem * )  ), this, SLOT( itemSelected( CollectionTreeItem * ) ) );
-
-
     m_bottomPanel = new KVBox( this );
     m_bottomPanel->setFixedHeight( 50 );
     m_bottomPanel->setFrameStyle( QFrame::NoFrame );
@@ -174,7 +180,8 @@ ServiceBase::ServiceBase( const QString &name, ServiceFactory *parent )
     m_menubar->hide();
 
     m_searchWidget = new SearchWidget( m_topPanel );
-    m_searchWidget->setup( m_contentView );
+    if ( m_contentView )
+        m_searchWidget->setup( m_contentView );
 
     //setFrameShape( QFrame::StyledPanel );
     //setFrameShadow( QFrame::Sunken );
@@ -186,11 +193,19 @@ ServiceBase::~ServiceBase()
 }
 
 QString
-ServiceBase::name( ) const
+ServiceBase::name() const
 {
     return m_name;
 }
 
+QString
+ServiceBase::prettyName() const
+{
+    if( !m_prettyName.isEmpty() )
+      return m_prettyName;
+    return m_name;
+}
+
 ServiceFactory*
 ServiceBase::parent() const
 {
@@ -216,7 +231,7 @@ ServiceBase::setLongDescription( const QString &longDescription )
 }
 
 QString
-ServiceBase::longDescription( ) const
+ServiceBase::longDescription() const
 {
     return m_longDescription;
 }
@@ -224,11 +239,12 @@ ServiceBase::longDescription( ) const
 void
 ServiceBase::setIcon( const QIcon &icon )
 {
+    DEBUG_BLOCK
     m_icon = icon;
 }
 
 QIcon
-ServiceBase::icon( ) const
+ServiceBase::icon() const
 {
     return m_icon;
 }
@@ -247,7 +263,7 @@ ServiceBase::itemActivated ( const QModelIndex & index )
 
 
 void
-ServiceBase::setModel( SingleCollectionTreeItemModel * model )
+ServiceBase::setModel( QAbstractItemModel * model )
 {
     //m_filterModel->setSourceModel( model );
     //m_contentView->setModel( m_filterModel );
@@ -255,12 +271,28 @@ ServiceBase::setModel( SingleCollectionTreeItemModel * model )
     m_model  = model;
 }
 
-SingleCollectionTreeItemModel *
+QAbstractItemModel *
 ServiceBase::model()
 {
     return m_model;
 }
 
+QTreeView *
+ServiceBase::view()
+{
+    return m_contentView;
+}
+
+void
+ServiceBase::setView( QTreeView * view )
+{
+    if( !view)
+        return;
+    m_contentView = view;
+    if( m_model )
+        m_contentView->setModel( m_model );
+}
+
 bool
 ServiceBase::serviceReady() const
 {
@@ -272,7 +304,6 @@ void
 ServiceBase::infoChanged( const QString &infoHtml )
 {
     DEBUG_BLOCK
-
     QVariantMap map;
     map["service_name"] = m_name;
     map["main_info"] = infoHtml;
@@ -307,37 +338,55 @@ ServiceBase::generateWidgetInfo( const QString &html ) const
 void
 ServiceBase::setPlayableTracks(bool playable)
 {
-    m_contentView->setPlayableTracks( playable );
+    if( m_useCollectionTreeView ) {
+        if( ServiceCollectionTreeView* view = dynamic_cast<ServiceCollectionTreeView*>(m_contentView) )
+            view->setPlayableTracks( playable );
+    }
 }
 
 void
 ServiceBase::sortByArtist()
 {
-    m_contentView->setLevels( QList<int>() << CategoryId::Artist );
+    if( m_useCollectionTreeView ) {
+        if( ServiceCollectionTreeView* view = dynamic_cast<ServiceCollectionTreeView*>(m_contentView) )
+            view->setLevels( QList<int>() << CategoryId::Artist );
+    }
 }
 
 void
 ServiceBase::sortByArtistAlbum()
 {
-    m_contentView->setLevels( QList<int>() << CategoryId::Artist << CategoryId::Album );
+    if( m_useCollectionTreeView ) {
+        if( ServiceCollectionTreeView* view = dynamic_cast<ServiceCollectionTreeView*>(m_contentView) )
+            view->setLevels( QList<int>() << CategoryId::Artist << CategoryId::Album );
+    }
 }
 
 void
 ServiceBase::sortByAlbum()
 {
-    m_contentView->setLevels( QList<int>() << CategoryId::Album );
+    if( m_useCollectionTreeView ) {
+        if( ServiceCollectionTreeView* view = dynamic_cast<ServiceCollectionTreeView*>(m_contentView) )
+            view->setLevels( QList<int>() << CategoryId::Album );
+    }
 }
 
 void
 ServiceBase::sortByGenreArtist()
 {
-    m_contentView->setLevels( QList<int>() << CategoryId::Genre << CategoryId::Artist );
+    if( m_useCollectionTreeView ) {
+        if( ServiceCollectionTreeView* view = dynamic_cast<ServiceCollectionTreeView*>(m_contentView) )
+            view->setLevels( QList<int>() << CategoryId::Genre << CategoryId::Artist );
+    }
 }
 
 void
 ServiceBase::sortByGenreArtistAlbum()
 {
-    m_contentView->setLevels( QList<int>() << CategoryId::Genre << CategoryId::Artist << CategoryId::Album );
+    if( m_useCollectionTreeView ) {
+        if( ServiceCollectionTreeView* view = dynamic_cast<ServiceCollectionTreeView*>(m_contentView) )
+            view->setLevels( QList<int>() << CategoryId::Genre << CategoryId::Artist << CategoryId::Album );
+    }
 }
 
 void
@@ -374,6 +423,23 @@ ServiceBase::sendMessage( const QString & message )
     return i18n( "ERROR: unknown message" );
 }
 
+QString ServiceBase::filter()
+{
+    return m_searchWidget->lineEdit()->text();
+}
+
+QList<int> ServiceBase::levels()
+{
+    CollectionTreeView *contentView = qobject_cast<CollectionTreeView*>(m_contentView);
+    if( contentView != 0 )
+        return contentView->levels();
+    return QList<int>();
+}
+
+
+
 
 #include "ServiceBase.moc"
 
+
+
