@@ -23,6 +23,7 @@
 #include "AmpacheConfig.h"
 #include "collection/CollectionManager.h"
 #include "Debug.h"
+#include "sha256/sha256.h"
 #include "statusbar/StatusBar.h"
 
 
@@ -30,10 +31,31 @@
 #include <kpassworddialog.h>
 #include <KMD5>
 
+
 #include <QDomDocument>
 
 AMAROK_EXPORT_PLUGIN( AmpacheServiceFactory )
 
+QString sha256( QString in )
+{
+    unsigned char digest[ SHA512_DIGEST_SIZE];
+    unsigned char* toHash = (unsigned char*)in.toUtf8().data();
+    
+    sha256( toHash , qstrlen( ( char* )toHash ), digest );
+
+    // this part copied from main() in sha256.cpp
+    unsigned char output[2 * SHA512_DIGEST_SIZE + 1];
+    int i;
+
+    output[2 * SHA256_DIGEST_SIZE ] = '\0';
+
+    for (i = 0; i < SHA256_DIGEST_SIZE ; i++) {
+       sprintf((char *) output + 2*i, "%02x", digest[i]);
+    }
+    
+    return QString::fromAscii( (const char*)output );
+}
+
 void AmpacheServiceFactory::init()
 {
     //read config and create the needed number of services
@@ -93,7 +115,10 @@ AmpacheService::AmpacheService( AmpacheServiceFactory* parent, const QString & n
     , m_sessionId ( QString() )
     , m_collection( 0 )
 {
-    setShortDescription( i18n( "Use Amarok as a seamless frontend to your Ampache server!" ) );
+    DEBUG_BLOCK
+
+
+    setShortDescription( i18n( "Use Amarok as a seamless frontend to your Ampache server." ) );
     setIcon( KIcon( "view-services-ampache-amarok" ) );
 
     //we are using http queries later on, so we require
@@ -111,11 +136,22 @@ AmpacheService::AmpacheService( AmpacheServiceFactory* parent, const QString & n
         kurl.setProtocol( "http" );
         kurl.setAuthority( url );
     }
+
     m_server = kurl.url();
+
+    // We need to check the version of Ampache we are attempting to authenticate against, as this changes how we deal with it
+
+    QString versionString = "<server>/server/xml.server.php?action=ping";
+
+    versionString.replace(QString("<server>"), m_server);
+
+    debug() << "Verifying Ampache Version Using: " << versionString;
+
     m_username = username;
     m_password = password;
 
-    authenticate();
+    m_xmlVersionJob = KIO::storedGet( versionString, KIO::Reload, KIO::HideProgressInfo );
+    connect( m_xmlVersionJob, SIGNAL(result(KJob *)), this, SLOT( authenticate(KJob *) ) );
 }
 
 AmpacheService::~AmpacheService()
@@ -134,10 +170,12 @@ AmpacheService::polish()
 }
 
 void
-AmpacheService::authenticate()
+AmpacheService::authenticate(KJob * job)
 {
     DEBUG_BLOCK
 
+    versionVerify(job); 
+
     //lets keep this around for now if we want to allow pwople to add a service that prompts for stuff
     if ( m_server.isEmpty() || m_password.isEmpty() )
     {
@@ -166,12 +204,28 @@ AmpacheService::authenticate()
 
     QString timestamp = QString::number( QDateTime::currentDateTime().toTime_t() );
 
-    QString rawHandshake = timestamp + m_password;
-    KMD5 context( rawHandshake.toUtf8() );
+    QString rawHandshake;
+    QString authenticationString;
+    QString passPhrase;
 
-    QString passPhrase = context.hexDigest().data();
+    // We need to use different authentication strings depending on the version of ampache
+    if ( m_version > 350000 )
+    {
+	debug() << "New Password Scheme " << m_version; 
+        authenticationString = "<server>/server/xml.server.php?action=handshake<username>&auth=<passphrase>&timestamp=<timestamp>&version=350001";
 
-    QString authenticationString = "<server>/server/xml.server.php?action=handshake<username>&auth=<passphrase>&timestamp=<timestamp>";
+        rawHandshake = timestamp + sha256( m_password );
+
+        passPhrase = sha256( rawHandshake );
+	debug() << "Version Greater then 35001 Generating new SHA256 Auth" << authenticationString << passPhrase;
+    }
+    else { 
+        debug() << "Version Older then 35001 Generated MD5 Auth " << m_version;
+        authenticationString = "<server>/server/xml.server.php?action=handshake<username>&auth=<passphrase>&timestamp=<timestamp>";
+        rawHandshake = timestamp + m_password;
+        KMD5 context( rawHandshake.toUtf8() );
+        passPhrase = context.hexDigest().data();
+    }
 
     authenticationString.replace(QString("<server>"), m_server);
     if ( !m_username.isEmpty() )
@@ -181,15 +235,11 @@ AmpacheService::authenticate()
     authenticationString.replace(QString("<passphrase>"), passPhrase);
     authenticationString.replace(QString("<timestamp>"), timestamp);
 
-    debug() << "Authenticating with string: " << authenticationString;
+    debug() << "Authenticating with string: " << authenticationString << passPhrase;
 
     m_xmlDownloadJob = KIO::storedGet( authenticationString, KIO::NoReload, KIO::HideProgressInfo );
-    /*connect( m_xmlDownloadJob, SIGNAL(result(KJob *)), this, SLOT( authenticationComplete( KJob*) ) );
-    The::statusBar() ->newProgressOperation( m_xmlDownloadJob )
-    .setDescription( i18n( "Authenticating" ) );*/
-
-    if ( m_xmlDownloadJob->exec() )
-        authenticationComplete( m_xmlDownloadJob );
+    connect( m_xmlDownloadJob, SIGNAL(result(KJob *)), this, SLOT( authenticationComplete( KJob*) ) );
+    The::statusBar()->newProgressOperation( m_xmlDownloadJob, i18n( "Authenticating with Ampache" ) );
 }
 
 void AmpacheService::authenticationComplete(KJob * job)
@@ -217,7 +267,7 @@ void AmpacheService::authenticationComplete(KJob * job)
 
     if ( !error.isNull() )
     {
-        KMessageBox::error ( this, error.text(), i18n( "Authentication Error!" ) );
+        KMessageBox::error ( this, error.text(), i18n( "Authentication Error" ) );
     }
     else
     {
@@ -243,5 +293,49 @@ void AmpacheService::authenticationComplete(KJob * job)
 
 }
 
+void AmpacheService::versionVerify(KJob * job)
+{
+
+    DEBUG_BLOCK
+
+    if( !job->error() == 0 )
+    {
+	debug() << "Job Error" << job->error();
+	// If an error has occured, it's non-fatal unless they are using 3.5, as we default to 3.4 currently
+        return;
+    }
+    QString xmlReply = ((KIO::StoredTransferJob* )job)->data();
+    debug() << "Version Verify reply: " << xmlReply;
+
+    //so lets figure out what we got here:
+    QDomDocument doc( "version" );
+
+    doc.setContent(m_xmlVersionJob->data() );
+    QDomElement root = doc.firstChildElement("root");
+    //is this an error?
+
+    QDomElement error = root.firstChildElement("error");
+
+    // It's ok if we get a null response from the version, that just means we're dealing with an older version
+    if ( !error.isNull() )
+    {
+	// Default the Version down if it didn't work 
+	m_version = 100000;
+	debug() << "AmpacheService::versionVerify Error: " << error.text();
+    }
+    else
+    {
+        //find status code:
+        QDomElement element = root.firstChildElement("version");
+
+        m_version = element.text().toInt();
+
+        debug() << "versionVerify Returned: " << m_version;
+
+        job->deleteLater();
+    }
+
+}
+
 #include "AmpacheService.moc"
 
