@@ -17,6 +17,8 @@
 #include "ui_LastFmConfigWidget.h"
 #include "Debug.h"
 
+#include "kdenetwork/knetworkaccessmanager.h"
+
 #include <lastfm/Scrobbler.h> // from liblastfm
 #include <lastfm/ws/WsKeys.h>
 #include <lastfm/ws/WsReply.h>
@@ -88,20 +90,21 @@ LastFmServiceSettings::testLogin()
     Ws::Username = qstrdup( m_configDialog->kcfg_ScrobblerUsername->text().toLatin1().data() );
     
     // set up proxy
-    //WsAccessManager* qnam = new KNetworkAccessManager( this );
-    //WsRequestBuilder::setWAM( qnam );
+    // NOTE yes we instantiate two KNAMs here, one in this kcm module and one in the servce itself.
+    // but there is no way to share the class easily across the lib boundary as they are not guaranteed to
+    // always exist at the same time... so 1 class seems to be a relatively minor penalty for a working Test button
+    WsAccessManager* qnam = new KNetworkAccessManager( this );
+    WsRequestBuilder::setWAM( qnam );
     
     debug() << "username:" << QString( QUrl::toPercentEncoding( Ws::Username ) );
 
     QString authToken =  md5( ( m_configDialog->kcfg_ScrobblerUsername->text() + md5( m_configDialog->kcfg_ScrobblerPassword->text().toUtf8() ) ).toUtf8() );
-    QString sign_key = md5( ( "api_key" + QString( Ws::ApiKey ) + "authToken" + authToken + "methodauth.getMobileSession" + QString( Ws::SharedSecret ) ).toUtf8() );
     
     // now authenticate w/ last.fm and get our session key
     WsReply* reply = WsRequestBuilder( "auth.getMobileSession" )
     .add( "username", m_configDialog->kcfg_ScrobblerUsername->text() )
     .add( "authToken", authToken )
     .add( "api_key", Ws::ApiKey )
-    .add( "api_sig", sign_key )
     .get();
     
     connect( reply, SIGNAL( finished( WsReply* ) ), SLOT( onAuthenticated( WsReply* ) ) );
