@@ -54,7 +54,8 @@ OSDWidget::OSDWidget( QWidget *parent, const char *name )
         , m_y( MARGIN )
         , m_drawShadow( true )
         , m_rating( 0 )
-        , m_volume( false )
+        , m_volume( The::engineController()->volume() )
+        , m_showVolume( false )
 {
     Qt::WindowFlags flags;
     flags = Qt::WindowStaysOnTopHint | Qt::FramelessWindowHint;
@@ -87,7 +88,7 @@ void
 OSDWidget::show( const QString &text, QImage newImage )
 {
     DEBUG_BLOCK
-    m_volume = false;
+    m_showVolume = false;
     if ( !newImage.isNull() )
     {
         m_cover = newImage;
@@ -122,13 +123,15 @@ OSDWidget::ratingChanged( const short rating )
 }
 
 void
-OSDWidget::volChanged( int volume )
+OSDWidget::volumeChanged( int volume )
 {
     if ( isEnabled() )
     {
-        m_volume = true;
-        m_newvolume = volume;
-        m_text = m_newvolume ? i18n("Volume: %1%", m_newvolume) : i18nc( "State, as in, The playback is silent", "Mute" );
+        QString muteState = "";
+        m_showVolume = true;
+        m_volume = volume;
+
+        m_text = i18n("Volume: %1% %2", m_volume, ( The::engineController()->isMuted() ? "(muted)" : "" ) );
 
         show();
     }
@@ -182,11 +185,10 @@ OSDWidget::determineMetrics( const int M )
             Qt::AlignCenter | Qt::TextWordWrap, m_text );
     rect.setHeight( rect.height() + M + M );
 
-    if( m_volume )
+    if( m_showVolume )
     {
         static const QString tmp = QString ("******").insert( 3,
-            ( i18n("Volume: 100%").length() >= i18nc( "State, as in, the playback is silent", "Mute" ).length() )?
-            i18n("Volume: 100%") : i18nc( "State, as in, the playback is silent", "Mute" ) );
+            ( i18n("Volume: 100% (muted)" ) ) );
 
         QRect tmpRect = fontMetrics().boundingRect( 0, 0,
             max.width() - image.width(), max.height() - fontMetrics().height(),
@@ -195,14 +197,14 @@ OSDWidget::determineMetrics( const int M )
 
         rect = tmpRect;
 
-        if( m_newvolume > 66 )
-            m_cover = KIcon( "audio-volume-high-amarok" ).pixmap( 100, 100 ).toImage();
-        else if ( m_newvolume > 33 )
-            m_cover = KIcon( "audio-volume-medium-amarok" ).pixmap( 100, 100 ).toImage();
-        else if ( m_newvolume > 0 )
-            m_cover = KIcon( "audio-volume-low-amarok" ).pixmap( 100, 100 ).toImage();
-        else
+        if ( The::engineController()->isMuted() )
             m_cover = KIcon( "audio-volume-muted-amarok" ).pixmap( 100, 100 ).toImage();
+        else if( m_volume > 66 )
+            m_cover = KIcon( "audio-volume-high-amarok" ).pixmap( 100, 100 ).toImage();
+        else if ( m_volume > 33 )
+            m_cover = KIcon( "audio-volume-medium-amarok" ).pixmap( 100, 100 ).toImage();
+        else
+            m_cover = KIcon( "audio-volume-low-amarok" ).pixmap( 100, 100 ).toImage();
     }
     // Don't show both volume and rating
     else if( m_rating )
@@ -311,7 +313,7 @@ OSDWidget::paintEvent( QPaintEvent *e )
 
     int graphicsHeight = 0;
 
-    if( !m_volume && m_rating > 0 && !m_paused )
+    if( !m_showVolume && m_rating > 0 && !m_paused )
     {
         QPixmap* star = StarManager::instance()->getStar( m_rating/2 );
         QRect r( rect );
@@ -620,28 +622,15 @@ Amarok::OSD::forceToggleOSD()
 void
 Amarok::OSD::engineVolumeChanged( int newVolume )
 {
-    volChanged( newVolume );
+    volumeChanged( newVolume );
 }
 
-/* Try to detect MetaData spam in Streams. */
-bool
-Amarok::OSD::isMetaDataSpam( const QHash<qint64, QString> &newMetaData )
+void
+Amarok::OSD::engineMuteStateChanged( bool mute )
 {
-    // search for Metadata in history
-    for( int i = 0; i < m_metaDataHistory.size(); i++)
-    {
-        if( m_metaDataHistory.at( i ) == newMetaData ) // we already had that one -> spam!
-        {
-            m_metaDataHistory.move( i, 0 ); // move spam to the beginning of the list
-            return true;
-        }
-    }
+    Q_UNUSED( mute )
 
-    if( m_metaDataHistory.size() == 12 )
-        m_metaDataHistory.removeLast();
-
-    m_metaDataHistory.insert( 0, newMetaData );
-    return false;
+    volumeChanged( m_volume );
 }
 
 void
