@@ -1,5 +1,5 @@
 /***************************************************************************
- * copyright     : (C) 2007 Dan Meltzer <hydrogen@notyetimplemented.com>   *
+ * copyright     : (C) 2007 Dan Meltzer <parallelgrapefruit@gmail.com>   *
  **************************************************************************/
 
  /***************************************************************************
@@ -15,6 +15,7 @@
 
 #include <QVBoxLayout>
 
+#include <KIcon>
 #include <KLineEdit>
 #include <KLocale>
 #include <KHBox>
@@ -24,7 +25,7 @@
 SearchWidget::SearchWidget( QWidget *parent, bool advanced )
     : QWidget( parent )
     , m_sw( 0 )
-    , m_filterButton( 0 )
+    , m_filterAction( 0 )
 {
     init( parent, advanced );
 }
@@ -32,7 +33,7 @@ SearchWidget::SearchWidget( QWidget *parent, bool advanced )
 SearchWidget::SearchWidget( QWidget *parent, QWidget *caller, bool advanced )
     : QWidget( parent )
     , m_sw( 0 )
-    , m_filterButton( 0 )
+    , m_filterAction( 0 )
 {
     init( parent, advanced );
     setup( caller );
@@ -62,20 +63,20 @@ SearchWidget::init( QWidget *parent, bool advanced )
     m_sw->setSizePolicy( QSizePolicy::Expanding, QSizePolicy::Fixed );
     m_sw->setClearButtonShown( true );
     m_sw->setFrame( true );
-    m_sw->setToolTip( i18n( "Enter space-separated terms to search in the playlist." ) );
+    m_sw->setToolTip( i18n( "Enter space-separated terms to search." ) );
     QVBoxLayout *layout = new QVBoxLayout();
     layout->addWidget( searchBox );
     layout->setContentsMargins( 0, 0, 0, 0 );
     setLayout( layout );
 
-    if ( advanced ) {
-        m_filterButton = new KPushButton( i18n( "Advanced" ), searchBox );
-        //m_filterButton->setFlat( true ); //TODO: maybe?
-        m_filterButton->setObjectName( "filter" );
-        m_filterButton->setSizePolicy( QSizePolicy::Preferred, QSizePolicy::Fixed );
-        m_filterButton->setToolTip( i18n( "Click to edit playlist filter" ) );
+    m_toolBar = new QToolBar( searchBox );
 
-        connect ( m_filterButton, SIGNAL( clicked() ), this, SLOT( slotShowFilterEditor() ) );
+    if ( advanced ) {
+        m_filterAction = new QAction( KIcon( "document-properties" ), i18n( "Edit filter" ), this );
+        m_filterAction->setObjectName( "filter" );
+        m_toolBar->addAction( m_filterAction );
+
+        connect ( m_filterAction, SIGNAL( triggered() ), this, SLOT( slotShowFilterEditor() ) );
     }
 }
 
@@ -96,5 +97,35 @@ SearchWidget::slotShowFilterEditor()
     fd->exec();
 }
 
+QToolBar * SearchWidget::toolBar()
+{
+    return m_toolBar;
+}
+
+void SearchWidget::showAdvancedButton(bool show)
+{
+    if ( show ) {
+        if ( m_filterAction != 0 ) {
+            m_filterAction = new QAction( KIcon( "document-properties" ), i18n( "Edit filter" ), this );
+            m_filterAction->setObjectName( "filter" );
+            m_toolBar->addAction( m_filterAction );
+            connect ( m_filterAction, SIGNAL( triggered() ), this, SLOT( slotShowFilterEditor() ) );
+        }
+    }
+    else
+    {
+        delete m_filterAction;
+        m_filterAction = 0;
+    }
+}
+
+void SearchWidget::setClickMessage( QString message )
+{
+    m_sw->setClickMessage( message );
+}
+
+
 #include "SearchWidget.moc"
 
+
+
