@@ -1,5 +1,5 @@
 /***************************************************************************
- * copyright     : (C) 2007 Dan Meltzer <hydrogen@notyetimplemented.com>   *
+ * copyright     : (C) 2007 Dan Meltzer <parallelgrapefruit@gmail.com>   *
  **************************************************************************/
 
  /***************************************************************************
@@ -13,6 +13,9 @@
 #ifndef SEARCHWIDGET_H
 #define SEARCHWIDGET_H
 
+#include "amarok_export.h"
+
+#include <QToolBar>
 #include <QWidget>
 
 class KLineEdit;
@@ -20,7 +23,7 @@ class KPushButton;
 // A Custom Widget that can be used globally to implement
 // searching a treeview.
 
-class SearchWidget : public QWidget
+class AMAROK_EXPORT SearchWidget : public QWidget
 {
     Q_OBJECT
     public:
@@ -30,6 +33,16 @@ class SearchWidget : public QWidget
         void setup( QObject* caller );
         void setSearchString( const QString &searchString );
 
+        QToolBar * toolBar();
+
+        void showAdvancedButton( bool show );
+        
+        /**
+         * Sets the string that will be visible when the KLineEdit is empty.
+         * @param message the string that will be visible then the KLineEdit is empty.
+         */
+        void setClickMessage( QString message );
+
     signals:
 
         void filterNow();
@@ -42,7 +55,8 @@ class SearchWidget : public QWidget
         void init( QWidget *parent, bool advanced );
 
         KLineEdit   *m_sw;
-        KPushButton *m_filterButton;
+        QAction *m_filterAction;
+        QToolBar * m_toolBar;
 };
 
 #endif
