@@ -7,7 +7,7 @@
  copyright            : (C) 2005 by Gábor Lehel
  email                : illissius@gmail.com
  copyright            : (C) 2008 by Dan Meltzer
- email                : hydrogen@notyetimplemented.com
+ email                : parallelgrapefruit@gmail.com
 ***************************************************************************/
 
 /***************************************************************************
@@ -22,12 +22,14 @@
 #ifndef SLIDERWIDGET_H
 #define SLIDERWIDGET_H
 
+#include <QList>
 #include <QPixmap>
 #include <QSlider>
 #include <QVector>
 
 class QPalette;
 class QTimer;
+class BookmarkTriangle;
 
 namespace Amarok
 {
@@ -36,7 +38,7 @@ namespace Amarok
         Q_OBJECT
 
         public:
-            Slider( Qt::Orientation, QWidget*, uint max = 0 );
+            Slider( Qt::Orientation, uint max = 0, QWidget* parent = 0 );
 
             virtual void setValue( int );
 
@@ -59,8 +61,6 @@ namespace Amarok
             virtual void slideEvent( QMouseEvent* );
             virtual void resizeEvent( QResizeEvent * ) { m_needsResize = true; }
 
-            virtual void paletteChange( const QPalette & oldPalette );
-
             void paintCustomSlider( QPainter *p, int x, int y, int width, int height, double pos = -1.0 );
 
             bool m_sliding;
@@ -72,11 +72,17 @@ namespace Amarok
                return orientation() == Qt::Vertical ? mp - (v - mp) : v;
             }
 
+            static const int m_borderWidth = 6;
+            static const int m_borderHeight = 6;
+
+            static const int m_sliderInsertX = 5;
+            static const int m_sliderInsertY = 5;
+
         private:
             bool m_outside;
             int  m_prevValue;
             bool m_needsResize;
-            
+
             QPixmap m_topLeft;
             QPixmap m_topRight;
             QPixmap m_top;
@@ -94,35 +100,18 @@ namespace Amarok
         Q_OBJECT
 
         public:
-            explicit VolumeSlider( QWidget *parent, uint max = 0 );
+            explicit VolumeSlider( uint max, QWidget *parent = 0 );
 
         protected:
             virtual void paintEvent( QPaintEvent* );
 
-            virtual void paletteChange( const QPalette& );
-            virtual void slideEvent( QMouseEvent* );
             virtual void mousePressEvent( QMouseEvent* );
             virtual void contextMenuEvent( QContextMenuEvent* );
             virtual void wheelEvent( QWheelEvent *e );
             virtual void resizeEvent(QResizeEvent * event);
 
-        signals:
-            void mute();
-
         private:
             Q_DISABLE_COPY( VolumeSlider )
-
-            ////////////////////////////////////////////////////////////////
-            int m_iconHeight;
-            int m_iconWidth;
-            int m_textWidth;
-            int m_sliderWidth;
-            int m_sliderHeight;
-            int m_sliderX;
-
-            int m_margin;
-
-            int m_textHeight;
     };
 
     class TimeSlider : public Amarok::Slider
@@ -133,20 +122,22 @@ namespace Amarok
             TimeSlider( QWidget *parent );
 
             void setSliderValue( int value );
+            void drawTriangle( const QString &name, int x );
+            void clearTriangles();
+
+        public slots:
+            void slotTriangleClicked( int );
 
         protected:
             virtual void paintEvent( QPaintEvent* );
-
-            virtual void paletteChange( const QPalette& );
+            virtual void mousePressEvent( QMouseEvent* );
             virtual void resizeEvent(QResizeEvent * event);
 
         private:
             Q_DISABLE_COPY( TimeSlider )
-            ////////////////////////////////////////////////////////////////
 
             QTimer *m_animTimer; // Used for a smooth progress.
-
-            int m_sliderHeight;
+            QList<BookmarkTriangle*> m_triangles;
             double m_knobX; // The position of the current indicator.
     };
 }
